@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,10 +7,14 @@
 #include <config.h>
 #include <asiolink/asio_wrapper.h>
 #include <cc/command_interpreter.h>
+#include <config/command_mgr.h>
 #include <d2/d2_log.h>
 #include <d2/d2_cfg_mgr.h>
+#include <d2/d2_controller.h>
 #include <d2/d2_process.h>
 
+using namespace isc::process;
+
 namespace isc {
 namespace d2 {
 
@@ -20,7 +24,7 @@ const unsigned int D2Process::QUEUE_RESTART_PERCENT =  80;
 
 D2Process::D2Process(const char* name, const asiolink::IOServicePtr& io_service)
     : DProcessBase(name, io_service, DCfgMgrBasePtr(new D2CfgMgr())),
-     reconf_queue_flag_(false), shutdown_type_(SD_NORMAL) {
+      reconf_queue_flag_(false), shutdown_type_(SD_NORMAL) {
 
     // Instantiate queue manager.  Note that queue manager does not start
     // listening at this point.  That can only occur after configuration has
@@ -38,14 +42,21 @@ D2Process::D2Process(const char* name, const asiolink::IOServicePtr& io_service)
 
 void
 D2Process::init() {
+    // CommandMgr uses IO service to run asynchronous socket operations.
+    isc::config::CommandMgr::instance().setIOService(getIoService());
 };
 
 void
 D2Process::run() {
-    LOG_INFO(dctl_logger, DHCP_DDNS_STARTED).arg(VERSION);
-    // Loop forever until we are allowed to shutdown.
-    while (!canShutdown()) {
-        try {
+    LOG_INFO(d2_logger, DHCP_DDNS_STARTED).arg(VERSION);
+    D2ControllerPtr controller =
+        boost::dynamic_pointer_cast<D2Controller>(D2Controller::instance());
+    try {
+        // Now logging was initialized so commands can be registered.
+        controller->registerCommands();
+
+        // Loop forever until we are allowed to shutdown.
+        while (!canShutdown()) {
             // Check on the state of the request queue. Take any
             // actions necessary regarding it.
             checkQueueStatus();
@@ -59,7 +70,8 @@ D2Process::run() {
             //   a. NCR message has been received
             //   b. Transaction IO has completed
             //   c. Interval timer expired
-            //   d. Something stopped IO service (runIO returns 0)
+            //   d. Control channel event
+            //   e. Something stopped IO service (runIO returns 0)
             if (runIO() == 0) {
                 // Pretty sure this amounts to an unexpected stop and we
                 // should bail out now.  Normal shutdowns do not utilize
@@ -67,18 +79,21 @@ D2Process::run() {
                 isc_throw(DProcessBaseError,
                           "Primary IO service stopped unexpectedly");
             }
-        } catch (const std::exception& ex) {
-            LOG_FATAL(dctl_logger, DHCP_DDNS_FAILED).arg(ex.what());
-            isc_throw (DProcessBaseError,
-                       "Process run method failed: " << ex.what());
         }
+    } catch (const std::exception& ex) {
+        LOG_FATAL(d2_logger, DHCP_DDNS_FAILED).arg(ex.what());
+        controller->deregisterCommands();
+        isc_throw (DProcessBaseError,
+                   "Process run method failed: " << ex.what());
     }
 
     // @todo - if queue isn't empty, we may need to persist its contents
     // this might be the place to do it, once there is a persistence mgr.
     // This may also be better in checkQueueStatus.
 
-    LOG_DEBUG(dctl_logger, DBGLVL_START_SHUT, DHCP_DDNS_RUN_EXIT);
+    controller->deregisterCommands();
+
+    LOG_DEBUG(d2_logger, isc::log::DBGLVL_START_SHUT, DHCP_DDNS_RUN_EXIT);
 
 };
 
@@ -145,9 +160,9 @@ D2Process::canShutdown() const {
         }
 
         if (all_clear) {
-            LOG_DEBUG(dctl_logger, DBGLVL_START_SHUT,
+            LOG_DEBUG(d2_logger, isc::log::DBGLVL_START_SHUT,
                      DHCP_DDNS_CLEARED_FOR_SHUTDOWN)
-                     .arg(getShutdownTypeStr(shutdown_type_));
+                .arg(getShutdownTypeStr(shutdown_type_));
         }
     }
 
@@ -156,8 +171,9 @@ D2Process::canShutdown() const {
 
 isc::data::ConstElementPtr
 D2Process::shutdown(isc::data::ConstElementPtr args) {
-    LOG_DEBUG(dctl_logger, DBGLVL_START_SHUT, DHCP_DDNS_SHUTDOWN_COMMAND)
-              .arg(args ? args->str() : "(no arguments)");
+    LOG_DEBUG(d2_logger, isc::log::DBGLVL_START_SHUT,
+              DHCP_DDNS_SHUTDOWN_COMMAND)
+        .arg(args ? args->str() : "(no arguments)");
 
     // Default shutdown type is normal.
     std::string type_str(getShutdownTypeStr(SD_NORMAL));
@@ -189,13 +205,20 @@ D2Process::shutdown(isc::data::ConstElementPtr args) {
 }
 
 isc::data::ConstElementPtr
-D2Process::configure(isc::data::ConstElementPtr config_set) {
-    LOG_DEBUG(dctl_logger, DBGLVL_TRACE_BASIC,
-              DHCP_DDNS_CONFIGURE).arg(config_set->str());
+D2Process::configure(isc::data::ConstElementPtr config_set, bool check_only) {
+    LOG_DEBUG(d2_logger, isc::log::DBGLVL_TRACE_BASIC, DHCP_DDNS_CONFIGURE)
+        .arg(check_only ? "check" : "update")
+        .arg(config_set->str());
+
+    isc::data::ConstElementPtr answer;
+    answer = getCfgMgr()->simpleParseConfig(config_set, check_only,
+                boost::bind(&D2Process::reconfigureCommandChannel, this));
+    if (check_only) {
+        return (answer);
+    }
 
     int rcode = 0;
     isc::data::ConstElementPtr comment;
-    isc::data::ConstElementPtr answer = getCfgMgr()->parseConfig(config_set);;
     comment = isc::config::parseAnswer(rcode, answer);
 
     if (rcode) {
@@ -235,15 +258,14 @@ D2Process::checkQueueStatus() {
             // canceling active listening which may generate an IO event, so
             // instigate the stop and get out.
             try {
-                LOG_DEBUG(dctl_logger, DBGLVL_START_SHUT,
+                LOG_DEBUG(d2_logger, isc::log::DBGLVL_START_SHUT,
                           DHCP_DDNS_QUEUE_MGR_STOPPING)
-                         .arg(reconf_queue_flag_ ? "reconfiguration"
-                                                 : "shutdown");
+                    .arg(reconf_queue_flag_ ? "reconfiguration" : "shutdown");
                 queue_mgr_->stopListening();
             } catch (const isc::Exception& ex) {
-                // It is very unlikey that we would experience an error
+                // It is very unlikely that we would experience an error
                 // here, but theoretically possible.
-                LOG_ERROR(dctl_logger, DHCP_DDNS_QUEUE_MGR_STOP_ERROR)
+                LOG_ERROR(d2_logger, DHCP_DDNS_QUEUE_MGR_STOP_ERROR)
                           .arg(ex.what());
             }
         }
@@ -256,12 +278,12 @@ D2Process::checkQueueStatus() {
             size_t threshold = (((queue_mgr_->getMaxQueueSize()
                                 * QUEUE_RESTART_PERCENT)) / 100);
             if (queue_mgr_->getQueueSize() <= threshold) {
-                LOG_INFO (dctl_logger, DHCP_DDNS_QUEUE_MGR_RESUMING)
+                LOG_INFO (d2_logger, DHCP_DDNS_QUEUE_MGR_RESUMING)
                           .arg(threshold).arg(queue_mgr_->getMaxQueueSize());
                 try {
                     queue_mgr_->startListening();
                 } catch (const isc::Exception& ex) {
-                    LOG_ERROR(dctl_logger, DHCP_DDNS_QUEUE_MGR_RESUME_ERROR)
+                    LOG_ERROR(d2_logger, DHCP_DDNS_QUEUE_MGR_RESUME_ERROR)
                               .arg(ex.what());
                 }
             }
@@ -278,7 +300,7 @@ D2Process::checkQueueStatus() {
         // to keep from endlessly retrying over and over, with little time
         // in between.
         if (!shouldShutdown()) {
-            LOG_INFO (dctl_logger, DHCP_DDNS_QUEUE_MGR_RECOVERING);
+            LOG_INFO (d2_logger, DHCP_DDNS_QUEUE_MGR_RECOVERING);
             reconfigureQueueMgr();
         }
         break;
@@ -295,7 +317,7 @@ D2Process::checkQueueStatus() {
         // we can do the reconfigure. In other words, we aren't RUNNING or
         // STOPPING.
         if (reconf_queue_flag_) {
-            LOG_DEBUG(dctl_logger, DBGLVL_TRACE_BASIC,
+            LOG_DEBUG(d2_logger, isc::log::DBGLVL_TRACE_BASIC,
                       DHCP_DDNS_QUEUE_MGR_RECONFIGURING);
             reconfigureQueueMgr();
         }
@@ -326,7 +348,7 @@ D2Process::reconfigureQueueMgr() {
         /// @todo Remove this once we provide a secure mechanism.
         std::string ip_address =  d2_params->getIpAddress().toText();
         if (ip_address != "127.0.0.1" && ip_address != "::1") {
-            LOG_WARN(dctl_logger, DHCP_DDNS_NOT_ON_LOOPBACK).arg(ip_address);
+            LOG_WARN(d2_logger, DHCP_DDNS_NOT_ON_LOOPBACK).arg(ip_address);
         }
 
         // Instantiate the listener.
@@ -350,23 +372,10 @@ D2Process::reconfigureQueueMgr() {
         // Queue manager failed to initialize and therefore not listening.
         // This is most likely due to an unavailable IP address or port,
         // which is a configuration issue.
-        LOG_ERROR(dctl_logger, DHCP_DDNS_QUEUE_MGR_START_ERROR).arg(ex.what());
+        LOG_ERROR(d2_logger, DHCP_DDNS_QUEUE_MGR_START_ERROR).arg(ex.what());
     }
 }
 
-isc::data::ConstElementPtr
-D2Process::command(const std::string& command,
-                   isc::data::ConstElementPtr args) {
-    // @todo This is the initial implementation.  If and when D2 is extended
-    // to support its own commands, this implementation must change. Otherwise
-    // it should reject all commands as it does now.
-    LOG_DEBUG(dctl_logger, DBGLVL_TRACE_BASIC, DHCP_DDNS_COMMAND)
-        .arg(command).arg(args ? args->str() : "(no args)");
-
-    return (isc::config::createAnswer(COMMAND_INVALID, "Unrecognized command: "
-                                      + command));
-}
-
 D2Process::~D2Process() {
 };
 
@@ -397,5 +406,38 @@ const char* D2Process::getShutdownTypeStr(const ShutdownType& type) {
     return (str);
 }
 
+void
+D2Process::reconfigureCommandChannel() {
+    // Get new socket configuration.
+    isc::data::ConstElementPtr sock_cfg = getD2CfgMgr()->getControlSocketInfo();
+
+    // Determine if the socket configuration has changed. It has if
+    // both old and new configuration is specified but respective
+    // data elements aren't equal.
+    bool sock_changed = (sock_cfg && current_control_socket_ &&
+                         !sock_cfg->equals(*current_control_socket_));
+
+    // If the previous or new socket configuration doesn't exist or
+    // the new configuration differs from the old configuration we
+    // close the existing socket and open a new socket as appropriate.
+    // Note that closing an existing socket means the client will not
+    // receive the configuration result.
+    if (!sock_cfg || !current_control_socket_ || sock_changed) {
+        // Close the existing socket.
+        if (current_control_socket_) {
+            isc::config::CommandMgr::instance().closeCommandSocket();
+            current_control_socket_.reset();
+        }
+
+        // Open the new socket.
+        if (sock_cfg) {
+            isc::config::CommandMgr::instance().openCommandSocket(sock_cfg);
+        }
+    }
+
+    // Commit the new socket configuration.
+    current_control_socket_ = sock_cfg;
+}
+
 }; // namespace isc::d2
 }; // namespace isc
