@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -26,6 +26,7 @@
 #include <dhcpsrv/lease_mgr_factory.h>
 #include <log/logger_support.h>
 #include <stats/stats_mgr.h>
+#include <sstream>
 
 using namespace std;
 using namespace isc::asiolink;
@@ -272,7 +273,8 @@ HWAddrPtr Dhcpv4SrvTest::generateHWAddr(size_t size /*= 6*/) {
 void Dhcpv4SrvTest::checkAddressParams(const Pkt4Ptr& rsp,
                                        const Subnet4Ptr subnet,
                                        bool t1_present,
-                                       bool t2_present) {
+                                       bool t2_present,
+                                       uint32_t expected_valid) {
 
     // Technically inPool implies inRange, but let's be on the safe
     // side and check both.
@@ -285,10 +287,18 @@ void Dhcpv4SrvTest::checkAddressParams(const Pkt4Ptr& rsp,
     if (!opt) {
         ADD_FAILURE() << "Lease time option missing in response or the"
             " option has unexpected type";
+    } else if (subnet->getValid().getMin() != subnet->getValid().getMax()) {
+        EXPECT_GE(opt->getValue(), subnet->getValid().getMin());
+        EXPECT_LE(opt->getValue(), subnet->getValid().getMax());
     } else {
         EXPECT_EQ(opt->getValue(), subnet->getValid());
     }
 
+    // Check expected value when wanted.
+    if (opt && expected_valid) {
+      EXPECT_EQ(opt->getValue(), expected_valid);
+    }
+
     // Check T1 timer
     opt = boost::dynamic_pointer_cast<
         OptionUint32>(rsp->getOption(DHO_DHCP_RENEWAL_TIME));
@@ -634,10 +644,66 @@ Dhcpv4SrvTest::configure(const std::string& config, NakedDhcpv4Srv& srv,
     } );
 
     if (commit) {
+        auto cfg = CfgMgr::instance().getStagingCfg()->getD2ClientConfig();
+        cfg->setFetchGlobalsFn([]() -> ConstElementPtr {
+            return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
+        });
         CfgMgr::instance().commit();
     }
  }
 
+std::pair<int, std::string>
+Dhcpv4SrvTest::configureWithStatus(const std::string& config, NakedDhcpv4Srv& srv,
+                                   const bool commit, const int exp_rcode) {
+    ConstElementPtr json;
+    try {
+        json = parseJSON(config);
+    } catch (const std::exception& ex){
+        // Fatal falure on parsing error
+
+        std::stringstream tmp;
+        tmp << "parsing failure:"
+            << "config:" << config << std::endl
+            << "error: " << ex.what();
+        ADD_FAILURE() << tmp.str();
+        return (std::make_pair(-1, tmp.str()));
+    }
+
+    ConstElementPtr status;
+
+    // Disable the re-detect flag
+    disableIfacesReDetect(json);
+
+    // Configure the server and make sure the config is accepted
+    EXPECT_NO_THROW(status = configureDhcp4Server(srv, json));
+    EXPECT_TRUE(status);
+    if (!status) {
+        return (make_pair(-1, "configureDhcp4Server didn't return anythin"));
+    }
+
+    int rcode;
+    ConstElementPtr comment = config::parseAnswer(rcode, status);
+    EXPECT_EQ(exp_rcode, rcode) << comment->stringValue();
+
+    // Use specified lease database backend.
+    if (rcode == 0) {
+        EXPECT_NO_THROW( {
+            CfgDbAccessPtr cfg_db = CfgMgr::instance().getStagingCfg()->getCfgDbAccess();
+            cfg_db->setAppendedParameters("universe=4");
+            cfg_db->createManagers();
+            } );
+        if (commit) {
+            auto cfg = CfgMgr::instance().getStagingCfg()->getD2ClientConfig();
+            cfg->setFetchGlobalsFn([]() -> ConstElementPtr {
+                return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
+            });
+            CfgMgr::instance().commit();
+        }
+    }
+
+    return (std::make_pair(rcode, comment->stringValue()));
+ }
+
 Dhcpv4Exchange
 Dhcpv4SrvTest::createExchange(const Pkt4Ptr& query) {
     bool drop = false;
