@@ -1,4 +1,4 @@
-// Copyright (C) 2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2017 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -81,9 +81,9 @@ const bool STATEFUL = false;
 ///   - Single subnet 10.0.0.0/24 with a pool of 10.0.0.10-10.0.0.100
 ///   - Single reservation within the subnet:
 ///     - HW address: aa:bb:cc:dd:ee:ff
-///     - ip-adress 10.0.0.7
+///     - ip-address 10.0.0.7
 ///     - Vendor option for Cable Labs vendor id specified for the reservation:
-///       - TFTP servers suboption overriding globally spececified suboption:
+///       - TFTP servers suboption overriding globally specified suboption:
 ///         10.1.1.202,10.1.1.203
 ///
 const char* HOST_CONFIGS[] = {
@@ -203,7 +203,7 @@ const char* HOST_CONFIGS[] = {
         "\"valid-lifetime\": 600,"
         "\"option-data\": [ {"
         "    \"name\": \"vivso-suboptions\","
-        "    \"data\": 4491"
+        "    \"data\": \"4491\""
         "},"
         "{"
         "    \"name\": \"tftp-servers\","
@@ -221,7 +221,7 @@ const char* HOST_CONFIGS[] = {
         "        \"ip-address\": \"10.0.0.7\","
         "        \"option-data\": [ {"
         "            \"name\": \"vivso-suboptions\","
-        "            \"data\": 4491"
+        "            \"data\": \"4491\""
         "        },"
         "        {"
         "            \"name\": \"tftp-servers\","
@@ -252,7 +252,7 @@ public:
     /// Overridden options are requested with Parameter Request List
     /// option.
     ///
-    /// @param stateless Boolean value indicating if statless or stateful
+    /// @param stateless Boolean value indicating if stateless or stateful
     /// configuration should be performed.
     void testOverrideRequestedOptions(const bool stateless);
 
@@ -262,21 +262,21 @@ public:
     /// Overridden options are the options which server sends regardless
     /// if they are requested with Parameter Request List option or not.
     ///
-    /// @param stateless Boolean value indicating if statless or stateful
+    /// @param stateless Boolean value indicating if stateless or stateful
     /// configuration should be performed.
     void testOverrideDefaultOptions(const bool stateless);
 
     /// @brief Verifies that client receives options when they are solely
     /// defined in the host scope (and not in the global or subnet scope).
     ///
-    /// @param stateless Boolean value indicating if statless or stateful
+    /// @param stateless Boolean value indicating if stateless or stateful
     /// configuration should be performed.
     void testHostOnlyOptions(const bool stateless);
 
     /// @brief Verifies that host specific vendor options override vendor
     /// options defined in the global scope.
     ///
-    /// @param stateless Boolean value indicating if statless or stateful
+    /// @param stateless Boolean value indicating if stateless or stateful
     /// configuration should be performed.
     void testOverrideVendorOptions(const bool stateless);
 
