@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,8 +7,8 @@
 #include <config.h>
 
 #include <gtest/gtest.h>
-#include <cc/data.h>
 #include <dhcp4/parser_context.h>
+#include <dhcpsrv/parsers/simple_parser4.h>
 #include <testutils/io_utils.h>
 #include <testutils/user_context_utils.h>
 
@@ -288,6 +288,17 @@ TEST(ParserTest, file) {
     }
 }
 
+// This test loads the all-keys.json file and check global parameters.
+TEST(ParserTest, globalParameters) {
+    ConstElementPtr json;
+    Parser4Context ctx;
+    string fname = string(CFG_EXAMPLES) + "/" + "all-keys.json";
+    EXPECT_NO_THROW(json = ctx.parseFile(fname, Parser4Context::PARSER_DHCP4));
+    EXPECT_NO_THROW(json = json->get("Dhcp4"));
+    SimpleParser4 parser;
+    EXPECT_NO_THROW(parser.checkKeywords(parser.GLOBAL4_PARAMETERS, json));
+}
+
 // Basic test that checks if it's possible to specify outbound-interface.
 TEST(ParserTest, outboundIface) {
     std::string fname = string(CFG_EXAMPLES) + "/" + "advanced.json";
