@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -234,7 +234,7 @@ ControlledDhcpv6Srv::commandConfigReloadHandler(const string&,
         LOG_ERROR(dhcp6_logger, DHCP6_DYNAMIC_RECONFIGURATION_FAIL)
             .arg(file);
         return (createAnswer(CONTROL_RESULT_ERROR,
-                             "Config reload failed:" + string(ex.what())));
+                             "Config reload failed: " + string(ex.what())));
     }
 }
 
@@ -306,9 +306,8 @@ ControlledDhcpv6Srv::commandConfigSetHandler(const string&,
     string message;
 
     // Command arguments are expected to be:
-    // { "Dhcp6": { ... }, "Logging": { ... } }
-    // The Logging component is technically optional. If it's not supplied
-    // logging will revert to default logging.
+    // { "Dhcp6": { ... } }
+    // The Logging component is supported by backward compatiblity.
     if (!args) {
         message = "Missing mandatory 'arguments' parameter.";
     } else {
@@ -332,19 +331,39 @@ ControlledDhcpv6Srv::commandConfigSetHandler(const string&,
     // configuration attempts.
     CfgMgr::instance().rollback();
 
-    // Logging is a sibling element and must be parsed explicitly.
-    // The call to configureLogger parses the given Logging element if
-    // not null, into the staging config.  Note this does not alter the
-    // current loggers, they remain in effect until we apply the
-    // logging config below.  If no logging is supplied logging will
-    // revert to default logging.
-    Daemon::configureLogger(args->get("Logging"),
-                            CfgMgr::instance().getStagingCfg());
+    // Check deprecated, obsolete or unknown (aka unsupported) objects.
+    list<string> unsupported;
+    for (auto obj : args->mapValue()) {
+        const string& obj_name = obj.first;
+        if ((obj_name == "Dhcp6") || (obj_name == "Logging")) {
+            continue;
+        }
+        unsupported.push_back(obj_name);
+    }
+
+    // Relocate Logging: if there is a global Logging object takes its
+    // loggers entry, move the entry to Dhcp6 and remove now empty Logging.
+    Daemon::relocateLogging(args, "Dhcp6");
+
+    // Parse the logger configuration explicitly into the staging config.
+    // Note this does not alter the current loggers, they remain in
+    // effect until we apply the logging config below.  If no logging
+    // is supplied logging will revert to default logging.
+    Daemon::configureLogger(dhcp6, CfgMgr::instance().getStagingCfg());
 
     // Let's apply the new logging. We do it early, so we'll be able to print
-    // out what exactly is wrong with the new socnfig in case of problems.
+    // out what exactly is wrong with the new config in case of problems.
     CfgMgr::instance().getStagingCfg()->applyLoggingCfg();
 
+    // Log unsupported objects.
+    if (!unsupported.empty()) {
+        for (auto name : unsupported) {
+            LOG_ERROR(dhcp6_logger, DHCP6_CONFIG_UNSUPPORTED_OBJECT).arg(name);
+        }
+
+        // Will return an error in a future version.
+    }
+
     // Now we configure the server proper.
     ConstElementPtr result = processConfig(dhcp6);
 
@@ -356,6 +375,12 @@ ControlledDhcpv6Srv::commandConfigSetHandler(const string&,
     if (rcode == CONTROL_RESULT_SUCCESS) {
         CfgMgr::instance().getStagingCfg()->applyLoggingCfg();
 
+        // Update the fetch globals callback.
+        auto cfg = CfgMgr::instance().getStagingCfg()->getD2ClientConfig();
+        cfg->setFetchGlobalsFn([]() -> ConstElementPtr {
+            return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
+        });
+
         // Use new configuration.
         CfgMgr::instance().commit();
     } else {
@@ -376,9 +401,8 @@ ControlledDhcpv6Srv::commandConfigTestHandler(const string&,
     string message;
 
     // Command arguments are expected to be:
-    // { "Dhcp6": { ... }, "Logging": { ... } }
-    // The Logging component is technically optional. If it's not supplied
-    // logging will revert to default logging.
+    // { "Dhcp6": { ... } }
+    // The Logging component is supported by backward compatiblity.
     if (!args) {
         message = "Missing mandatory 'arguments' parameter.";
     } else {
@@ -402,6 +426,13 @@ ControlledDhcpv6Srv::commandConfigTestHandler(const string&,
     // configuration attempts.
     CfgMgr::instance().rollback();
 
+    // Check obsolete objects.
+
+    // Relocate Logging. Note this allows to check the loggers configuration.
+    Daemon::relocateLogging(args, "Dhcp6");
+
+    // Log obsolete objects and return an error.
+
     // Now we check the server proper.
     return (checkConfig(dhcp6));
 }
@@ -508,6 +539,17 @@ ControlledDhcpv6Srv::commandLeasesReclaimHandler(const string&,
     return (answer);
 }
 
+ConstElementPtr
+ControlledDhcpv6Srv::commandServerTagGetHandler(const std::string&,
+                                                ConstElementPtr) {
+    const std::string& tag =
+        CfgMgr::instance().getCurrentCfg()->getServerTag();
+    ElementPtr response = Element::createMap();
+    response->set("server-tag", Element::create(tag));
+
+    return (createAnswer(CONTROL_RESULT_SUCCESS, response));
+}
+
 isc::data::ConstElementPtr
 ControlledDhcpv6Srv::processCommand(const std::string& command,
                                     isc::data::ConstElementPtr args) {
@@ -562,6 +604,9 @@ ControlledDhcpv6Srv::processCommand(const std::string& command,
         } else if (command == "config-write") {
             return (srv->commandConfigWriteHandler(command, args));
 
+        } else if (command == "server-tag-get") {
+            return (srv->commandServerTagGetHandler(command, args));
+
         }
 
         return (isc::config::createAnswer(1, "Unrecognized command:"
@@ -617,7 +662,8 @@ ControlledDhcpv6Srv::processConfig(isc::data::ConstElementPtr config) {
 
     // Regenerate server identifier if needed.
     try {
-        const std::string duid_file = CfgMgr::instance().getDataDir() + "/" +
+        const std::string duid_file =
+            std::string(CfgMgr::instance().getDataDir()) + "/" +
             std::string(SERVER_DUID_FILE);
         DuidPtr duid = CfgMgr::instance().getStagingCfg()->getCfgDUID()->create(duid_file);
         server_->serverid_.reset(new Option(Option::V6, D6O_SERVERID, duid->getDuid()));
@@ -676,7 +722,8 @@ ControlledDhcpv6Srv::processConfig(isc::data::ConstElementPtr config) {
     // log warnings. Since we allow that this fails for some interfaces there
     // is no need to rollback configuration if socket fails to open on any
     // of the interfaces.
-    CfgMgr::instance().getStagingCfg()->getCfgIface()->openSockets(AF_INET6, srv->getPort());
+    CfgMgr::instance().getStagingCfg()->getCfgIface()->
+        openSockets(AF_INET6, srv->getServerPort());
 
     // Install the timers for handling leases reclamation.
     try {
@@ -693,6 +740,34 @@ ControlledDhcpv6Srv::processConfig(isc::data::ConstElementPtr config) {
         return (isc::config::createAnswer(1, err.str()));
     }
 
+    // Setup config backend polling, if configured for it.
+    auto ctl_info = CfgMgr::instance().getStagingCfg()->getConfigControlInfo();
+    if (ctl_info) {
+        long fetch_time = static_cast<long>(ctl_info->getConfigFetchWaitTime());
+        // Only schedule the CB fetch timer if the fetch wait time is greater
+        // than 0.
+        if (fetch_time > 0) {
+            // When we run unit tests, we want to use milliseconds unit for the
+            // specified interval. Otherwise, we use seconds. Note that using
+            // milliseconds as a unit in unit tests prevents us from waiting 1
+            // second on more before the timer goes off. Instead, we wait one
+            // millisecond which significantly reduces the test time.
+            if (!server_->inTestMode()) {
+                fetch_time = 1000 * fetch_time;
+            }
+
+            boost::shared_ptr<unsigned> failure_count(new unsigned(0));
+            TimerMgr::instance()->
+                registerTimer("Dhcp6CBFetchTimer",
+                              boost::bind(&ControlledDhcpv6Srv::cbFetchUpdates,
+                                          server_, CfgMgr::instance().getStagingCfg(),
+                                          failure_count),
+                              fetch_time,
+                              asiolink::IntervalTimer::ONE_SHOT);
+            TimerMgr::instance()->setup("Dhcp6CBFetchTimer");
+        }
+    }
+
     // Finally, we can commit runtime option definitions in libdhcp++. This is
     // exception free.
     LibDHCP::commitRuntimeOptionDefs();
@@ -737,8 +812,10 @@ ControlledDhcpv6Srv::checkConfig(isc::data::ConstElementPtr config) {
     return (configureDhcp6Server(*srv, config, true));
 }
 
-ControlledDhcpv6Srv::ControlledDhcpv6Srv(uint16_t port)
-    : Dhcpv6Srv(port), io_service_(), timer_mgr_(TimerMgr::instance()) {
+ControlledDhcpv6Srv::ControlledDhcpv6Srv(uint16_t server_port,
+                                         uint16_t client_port)
+    : Dhcpv6Srv(server_port, client_port), io_service_(),
+      timer_mgr_(TimerMgr::instance()) {
     if (server_) {
         isc_throw(InvalidOperation,
                   "There is another Dhcpv6Srv instance already.");
@@ -777,6 +854,9 @@ ControlledDhcpv6Srv::ControlledDhcpv6Srv(uint16_t port)
     CommandMgr::instance().registerCommand("leases-reclaim",
         boost::bind(&ControlledDhcpv6Srv::commandLeasesReclaimHandler, this, _1, _2));
 
+    CommandMgr::instance().registerCommand("server-tag-get",
+        boost::bind(&ControlledDhcpv6Srv::commandServerTagGetHandler, this, _1, _2));
+
     CommandMgr::instance().registerCommand("libreload",
         boost::bind(&ControlledDhcpv6Srv::commandLibReloadHandler, this, _1, _2));
 
@@ -807,6 +887,18 @@ ControlledDhcpv6Srv::ControlledDhcpv6Srv(uint16_t port)
 
     CommandMgr::instance().registerCommand("statistic-remove-all",
         boost::bind(&StatsMgr::statisticRemoveAllHandler, _1, _2));
+
+    CommandMgr::instance().registerCommand("statistic-sample-age-set",
+        boost::bind(&StatsMgr::statisticSetMaxSampleAgeHandler, _1, _2));
+
+    CommandMgr::instance().registerCommand("statistic-sample-age-set-all",
+        boost::bind(&StatsMgr::statisticSetMaxSampleAgeAllHandler, _1, _2));
+
+    CommandMgr::instance().registerCommand("statistic-sample-count-set",
+        boost::bind(&StatsMgr::statisticSetMaxSampleCountHandler, _1, _2));
+
+    CommandMgr::instance().registerCommand("statistic-sample-count-set-all",
+        boost::bind(&StatsMgr::statisticSetMaxSampleCountAllHandler, _1, _2));
 }
 
 void ControlledDhcpv6Srv::shutdown() {
@@ -838,6 +930,7 @@ ControlledDhcpv6Srv::~ControlledDhcpv6Srv() {
         CommandMgr::instance().deregisterCommand("dhcp-enable");
         CommandMgr::instance().deregisterCommand("leases-reclaim");
         CommandMgr::instance().deregisterCommand("libreload");
+        CommandMgr::instance().deregisterCommand("server-tag-get");
         CommandMgr::instance().deregisterCommand("shutdown");
         CommandMgr::instance().deregisterCommand("statistic-get");
         CommandMgr::instance().deregisterCommand("statistic-get-all");
@@ -845,6 +938,10 @@ ControlledDhcpv6Srv::~ControlledDhcpv6Srv() {
         CommandMgr::instance().deregisterCommand("statistic-remove-all");
         CommandMgr::instance().deregisterCommand("statistic-reset");
         CommandMgr::instance().deregisterCommand("statistic-reset-all");
+        CommandMgr::instance().deregisterCommand("statistic-sample-age-set");
+        CommandMgr::instance().deregisterCommand("statistic-sample-age-set-all");
+        CommandMgr::instance().deregisterCommand("statistic-sample-count-set");
+        CommandMgr::instance().deregisterCommand("statistic-sample-count-set-all");
         CommandMgr::instance().deregisterCommand("version-get");
 
     } catch (...) {
@@ -949,6 +1046,7 @@ ControlledDhcpv6Srv::dbLostCallback(ReconnectCtlPtr db_reconnect_ctl) {
         LOG_INFO(dhcp6_logger, DHCP6_DB_RECONNECT_DISABLED)
             .arg(db_reconnect_ctl->retriesLeft())
             .arg(db_reconnect_ctl->retryInterval());
+        ControlledDhcpv6Srv::processCommand("shutdown", ConstElementPtr());
         return(false);
     }
 
@@ -958,5 +1056,34 @@ ControlledDhcpv6Srv::dbLostCallback(ReconnectCtlPtr db_reconnect_ctl) {
     return(true);
 }
 
+void
+ControlledDhcpv6Srv::cbFetchUpdates(const SrvConfigPtr& srv_cfg,
+                                    boost::shared_ptr<unsigned> failure_count) {
+    try {
+        // Fetch any configuration backend updates since our last fetch.
+        server_->getCBControl()->databaseConfigFetch(srv_cfg,
+                                                     CBControlDHCPv6::FetchMode::FETCH_UPDATE);
+        (*failure_count) = 0;
+
+    } catch (const std::exception& ex) {
+        LOG_ERROR(dhcp6_logger, DHCP6_CB_FETCH_UPDATES_FAIL)
+            .arg(ex.what());
+
+        // We allow at most 10 consecutive failures after which we stop
+        // making further attempts to fetch the configuration updates.
+        // Let's return without re-scheduling the timer.
+        if (++(*failure_count) > 10) {
+            LOG_ERROR(dhcp6_logger, DHCP6_CB_FETCH_UPDATES_RETRIES_EXHAUSTED);
+            return;
+        }
+    }
+
+    // Reschedule the timer to fetch new updates or re-try if
+    // the previous attempt resulted in an error.
+    if (TimerMgr::instance()->isTimerRegistered("Dhcp6CBFetchTimer")) {
+        TimerMgr::instance()->setup("Dhcp6CBFetchTimer");
+    }
+}
+
 }; // end of isc::dhcp namespace
 }; // end of isc namespace
