@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,7 @@
 #ifndef DHCPV6_SRV_H
 #define DHCPV6_SRV_H
 
+#include <asiolink/io_service.h>
 #include <dhcp_ddns/ncr_msg.h>
 #include <dhcp/dhcp6.h>
 #include <dhcp/duid.h>
@@ -16,12 +17,16 @@
 #include <dhcp/option_definition.h>
 #include <dhcp/pkt6.h>
 #include <dhcpsrv/alloc_engine.h>
+#include <dhcpsrv/callout_handle_store.h>
+#include <dhcpsrv/cb_ctl_dhcp6.h>
 #include <dhcpsrv/cfg_option.h>
 #include <dhcpsrv/d2_client_mgr.h>
+#include <dhcpsrv/network_state.h>
 #include <dhcpsrv/subnet.h>
 #include <hooks/callout_handle.h>
-#include <dhcpsrv/daemon.h>
+#include <process/daemon.h>
 
+#include <functional>
 #include <iostream>
 #include <queue>
 
@@ -52,7 +57,11 @@ public:
 /// that is going to be used as server-identifier, receives incoming
 /// packets, processes them, manages leases assignment and generates
 /// appropriate responses.
-class Dhcpv6Srv : public Daemon {
+class Dhcpv6Srv : public process::Daemon {
+private:
+
+    /// @brief Pointer to IO service used by the server.
+    asiolink::IOServicePtr io_service_;
 
 public:
     /// @brief defines if certain option may, must or must not appear
@@ -72,17 +81,46 @@ public:
     /// network interaction. Will instantiate lease manager, and load
     /// old or create new DUID.
     ///
-    /// @param port port on will all sockets will listen
-    Dhcpv6Srv(uint16_t port = DHCP6_SERVER_PORT);
+    /// @param server_port port on which all sockets will listen
+    /// @param client_port port to which all responses will be sent
+    Dhcpv6Srv(uint16_t server_port = DHCP6_SERVER_PORT,
+              uint16_t client_port = 0);
 
     /// @brief Destructor. Used during DHCPv6 service shutdown.
     virtual ~Dhcpv6Srv();
 
+    /// @brief Checks if the server is running in unit test mode.
+    ///
+    /// @return true if the server is running in unit test mode,
+    /// false otherwise.
+    bool inTestMode() const {
+        return (server_port_ == 0);
+    }
+
+    /// @brief Returns pointer to the IO service used by the server.
+    asiolink::IOServicePtr& getIOService() {
+        return (io_service_);
+    }
+
+    /// @brief Returns pointer to the network state used by the server.
+    NetworkStatePtr& getNetworkState() {
+        return (network_state_);
+    }
+
+    /// @brief Returns an object which controls access to the configuration
+    /// backends.
+    ///
+    /// @return Pointer to the instance of the object which controls
+    /// access to the configuration backends.
+    CBControlDHCPv6Ptr getCBControl() const {
+        return (cb_control_);
+    }
+
     /// @brief returns Kea version on stdout and exit.
-    /// redeclaration/redefinition. @ref Daemon::getVersion()
+    /// redeclaration/redefinition. @ref isc::process::Daemon::getVersion()
     static std::string getVersion(bool extended);
 
-    /// @brief Returns server-indentifier option.
+    /// @brief Returns server-identifier option.
     ///
     /// @return server-id option
     OptionPtr getServerID() { return serverid_; }
@@ -120,8 +158,8 @@ public:
     /// used for testing purposes.
     ///
     /// @return UDP port on which server should listen.
-    uint16_t getPort() const {
-        return (port_);
+    uint16_t getServerPort() const {
+        return (server_port_);
     }
 
     /// @brief Starts DHCP_DDNS client IO if DDNS updates are enabled.
@@ -156,6 +194,11 @@ public:
                                       NameChangeSender::Result result,
                                       dhcp_ddns::NameChangeRequestPtr& ncr);
 
+    /// @brief Discards cached and parked packets
+    /// Clears the call_handle store and packet parking lots
+    /// of all packets.  Called during reconfigure and shutdown.
+    void discardPackets();
+
 protected:
 
     /// @brief Compare received server id with our server id
@@ -170,9 +213,9 @@ protected:
 
     /// @brief Check if the message can be sent to unicast.
     ///
-    /// This function checks if the received message conforms to the section 15
-    /// of RFC3315 which says that: "A server MUST discard any Solicit, Confirm,
-    /// Rebind or Information-request messages it receives with a unicast
+    /// This function checks if the received message conforms to the section 16
+    /// of RFC 8415 which says that: "A server MUST discard any Solicit, Confirm,
+    /// Rebind or Information-request messages it receives with a Layer 3 unicast
     /// destination address.
     ///
     /// @param pkt DHCPv6 message to be checked.
@@ -180,6 +223,16 @@ protected:
     /// not allowed according to RFC3315, section 15; true otherwise.
     bool testUnicast(const Pkt6Ptr& pkt) const;
 
+    /// @brief Verifies if specified packet meets RFC requirements
+    ///
+    /// Checks if mandatory option is really there, that forbidden option
+    /// is not there, and that client-id or server-id appears only once.
+    ///
+    /// @param pkt packet to be checked
+    /// @return false if the message should be dropped as a result of the
+    /// sanity check.
+    bool sanityCheck(const Pkt6Ptr& pkt);
+
     /// @brief verifies if specified packet meets RFC requirements
     ///
     /// Checks if mandatory option is really there, that forbidden option
@@ -192,6 +245,13 @@ protected:
     void sanityCheck(const Pkt6Ptr& pkt, RequirementLevel clientid,
                      RequirementLevel serverid);
 
+    /// @brief verifies if received DUID option (client-id or server-id) is sane
+    ///
+    /// @param opt option to be checked
+    /// @param opt_name text name to be printed
+    /// @throw RFCViolation if any issues are detected
+    void sanityCheckDUID(const OptionPtr& opt, const std::string& opt_name);
+
     /// @brief Processes incoming Solicit and returns response.
     ///
     /// Processes received Solicit message and verifies that its sender
@@ -203,10 +263,10 @@ protected:
     /// instead of Advertise and requested leases will be assigned
     /// immediately.
     ///
-    /// @param solicit Solicit message received from client
+    /// @param ctx Reference to client context
     ///
     /// @return Advertise, Reply message or NULL.
-    Pkt6Ptr processSolicit(const Pkt6Ptr& solicit);
+    Pkt6Ptr processSolicit(AllocEngine::ClientContext6& ctx);
 
     /// @brief Processes incoming Request and returns Reply response.
     ///
@@ -216,16 +276,17 @@ protected:
     /// prefixes, respectively. Uses LeaseMgr to allocate or update existing
     /// leases.
     ///
-    /// @param request a message received from client
+    /// @param ctx Reference to client context
     ///
     /// @return REPLY message or NULL
-    Pkt6Ptr processRequest(const Pkt6Ptr& request);
+    Pkt6Ptr processRequest(AllocEngine::ClientContext6& ctx);
 
     /// @brief Processes incoming Renew message.
     ///
-    /// @param renew message received from the client
+    /// @param ctx Reference to client context
+    ///
     /// @return Reply message to be sent to the client.
-    Pkt6Ptr processRenew(const Pkt6Ptr& renew);
+    Pkt6Ptr processRenew(AllocEngine::ClientContext6& ctx);
 
     /// @brief Processes incoming Rebind message.
     ///
@@ -235,14 +296,15 @@ protected:
     /// client is on the appropriate link or not. We don't seem to do it
     /// now.
     ///
-    /// @param rebind message received from the client.
+    /// @param ctx Reference to client context
+    ///
     /// @return Reply message to be sent to the client.
-    Pkt6Ptr processRebind(const Pkt6Ptr& rebind);
+    Pkt6Ptr processRebind(AllocEngine::ClientContext6& ctx);
 
     /// @brief Processes incoming Confirm message and returns Reply.
     ///
     /// This function processes Confirm message from the client according
-    /// to section 18.2.2. of RFC3315. It discards the Confirm message if
+    /// to section 18.3.3. of RFC 8415. It discards the Confirm message if
     /// the message sent by the client contains no addresses, i.e. it has
     /// no IA_NA options or all IA_NA options contain no IAAddr options.
     ///
@@ -258,17 +320,18 @@ protected:
     /// code NotOnLink is returned. Otherwise, the Reply message with the
     /// status code Success is returned.
     ///
-    /// @param confirm Confirm message sent by a client.
+    /// @param ctx Reference to client context
     ///
     /// @return Reply message from the server or NULL pointer if Confirm
     /// message should be discarded by the server.
-    Pkt6Ptr processConfirm(const Pkt6Ptr& confirm);
+    Pkt6Ptr processConfirm(AllocEngine::ClientContext6& ctx);
 
     /// @brief Process incoming Release message.
     ///
-    /// @param release message received from client
+    /// @param ctx Reference to client context
+    ///
     /// @return Reply message to be sent to the client.
-    Pkt6Ptr processRelease(const Pkt6Ptr& release);
+    Pkt6Ptr processRelease(AllocEngine::ClientContext6& ctx);
 
     /// @brief Process incoming Decline message.
     ///
@@ -277,17 +340,17 @@ protected:
     /// the client's message. Finally, it calls @ref declineLeases, where
     /// the actual address processing takes place.
     ///
-    /// @throw RFCViolation if Decline message is invalid (lacking mandatory
-    ///                     options)
+    /// @param ctx Reference to client context
     ///
-    /// @param decline message received from client
-    Pkt6Ptr processDecline(const Pkt6Ptr& decline);
+    /// @return Reply message to be sent to the client.
+    Pkt6Ptr processDecline(AllocEngine::ClientContext6& ctx);
 
     /// @brief Processes incoming Information-request message.
     ///
-    /// @param inf_request message received from client
+    /// @param ctx Reference to client context
+    ///
     /// @return Reply message to be sent to the client.
-    Pkt6Ptr processInfRequest(const Pkt6Ptr& inf_request);
+    Pkt6Ptr processInfRequest(AllocEngine::ClientContext6& ctx);
 
     /// @brief Processes incoming DHCPv4-query message.
     ///
@@ -297,13 +360,15 @@ protected:
     /// to the client once we get back DHCP4-REPLY from the DHCPv4 server.
     ///
     /// @param dhcp4_query message received from client
+    /// Does not throw
     void processDhcp4Query(const Pkt6Ptr& dhcp4_query);
 
     /// @brief Selects a subnet for a given client's packet.
     ///
     /// @param question client's message
+    /// @param drop if it is true the packet will be dropped
     /// @return selected subnet (or NULL if no suitable subnet was found)
-    isc::dhcp::Subnet6Ptr selectSubnet(const Pkt6Ptr& question);
+    isc::dhcp::Subnet6Ptr selectSubnet(const Pkt6Ptr& question, bool& drop);
 
     /// @brief Processes IA_NA option (and assigns addresses if necessary).
     ///
@@ -335,7 +400,7 @@ protected:
     /// allocation failure.
     ///
     /// @param query client's message (typically SOLICIT or REQUEST)
-    /// @param answer server's response to the client's message.
+    /// @param answer server's response to the client's message (unused).
     /// @param ctx client context (contains subnet, duid and other parameters)
     /// @param ia pointer to client's IA_PD option (client's request)
     /// @return IA_PD option (server's response)
@@ -351,18 +416,6 @@ protected:
     /// lease is found, an IA_NA response is generated with an appropriate
     /// status code.
     ///
-    /// @todo The behavior of this function will need to be extended to support
-    /// draft-ietf-dhc-dhcpv6-stateful-issues. This draft modifies the behavior
-    /// described in RFC3315 with respect to Renew and Rebind processing. Key
-    /// changes are (version -05):
-    /// - Renewing and Rebinding client MAY request additional bindings by
-    /// putting an IA for all bindings it desires but has been unable to obtain.
-    /// Server MAY allocate addresses if it finds that they are appropriate for
-    /// the link that client is attached to.
-    /// - When receiving Rebind, if the server determines that the addresses are
-    /// not appropriate for the link the client is attached to, the server MAY
-    /// send the IA with address lifetimes set to 0 or discard the message.
-    ///
     /// @param query client's message (Renew or Rebind)
     /// @param answer server's response to the client's message. This
     /// message should contain Client FQDN option being sent by the server
@@ -380,11 +433,46 @@ protected:
     /// This function is called by the logic which processes Renew and Rebind
     /// messages to extend the lifetime of the existing prefix.
     ///
-    /// The behavior of this function is different in that when there is no
-    /// binding found in the lease database for the particular client the
-    /// NoBinding status code is returned when processing Renew, the exception
-    /// is thrown when there is no binding and the Rebind message is processed
-    /// (see RFC3633, section 12.2. for details).
+    /// The behavior of this function is different than @c extendIA_NA in that
+    /// when there is no subnet found for the rebinding case, the Rebind message
+    /// is discarded by the server. That behavior is based on the following
+    /// statement from the RFC 8415, section 18.3.5:
+    ///
+    /// "If the server chooses to not include any IAs containing IA Address or
+    /// IA Prefix options with lifetimes of 0 and the server does not include
+    /// any other IAs with leases and/or status codes, the server does not send
+    /// a Reply message.  In this situation, the server discards the Rebind
+    /// message".
+    ///
+    /// @todo We should consider unification of the server behavior for address
+    /// assignment and prefix delegation with respect to Rebind message
+    /// processing. The RFC 8415, section 18.3.5 doesn't really differentiate
+    /// between IA_NA and IA_PD in how they should be processed by the server.
+    /// The intention of the spec is as follows:
+    ///
+    /// - If the server finds a lease but addresses and/or prefixes are not
+    ///   appropriate anymore, it sends them with zero lifetimes.
+    /// - If the server doesn't find a lease the server checks if the addresses
+    ///   and/or prefixes the client sends are appropriate and sends them back
+    ///   with zero lifetimes if they aren't.
+    /// - The server may choose to not respond at all, if it cannot determine
+    ///   whether the addresses and/or prefixes are appropriate and it doesn't
+    ///   allocate any other addresses and/or prefixes.
+    /// - If the server cannot find the leases included in the Rebind, the
+    ///   server may either allocate the leases or simply return NoBinding.
+    ///
+    /// The @c extendIA_PD function drops the Rebind message if it cannot find
+    /// the client entry (as a result of not finding a subnet for the client),
+    /// the @c extendIA_NA function sends NoBinding status code in that case.
+    /// Perhaps we should introduce an "Authoritative" configuration flag which,
+    /// if enabled, would cause the server to always respond, either indicating
+    /// that the address/prefix is inappropriate (with zero lifetimes) or that
+    /// there is no binding (NoBinding status code) for both addresses and
+    /// prefixes. When the "Authoritative" flag is disabled the server would
+    /// drop the Rebind for which there is neither subnet selected nor client
+    /// entry found (as it could be handled by another DHCP server). If nothing
+    /// else we could consider unifying the behavior of @c extendIA_NA and
+    /// @c extendIA_PD with respect to Rebind processing.
     ///
     /// @param query client's message
     /// @param ctx client context (contains subnet, duid and other parameters)
@@ -404,8 +492,8 @@ protected:
     /// lease is found, an IA_NA response is generated with an appropriate
     /// status code.
     ///
-    /// As RFC 3315 requires that a single status code be sent for the whole message,
-    /// this method may update the passed general_status: it is set to SUCCESS when
+    /// The server sends top-level Status Code option. This method may update the
+    /// passed value of that option, i.e. general_status. It is set to SUCCESS when
     /// message processing begins, but may be updated to some error code if the
     /// release process fails.
     ///
@@ -413,10 +501,12 @@ protected:
     /// @param query client's message
     /// @param general_status a global status (it may be updated in case of errors)
     /// @param ia IA_NA option that is being released
+    /// @param old_lease a pointer to the lease being released
     /// @return IA_NA option (server's response)
     OptionPtr releaseIA_NA(const DuidPtr& duid, const Pkt6Ptr& query,
                            int& general_status,
-                           boost::shared_ptr<Option6IA> ia);
+                           boost::shared_ptr<Option6IA> ia,
+                           Lease6Ptr& old_lease);
 
     /// @brief Releases specific IA_PD option
     ///
@@ -429,10 +519,12 @@ protected:
     /// @param query client's message
     /// @param general_status a global status (it may be updated in case of errors)
     /// @param ia IA_PD option that is being released
+    /// @param old_lease a pointer to the lease being released
     /// @return IA_PD option (server's response)
     OptionPtr releaseIA_PD(const DuidPtr& duid, const Pkt6Ptr& query,
                            int& general_status,
-                           boost::shared_ptr<Option6IA> ia);
+                           boost::shared_ptr<Option6IA> ia,
+                           Lease6Ptr& old_lease);
 
     /// @brief Copies required options from client message to server answer.
     ///
@@ -475,6 +567,7 @@ protected:
     ///
     /// @param question client's message
     /// @param answer server's message (options will be added here)
+    ///
     /// @param co_list configured option list
     void appendRequestedOptions(const Pkt6Ptr& question, Pkt6Ptr& answer,
                                 const CfgOptionList& co_list);
@@ -560,9 +653,11 @@ protected:
     /// The FQDN option carries response to the client about DNS updates that
     /// server intends to perform for the DNS client. Based on this, the
     /// function will create zero or more @c isc::dhcp_ddns::NameChangeRequest
-    /// objects and store them in the internal queue. Requests created by this
-    /// function are only for adding or updating DNS records. If DNS updates are
-    /// disabled, this method returns immediately.
+    /// objects and store them in the internal queue.  To catch lease renewals
+    /// that alter the FQDN, the function first looks at the context's changed
+    /// list of leases (if any) to determine if DNS entries need to be removed.
+    /// It then looks at the valid leases to determine if any DNS entries need
+    /// to be added. If DNS updates are disabled, this method returns immediately.
     ///
     /// @todo Add support for multiple IAADDR options in the IA_NA.
     ///
@@ -586,6 +681,33 @@ protected:
     void extendLeases(const Pkt6Ptr& query, Pkt6Ptr& reply,
                       AllocEngine::ClientContext6& ctx);
 
+    /// @brief Sets the T1 and T2 timers in the outbound IA
+    ///
+    /// This method determines the values for both the T1 and T2
+    /// timers for the given IA. It is influenced by the
+    /// lease's subnet's values for renew-timer, rebind-timer,
+    /// calculate-tee-times, t1-percent, and t2-percent as follows:
+    ///
+    /// T2:
+    ///
+    /// The value for T2 defaults to zero. If the rebind-timer value is
+    /// specified then use it.  If not and calculate-tee-times is true, then
+    /// use the value given by: preferred lease time * t2-percent.
+    ///
+    /// T1:
+    ///
+    /// The candidate value for T1 defaults to zero. If the renew-timer value
+    /// is specified then use it. If not and calculate-tee-times is true, then
+    /// use the value given by: preferred lease time * t1-percent.
+    ///
+    /// The T1 candidate will be used provided it less than to T2,
+    /// otherwise it will be set T1 to zero.
+    ///
+    /// @param preferred_lft preferred lease time of the lease being assigned to the client
+    /// @param subnet the subnet to which the lease belongs
+    /// @param resp outbound IA option in which the timers are set.
+    void setTeeTimes(uint32_t preferred_lft, const Subnet6Ptr& subnet, Option6IAPtr& resp);
+
     /// @brief Attempts to release received addresses
     ///
     /// It iterates through received IA_NA options and attempts to release
@@ -631,6 +753,18 @@ protected:
     /// @param pkt packet to be classified
     void classifyPacket(const Pkt6Ptr& pkt);
 
+    /// @brief Evaluate classes.
+    ///
+    /// @note Second part of the classification.
+    ///
+    /// Evaluate expressions of client classes: if it returns true the class
+    /// is added to the incoming packet.
+    ///
+    /// @param pkt packet to be classified.
+    /// @param depend_on_known if false classes depending on the KNOWN or
+    /// UNKNOWN classes are skipped, if true only these classes are evaluated.
+    void evaluateClasses(const Pkt6Ptr& pkt, bool depend_on_known);
+
     /// @brief Assigns classes retrieved from host reservation database.
     ///
     /// @param pkt Pointer to the packet to which classes will be assigned.
@@ -638,7 +772,20 @@ protected:
     void setReservedClientClasses(const Pkt6Ptr& pkt,
                                   const AllocEngine::ClientContext6& ctx);
 
-    /// @brief Attempts to get a MAC/hardware address using configred sources
+    /// @brief Assigns incoming packet to zero or more classes (required pass).
+    ///
+    /// @note This required classification evaluates all classes which
+    /// were marked for required evaluation. Classes are collected so
+    /// evaluated in the reversed order than output option processing.
+    ///
+    /// @note The only-if-required flag is related because it avoids
+    /// double evaluation (which is not forbidden).
+    ///
+    /// @param pkt packet to be classified
+    /// @param ctx allocation context where to get informations
+    void requiredClassify(const Pkt6Ptr& pkt, AllocEngine::ClientContext6& ctx);
+
+    /// @brief Attempts to get a MAC/hardware address using configured sources
     ///
     /// Tries to extract MAC/hardware address information from the packet
     /// using MAC sources configured in 'mac-sources' configuration parameter.
@@ -676,9 +823,12 @@ protected:
     ///
     /// @param pkt pointer to a packet for which context will be created.
     /// @param [out] ctx reference to context object to be initialized.
-    void initContext(const Pkt6Ptr& pkt, AllocEngine::ClientContext6& ctx);
+    /// @param [out] drop if it is true the packet will be dropped.
+    void initContext(const Pkt6Ptr& pkt,
+                     AllocEngine::ClientContext6& ctx,
+                     bool& drop);
 
-    /// @brief this is a prefix added to the contend of vendor-class option
+    /// @brief this is a prefix added to the content of vendor-class option
     ///
     /// If incoming packet has a vendor class option, its content is
     /// prepended with this prefix and then interpreted as a class.
@@ -691,7 +841,7 @@ protected:
     /// This method iterates over all IA_NA options and calls @ref declineIA on
     /// each of them.
     ///
-    /// @param decline Decline messege sent by a client
+    /// @param decline Decline message sent by a client
     /// @param reply Server's response (IA_NA with status will be added here)
     /// @param ctx context
     /// @return true when expected to continue, false when hooks told us to drop
@@ -710,10 +860,11 @@ protected:
     /// @param duid client's duid (used to verify if the client owns the lease)
     /// @param general_status [out] status in top-level message (may be updated)
     /// @param ia specific IA_NA option to process.
+    /// @param new_leases a collection of leases being declined.
     /// @return IA_NA option with response (to be included in Reply message)
     OptionPtr
     declineIA(const Pkt6Ptr& decline, const DuidPtr& duid, int& general_status,
-              boost::shared_ptr<Option6IA> ia);
+              boost::shared_ptr<Option6IA> ia, Lease6Collection& new_leases);
 
     /// @brief Declines specific IPv6 lease.
     ///
@@ -721,7 +872,7 @@ protected:
     /// - cleans up DNS, if necessary
     /// - updates subnet[X].declined-addresses (per subnet stat)
     /// - updates declined-addresses (global stat)
-    /// - deassociates client information from the lease
+    /// - disassociates client information from the lease
     /// - moves the lease to DECLINED state
     /// - sets lease expiration time to decline-probation-period
     /// - adds status-code success
@@ -742,6 +893,16 @@ protected:
     void setStatusCode(boost::shared_ptr<Option6IA>& container,
                        const OptionPtr& status);
 
+public:
+
+    /// Used for DHCPv4-over-DHCPv6 too.
+
+    /// @brief Check if the last relay added a relay-source-port option.
+    ///
+    /// @param query DHCPv6 message to be checked.
+    /// @return the port to use to join the relay or 0 for the default.
+    static uint16_t checkRelaySourcePort(const Pkt6Ptr& query);
+
 private:
 
     /// @public
@@ -753,6 +914,25 @@ private:
     /// @param classes a reference to added class names for logging
     void classifyByVendor(const Pkt6Ptr& pkt, std::string& classes);
 
+    /// @brief Update FQDN based on the reservations in the current subnet.
+    ///
+    /// When shared networks are in use the allocation engine may switch to
+    /// a different subnet than originally selected. If this new subnet has
+    /// hostname reservations there is a need to update the FQDN option
+    /// value.
+    ///
+    /// This method should be called after lease assignments to perform
+    /// such update when required.
+    ///
+    /// @param ctx Client context.
+    /// @param answer Message being sent to a client, which may hold an FQDN
+    /// option to be updated.
+    ///
+    /// @throw isc::Unexpected if specified message is NULL. This is treated
+    /// as a programmatic error.
+    void updateReservedFqdn(const AllocEngine::ClientContext6& ctx,
+                            const Pkt6Ptr& answer);
+
     /// @private
     /// @brief Generate FQDN to be sent to a client if none exists.
     ///
@@ -791,17 +971,32 @@ private:
     ///
     /// @param answer Message being sent to a client, which may hold IA_NA
     /// and Client FQDN options to be used to generate name for a client.
+    /// @param ctx Client context.
     ///
     /// @throw isc::Unexpected if specified message is NULL. This is treated
     /// as a programmatic error.
-    void generateFqdn(const Pkt6Ptr& answer);
+    void generateFqdn(const Pkt6Ptr& answer,
+                      AllocEngine::ClientContext6& ctx);
 
     /// @brief Updates statistics for received packets
     /// @param query packet received
     static void processStatsReceived(const Pkt6Ptr& query);
 
+    /// @brief Checks if the specified option code has been requested using
+    /// the Option Request option.
+    ///
+    /// @param query Pointer to the client's query.
+    /// @parma code Option code.
+    ///
+    /// @return true if option has been requested in the ORO.
+    bool requestedInORO(const Pkt6Ptr& query, const uint16_t code) const;
+
     /// UDP port number on which server listens.
-    uint16_t port_;
+    uint16_t server_port_;
+
+protected:
+    /// UDP port number to which server sends all responses.
+    uint16_t client_port_;
 
 public:
     /// @note used by DHCPv4-over-DHCPv6 so must be public and static
@@ -814,6 +1009,13 @@ public:
     /// @return the index of the buffer6_send hook
     static int getHookIndexBuffer6Send();
 
+    /// @brief Executes buffer6_send callout and sends the response.
+    ///
+    /// @param callout_handle pointer to the callout handle.
+    /// @param rsp pointer to a response.
+    void processPacketBufferSend(hooks::CalloutHandlePtr& callout_handle,
+                                 Pkt6Ptr& rsp);
+
 protected:
 
     /// Server DUID (to be sent in server-identifier option)
@@ -823,6 +1025,14 @@ protected:
     /// initiate server shutdown procedure.
     volatile bool shutdown_;
 
+    /// @brief Executes pkt6_send callout.
+    ///
+    /// @param callout_handle pointer to the callout handle.
+    /// @param query Pointer to a query.
+    /// @param rsp Pointer to a response.
+    void processPacketPktSend(hooks::CalloutHandlePtr& callout_handle,
+                              Pkt6Ptr& query, Pkt6Ptr& rsp);
+
     /// @brief Allocation Engine.
     /// Pointer to the allocation engine that we are currently using
     /// It must be a pointer, because we will support changing engines
@@ -832,6 +1042,13 @@ protected:
     /// Holds a list of @c isc::dhcp_ddns::NameChangeRequest objects, which
     /// are waiting for sending to kea-dhcp-ddns module.
     std::queue<isc::dhcp_ddns::NameChangeRequest> name_change_reqs_;
+
+    /// @brief Holds information about disabled DHCP service and/or
+    /// disabled subnet/network scopes.
+    NetworkStatePtr network_state_;
+
+    /// @brief Controls access to the configuration backends.
+    CBControlDHCPv6Ptr cb_control_;
 };
 
 }; // namespace isc::dhcp
