@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,8 +7,9 @@
 #ifndef COMMAND_OPTIONS_H
 #define COMMAND_OPTIONS_H
 
-#include <boost/noncopyable.hpp>
+#include <dhcp/option.h>
 
+#include <boost/noncopyable.hpp>
 #include <stdint.h>
 #include <string>
 #include <vector>
@@ -16,6 +17,11 @@
 namespace isc {
 namespace perfdhcp {
 
+enum class Scenario {
+    BASIC,
+    AVALANCHE
+};
+
 /// \brief Command Options.
 ///
 /// This class is responsible for parsing the command-line and storing the
@@ -24,6 +30,14 @@ namespace perfdhcp {
 class CommandOptions : public boost::noncopyable {
 public:
 
+    /// \brief Default Constructor.
+    ///
+    /// Private constructor as this is a singleton class.
+    /// Use CommandOptions::instance() to get instance of it.
+    CommandOptions() {
+        reset();
+    }
+
     /// @brief A vector holding MAC addresses.
     typedef std::vector<std::vector<uint8_t> > MacAddrsVector;
 
@@ -105,12 +119,6 @@ public:
         DORA_SARR
     };
 
-    /// CommandOptions is a singleton class. This method returns reference
-    /// to its sole instance.
-    ///
-    /// \return the only existing instance of command options
-    static CommandOptions& instance();
-
     /// \brief Reset to defaults
     ///
     /// Reset data members to default values. This is specifically
@@ -145,7 +153,7 @@ public:
     /// \return type of lease being requested by perfdhcp.
     LeaseType getLeaseType() const { return (lease_type_); }
 
-    /// \brief Returns echange rate.
+    /// \brief Returns exchange rate.
     ///
     /// \return exchange rate per second.
     int getRate() const { return rate_; }
@@ -237,22 +245,27 @@ public:
     /// \return number of preload exchanges.
     int getPreload() const { return preload_; }
 
-    /// \brief Returns aggressivity value.
-    ///
-    /// \return aggressivity value.
-    int getAggressivity() const { return aggressivity_; }
-
     /// \brief Returns local port number.
     ///
     /// \return local port number.
     int getLocalPort() const { return local_port_; }
 
+    /// \brief Returns remote port number.
+    ///
+    /// \return remote port number.
+    int getRemotePort() const { return remote_port_; }
+
+    /// @brief Returns the time in microseconds to delay the program by.
+    ///
+    /// @return the time in microseconds to delay the program by.
+    int getExitWaitTime() const { return exit_wait_time_; }
+
     /// \brief Checks if seed provided.
     ///
     /// \return true if seed was provided.
     bool isSeeded() const { return seeded_; }
 
-    /// \brief Returns radom seed.
+    /// \brief Returns random seed.
     ///
     /// \return random seed.
     uint32_t getSeed() const { return seed_; }
@@ -272,7 +285,7 @@ public:
     /// \return true if server-iD to be taken from first package.
     bool isUseFirst() const { return use_first_; }
 
-    /// \brief Check if generated DHCPv6 messages shuold appear as relayed.
+    /// \brief Check if generated DHCPv6 messages should appear as relayed.
     ///
     /// \return true if generated traffic should appear as relayed.
     bool isUseRelayedV6() const { return (v6_relay_encapsulation_level_ > 0); }
@@ -332,11 +345,38 @@ public:
     /// \return wrapped command (start/stop).
     std::string getWrapped() const { return wrapped_; }
 
+    /// @brief Returns extra options to be inserted.
+    ///
+    /// @return container with options
+    const isc::dhcp::OptionCollection& getExtraOpts() const { return extra_opts_; }
+
+    /// \brief Check if single-threaded mode is enabled.
+    ///
+    /// \return true if single-threaded mode is enabled.
+    bool isSingleThreaded() const { return single_thread_mode_; }
+
+    /// \brief Returns selected scenario.
+    ///
+    /// \return enum Scenario.
+    Scenario getScenario() const { return scenario_; }
+
     /// \brief Returns server name.
     ///
     /// \return server name.
     std::string getServerName() const { return server_name_; }
 
+
+    /// \brief Find if diagnostic flag has been set.
+    ///
+    /// \param diag diagnostic flag (a,e,i,s,r,t,T).
+    /// \return true if diagnostics flag has been set.
+    bool testDiags(const char diag) {
+        if (getDiags().find(diag) != std::string::npos) {
+            return (true);
+        }
+        return (false);
+    }
+
     /// \brief Print command line arguments.
     void printCommandLine() const;
 
@@ -351,15 +391,6 @@ public:
     void version() const;
 
 private:
-
-    /// \brief Default Constructor.
-    ///
-    /// Private constructor as this is a singleton class.
-    /// Use CommandOptions::instance() to get instance of it.
-    CommandOptions() {
-        reset();
-    }
-
     /// \brief Initializes class members based on the command line.
     ///
     /// Reads each command line parameter and sets class member values.
@@ -373,8 +404,11 @@ private:
 
     /// \brief Validates initialized options.
     ///
+    /// It checks provided options. If there are issues they are reported
+    /// and exception is raised. If possible some options are corrected
+    /// e.g. overriding drop_time in case of avalanche scenario.
     /// \throws isc::InvalidParameter if command line validation fails.
-    void validate() const;
+    void validate();
 
     /// \brief Throws !InvalidParameter exception if condition is true.
     ///
@@ -488,113 +522,162 @@ private:
     /// IP protocol version to be used, expected values are:
     /// 4 for IPv4 and 6 for IPv6, default value 0 means "not set"
     uint8_t ipversion_;
+
     /// Packet exchange mode (e.g. DORA/SARR)
     ExchangeMode exchange_mode_;
+
     /// Lease Type to be obtained: address only, IPv6 prefix only.
     LeaseType lease_type_;
+
     /// Rate in exchange per second
-    int rate_;
+    unsigned int rate_;
+
     /// A rate at which DHCPv6 Renew messages are sent.
-    int renew_rate_;
+    unsigned int renew_rate_;
+
     /// A rate at which DHCPv6 Release messages are sent.
-    int release_rate_;
-    /// Delay between generation of two consecutive
-    /// performance reports
+    unsigned int release_rate_;
+
+    /// Delay between generation of two consecutive performance reports
     int report_delay_;
+
     /// Number of simulated clients (aka randomization range).
     uint32_t clients_num_;
+
     /// MAC address template used to generate unique MAC
     /// addresses for simulated clients.
     std::vector<uint8_t> mac_template_;
+
     /// DUID template used to generate unique DUIDs for
     /// simulated clients
     std::vector<uint8_t> duid_template_;
+
     /// Collection of base values specified with -b<value>
     /// options. Supported "bases" are mac=<mac> and duid=<duid>
     std::vector<std::string> base_;
+
+    /// Number of microseconds by which you should delay the exit
+    int exit_wait_time_;
+
     /// Number of 2 or 4-way exchanges to perform.
     std::vector<int> num_request_;
+
     /// Test period in seconds
     int period_;
+
     /// Indicates number of -d<value> parameters specified by user.
     /// If this value goes above 2, command line parsing fails.
     uint8_t drop_time_set_;
+
     /// Time to elapse before request is lost. The first value of
     /// two-element vector refers to DO/SA exchanges,
     /// second value refers to RA/RR. Default values are { 1, 1 }
     std::vector<double> drop_time_;
+
     /// Maximum number of drops request before aborting test.
     /// First value of two-element vector specifies maximum
     /// number of drops for DO/SA exchange, second value
     /// specifies maximum number of drops for RA/RR.
     std::vector<int> max_drop_;
+
     /// Maximal percentage of lost requests before aborting test.
     /// First value of two-element vector specifies percentage for
     /// DO/SA exchanges, second value for RA/RR.
     std::vector<double> max_pdrop_;
+
     /// Local address or interface specified with -l<value> option.
     std::string localname_;
+
     /// Indicates that specified value with -l<value> is
     /// rather interface (not address)
     bool is_interface_;
+
     /// Number of preload packets. Preload packets are used to
     /// initiate communication with server before doing performance
     /// measurements.
     int preload_;
-    /// Number of exchanges sent before next pause.
-    int aggressivity_;
+
     /// Local port number (host endian)
     int local_port_;
+
+    /// Remote port number (host endian)
+    int remote_port_;
+
     /// Randomization seed.
     uint32_t seed_;
+
     /// Indicates that randomization seed was provided.
     bool seeded_;
+
     /// Indicates that we use broadcast address.
     bool broadcast_;
+
     /// Indicates that we do rapid commit option.
     bool rapid_commit_;
+
     /// Indicates that we take server id from first received packet.
     bool use_first_;
+
     /// Packet template file names. These files store template packets
     /// that are used for initiating exchanges. Template packets
     /// read from files are later tuned with variable data.
     std::vector<std::string> template_file_;
+
     /// Location of a file containing a list of MAC addresses, one per line.
     /// This can be used if you don't want to generate MAC address from a
     /// base MAC address, but rather provide the file with a list of MAC
     /// addresses to be randomly picked. Note that in DHCPv6 those MAC
     /// addresses will be used to generate DUID-LL.
     std::string mac_list_file_;
+
     /// List of MAC addresses loaded from a file.
     std::vector<std::vector<uint8_t> > mac_list_;
+
     /// Offset of transaction id in template files. First vector
     /// element points to offset for DISCOVER/SOLICIT messages,
     /// second element points to transaction id offset for
     /// REQUEST messages
     std::vector<int> xid_offset_;
+
     /// Random value offset in templates. Random value offset
     /// points to last octet of DUID. Up to 4 last octets of
     /// DUID are randomized to simulate different clients.
     std::vector<int> rnd_offset_;
+
     /// Offset of elapsed time option in template packet.
     int elp_offset_;
+
     /// Offset of server id option in template packet.
     int sid_offset_;
+
     /// Offset of requested ip data in template packet
     int rip_offset_;
+
     /// String representing diagnostic selectors specified
     /// by user with -x<value>.
     std::string diags_;
+
     /// Command to be executed at the beginning/end of the test.
     /// This command is expected to expose start and stop argument.
     std::string wrapped_;
+
     /// Server name specified as last argument of command line.
     std::string server_name_;
+
     /// Indicates how many DHCPv6 relay agents are simulated.
     uint8_t v6_relay_encapsulation_level_;
+
+    /// @brief Extra options to be sent in each packet.
+    isc::dhcp::OptionCollection extra_opts_;
+
+    /// @brief Option to switch modes between single-threaded and multi-threaded.
+    bool single_thread_mode_;
+
+    /// @brief Selected performance scenario. Default is basic.
+    Scenario scenario_;
 };
 
-} // namespace perfdhcp
-} // namespace isc
+}  // namespace perfdhcp
+}  // namespace isc
 
 #endif // COMMAND_OPTIONS_H
