@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -19,6 +19,7 @@
 
 #include <boost/foreach.hpp>
 #include <boost/scoped_ptr.hpp>
+#include <boost/bind.hpp>
 
 #include <cstring>
 #include <errno.h>
@@ -28,11 +29,21 @@
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <string.h>
+#include <sys/ioctl.h>
 #include <sys/select.h>
 
+#ifndef FD_COPY
+#define FD_COPY(orig, copy) \
+    do { \
+        memmove(copy, orig, sizeof(fd_set)); \
+    } while (0)
+#endif
+
 using namespace std;
 using namespace isc::asiolink;
 using namespace isc::util;
+using namespace isc::util::thread;
+using namespace isc::util::io;
 using namespace isc::util::io::internal;
 
 namespace isc {
@@ -68,7 +79,7 @@ Iface::closeSockets() {
 
 void
 Iface::closeSockets(const uint16_t family) {
-    // Check that the correect 'family' value has been specified.
+    // Check that the correct 'family' value has been specified.
     // The possible values are AF_INET or AF_INET6. Note that, in
     // the current code they are used to differentiate that the
     // socket is used to transmit IPv4 or IPv6 traffic. However,
@@ -82,6 +93,7 @@ Iface::closeSockets(const uint16_t family) {
                   << " specified when requested to close all sockets"
                   << " which belong to this family");
     }
+
     // Search for the socket of the specific type.
     SocketCollection::iterator sock = sockets_.begin();
     while (sock != sockets_.end()) {
@@ -134,7 +146,9 @@ void Iface::setMac(const uint8_t* mac, size_t len) {
                   << MAX_MAC_LEN);
     }
     mac_len_ = len;
-    memcpy(mac_, mac, len);
+    if (len > 0) {
+        memcpy(mac_, mac, len);
+    }
 }
 
 bool Iface::delAddress(const isc::asiolink::IOAddress& addr) {
@@ -166,20 +180,25 @@ bool Iface::delSocket(const uint16_t sockfd) {
 }
 
 IfaceMgr::IfaceMgr()
-    :control_buf_len_(CMSG_SPACE(sizeof(struct in6_pktinfo))),
-     control_buf_(new char[control_buf_len_]),
-     packet_filter_(new PktFilterInet()),
+    :packet_filter_(new PktFilterInet()),
      packet_filter6_(new PktFilterInet6()),
-     test_mode_(false)
-{
+     test_mode_(false),
+     allow_loopback_(false) {
+
+    // Ensure that PQMs have been created to guarantee we have
+    // default packet queues in place.
+    try {
+        packet_queue_mgr4_.reset(new PacketQueueMgr4());
+        packet_queue_mgr6_.reset(new PacketQueueMgr6());
+    } catch (const std::exception& ex) {
+        isc_throw(Unexpected, "Failed to create PacketQueueManagers: " << ex.what());
+    }
 
     try {
+
         // required for sending/receiving packets
         // let's keep it in front, just in case someone
         // wants to send anything during initialization
-
-        // control_buf_ = boost::scoped_array<char>();
-
         detectIfaces();
 
     } catch (const std::exception& ex) {
@@ -194,7 +213,7 @@ void Iface::addUnicast(const isc::asiolink::IOAddress& addr) {
                       << " already defined on the " << name_ << " interface.");
         }
     }
-    unicasts_.push_back(OptionalValue<IOAddress>(addr, true));
+    unicasts_.push_back(Optional<IOAddress>(addr));
 }
 
 bool
@@ -225,7 +244,7 @@ Iface::hasAddress(const isc::asiolink::IOAddress& address) const {
 
 void
 Iface::addAddress(const isc::asiolink::IOAddress& addr) {
-    addrs_.push_back(Address(addr, OptionalValueState(true)));
+    addrs_.push_back(Address(addr));
 }
 
 void
@@ -233,7 +252,7 @@ Iface::setActive(const IOAddress& address, const bool active) {
     for (AddressCollection::iterator addr_it = addrs_.begin();
          addr_it != addrs_.end(); ++addr_it) {
         if (address == addr_it->get()) {
-            addr_it->specify(OptionalValueState(active));
+            addr_it->unspecified(!active);
             return;
         }
     }
@@ -245,7 +264,7 @@ void
 Iface::setActive(const bool active) {
     for (AddressCollection::iterator addr_it = addrs_.begin();
          addr_it != addrs_.end(); ++addr_it) {
-        addr_it->specify(OptionalValueState(active));
+        addr_it->unspecified(!active);
     }
 }
 
@@ -253,7 +272,7 @@ unsigned int
 Iface::countActive4() const {
     uint16_t count = 0;
     BOOST_FOREACH(Address addr, addrs_) {
-        if (addr.get().isV4() && addr.isSpecified()) {
+        if (!addr.unspecified() && addr.get().isV4()) {
             ++count;
         }
     }
@@ -261,22 +280,31 @@ Iface::countActive4() const {
 }
 
 void IfaceMgr::closeSockets() {
+    // Stops the receiver thread if there is one.
+    stopDHCPReceiver();
+
     BOOST_FOREACH(IfacePtr iface, ifaces_) {
         iface->closeSockets();
     }
 }
 
-void
-IfaceMgr::closeSockets(const uint16_t family) {
-    BOOST_FOREACH(IfacePtr iface, ifaces_) {
-        iface->closeSockets(family);
+void IfaceMgr::stopDHCPReceiver() {
+    if (isDHCPReceiverRunning()) {
+        dhcp_receiver_->stop();
+    }
+
+    dhcp_receiver_.reset();
+
+    if (getPacketQueue4()) {
+        getPacketQueue4()->clear();
+    }
+
+    if (getPacketQueue6()) {
+        getPacketQueue6()->clear();
     }
 }
 
 IfaceMgr::~IfaceMgr() {
-    // control_buf_ is deleted automatically (scoped_ptr)
-    control_buf_len_ = 0;
-
     closeSockets();
 }
 
@@ -318,6 +346,23 @@ IfaceMgr::deleteExternalSocket(int socketfd) {
     }
 }
 
+int
+IfaceMgr::purgeBadSockets() {
+    std::vector<int> bad_fds;
+    for (SocketCallbackInfo s : callbacks_) {
+        errno = 0;
+        if (fcntl(s.socket_, F_GETFD) < 0 && (errno == EBADF)) {
+            bad_fds.push_back(s.socket_);
+        }
+    }
+
+    for (auto bad_fd : bad_fds) {
+        deleteExternalSocket(bad_fd);
+    }
+
+    return (bad_fds.size());
+}
+
 void
 IfaceMgr::deleteAllExternalSockets() {
     callbacks_.clear();
@@ -432,7 +477,7 @@ void IfaceMgr::stubDetectIfaces() {
     iface->flag_running_ = true;
 
     // Note that we claim that this is not a loopback. iface_mgr tries to open a
-    // socket on all interaces that are up, running and not loopback. As this is
+    // socket on all interfaces that are up, running and not loopback. As this is
     // the only interface we were able to detect, let's pretend this is a normal
     // interface.
     iface->flag_loopback_ = false;
@@ -463,7 +508,9 @@ IfaceMgr::openSockets4(const uint16_t port, const bool use_bcast,
             // that the interface configuration is valid and that the interface
             // is not a loopback interface. In both cases, we want to report
             // that the socket will not be opened.
-            if (iface->flag_loopback_) {
+            // Relax the check when the loopback interface was explicitely
+            // allowed
+            if (iface->flag_loopback_ && !allow_loopback_) {
                 IFACEMGR_ERROR(SocketConfigError, error_handler,
                                "must not open socket on the loopback"
                                " interface " << iface->getName());
@@ -471,20 +518,32 @@ IfaceMgr::openSockets4(const uint16_t port, const bool use_bcast,
 
             }
 
-            IOAddress out_address("0.0.0.0");
-            if (!iface->flag_up_ || !iface->flag_running_ ||
-                !iface->getAddress4(out_address)) {
+            if (!iface->flag_up_) {
                 IFACEMGR_ERROR(SocketConfigError, error_handler,
                                "the interface " << iface->getName()
-                               << " is down or has no usable IPv4"
-                               " addresses configured");
+                               << " is down");
+                continue;
+            }
+
+            if (!iface->flag_running_) {
+                IFACEMGR_ERROR(SocketConfigError, error_handler,
+                               "the interface " << iface->getName()
+                               << " is not running");
+                continue;
+            }
+
+            IOAddress out_address("0.0.0.0");
+            if (!iface->getAddress4(out_address)) {
+                IFACEMGR_ERROR(SocketConfigError, error_handler,
+                               "the interface " << iface->getName()
+                               << " has no usable IPv4 addresses configured");
                 continue;
             }
         }
 
         BOOST_FOREACH(Iface::Address addr, iface->getAddresses()) {
             // Skip non-IPv4 addresses and those that weren't selected..
-            if (!addr.get().isV4() || !addr.isSpecified()) {
+            if (addr.unspecified() || !addr.get().isV4()) {
                 continue;
             }
 
@@ -550,6 +609,13 @@ IfaceMgr::openSockets4(const uint16_t port, const bool use_bcast,
 
         }
     }
+
+    // If we have open sockets, start the receiver.
+    if (count > 0) {
+        // starts the receiver thread (if queueing is enabled).
+        startDHCPReceiver(AF_INET);
+    }
+
     return (count > 0);
 }
 
@@ -568,17 +634,23 @@ IfaceMgr::openSockets6(const uint16_t port,
             // that the interface configuration is valid and that the interface
             // is not a loopback interface. In both cases, we want to report
             // that the socket will not be opened.
-            if (iface->flag_loopback_) {
+            // Relax the check when the loopback interface was explicitely
+            // allowed
+            if (iface->flag_loopback_ && !allow_loopback_) {
                 IFACEMGR_ERROR(SocketConfigError, error_handler,
                                "must not open socket on the loopback"
                                " interface " << iface->getName());
                 continue;
 
-            } else if (!iface->flag_up_ || !iface->flag_running_) {
+            } else if (!iface->flag_up_) {
                 IFACEMGR_ERROR(SocketConfigError, error_handler,
                                "the interface " << iface->getName()
-                               << " is down or has no usable IPv6"
-                               " addresses configured");
+                               << " is down");
+                continue;
+            } else if (!iface->flag_running_) {
+                IFACEMGR_ERROR(SocketConfigError, error_handler,
+                               "the interface " << iface->getName()
+                               << " is not running");
                 continue;
             }
 
@@ -626,9 +698,49 @@ IfaceMgr::openSockets6(const uint16_t port,
 
         }
     }
+
+    // If we have open sockets, start the receiver.
+    if (count > 0) {
+        // starts the receiver thread (if queueing is enabled).
+        startDHCPReceiver(AF_INET6);
+    }
     return (count > 0);
 }
 
+void
+IfaceMgr::startDHCPReceiver(const uint16_t family) {
+    if (isDHCPReceiverRunning()) {
+        isc_throw(InvalidOperation, "a receiver thread already exists");
+    }
+
+    switch (family) {
+    case AF_INET:
+        // If the queue doesn't exist, packet queing has been configured
+        // as disabled. If there is no queue, we do not create a reciever.
+        if(!getPacketQueue4()) {
+            return;
+        }
+
+        dhcp_receiver_.reset(new WatchedThread());
+        dhcp_receiver_->start(boost::bind(&IfaceMgr::receiveDHCP4Packets, this));
+
+        break;
+    case AF_INET6:
+        // If the queue doesn't exist, packet queing has been configured
+        // as disabled. If there is no queue, we do not create a reciever.
+        if(!getPacketQueue6()) {
+            return;
+        }
+
+        dhcp_receiver_.reset(new WatchedThread());
+        dhcp_receiver_->start(boost::bind(&IfaceMgr::receiveDHCP6Packets, this));
+        break;
+    default:
+        isc_throw (BadValue, "startDHCPReceiver: invalid family: " << family);
+        break;
+    }
+}
+
 void
 IfaceMgr::printIfaces(std::ostream& out /*= std::cout*/) {
     BOOST_FOREACH(IfacePtr iface, ifaces_) {
@@ -863,7 +975,8 @@ IfaceMgr::send(const Pkt6Ptr& pkt) {
     }
 
     // Assuming that packet filter is not NULL, because its modifier checks it.
-    return (packet_filter6_->send(*iface, getSocket(*pkt), pkt));
+    // The packet filter returns an int but in fact it either returns 0 or throws.
+    return (packet_filter6_->send(*iface, getSocket(*pkt), pkt) == 0);
 }
 
 bool
@@ -876,11 +989,124 @@ IfaceMgr::send(const Pkt4Ptr& pkt) {
     }
 
     // Assuming that packet filter is not NULL, because its modifier checks it.
-    return (packet_filter_->send(*iface, getSocket(*pkt).sockfd_, pkt));
+    // The packet filter returns an int but in fact it either returns 0 or throws.
+    return (packet_filter_->send(*iface, getSocket(*pkt).sockfd_, pkt) == 0);
 }
 
-
 Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
+    if (isDHCPReceiverRunning()) {
+        return (receive4Indirect(timeout_sec, timeout_usec));
+    }
+
+    return (receive4Direct(timeout_sec, timeout_usec));
+}
+
+Pkt4Ptr IfaceMgr::receive4Indirect(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
+    // Sanity check for microsecond timeout.
+    if (timeout_usec >= 1000000) {
+        isc_throw(BadValue, "fractional timeout must be shorter than"
+                  " one million microseconds");
+    }
+
+    fd_set sockets;
+    int maxfd = 0;
+
+    FD_ZERO(&sockets);
+
+    // if there are any callbacks for external sockets registered...
+    if (!callbacks_.empty()) {
+        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+            addFDtoSet(s.socket_, maxfd, &sockets);
+        }
+    }
+
+    // Add Receiver ready watch socket
+    addFDtoSet(dhcp_receiver_->getWatchFd(WatchedThread::READY), maxfd, &sockets);
+
+    // Add Receiver error watch socket
+    addFDtoSet(dhcp_receiver_->getWatchFd(WatchedThread::ERROR), maxfd, &sockets);
+
+    // Set timeout for our next select() call.  If there are
+    // no DHCP packets to read, then we'll wait for a finite
+    // amount of time for an IO event.  Otherwise, we'll
+    // poll (timeout = 0 secs).  We need to poll, even if
+    // DHCP packets are waiting so we don't starve external
+    // sockets under heavy DHCP load.
+    struct timeval select_timeout;
+    if (getPacketQueue4()->empty()) {
+        select_timeout.tv_sec = timeout_sec;
+        select_timeout.tv_usec = timeout_usec;
+    } else {
+        select_timeout.tv_sec = 0;
+        select_timeout.tv_usec = 0;
+    }
+
+    // zero out the errno to be safe
+    errno = 0;
+
+    int result = select(maxfd + 1, &sockets, NULL, NULL, &select_timeout);
+
+    if ((result == 0) && getPacketQueue4()->empty()) {
+        // nothing received and timeout has been reached
+        return (Pkt4Ptr());
+    } else if (result < 0) {
+        // In most cases we would like to know whether select() returned
+        // an error because of a signal being received  or for some other
+        // reason. This is because DHCP servers use signals to trigger
+        // certain actions, like reconfiguration or graceful shutdown.
+        // By catching a dedicated exception the caller will know if the
+        // error returned by the function is due to the reception of the
+        // signal or for some other reason.
+        if (errno == EINTR) {
+            isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
+        } else {
+            isc_throw(SocketReadError, strerror(errno));
+        }
+    }
+
+    // We only check external sockets if select detected an event.
+    if (result > 0) {
+        // Check for receiver thread read errors.
+        if (dhcp_receiver_->isReady(WatchedThread::ERROR)) {
+            string msg = dhcp_receiver_->getLastError();
+            dhcp_receiver_->clearReady(WatchedThread::ERROR);
+            isc_throw(SocketReadError, msg);
+        }
+
+        // Let's find out which external socket has the data
+        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+            if (!FD_ISSET(s.socket_, &sockets)) {
+                continue;
+            }
+
+            // something received over external socket
+
+            // Calling the external socket's callback provides its service
+            // layer access without integrating any specific features
+            // in IfaceMgr
+            if (s.callback_) {
+                s.callback_(s.socket_);
+            }
+
+            return (Pkt4Ptr());
+        }
+    }
+
+    // If we're here it should only be because there are DHCP packets waiting.
+    Pkt4Ptr pkt = getPacketQueue4()->dequeuePacket();
+    if (!pkt) {
+        dhcp_receiver_->clearReady(WatchedThread::READY);
+    }
+
+    return (pkt);
+}
+
+Pkt4Ptr IfaceMgr::receive4Direct(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
     // Sanity check for microsecond timeout.
     if (timeout_usec >= 1000000) {
         isc_throw(BadValue, "fractional timeout must be shorter than"
@@ -898,15 +1124,10 @@ Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     /// provided set to indicated which sockets have something to read.
     BOOST_FOREACH(iface, ifaces_) {
         BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
-
             // Only deal with IPv4 addresses.
             if (s.addr_.isV4()) {
-
                 // Add this socket to listening set
-                FD_SET(s.sockfd_, &sockets);
-                if (maxfd < s.sockfd_) {
-                    maxfd = s.sockfd_;
-                }
+                addFDtoSet(s.sockfd_, maxfd, &sockets);
             }
         }
     }
@@ -914,10 +1135,8 @@ Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     // if there are any callbacks for external sockets registered...
     if (!callbacks_.empty()) {
         BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-            FD_SET(s.socket_, &sockets);
-            if (maxfd < s.socket_) {
-                maxfd = s.socket_;
-            }
+            // Add this socket to listening set
+            addFDtoSet(s.socket_, maxfd, &sockets);
         }
     }
 
@@ -937,13 +1156,18 @@ Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     } else if (result < 0) {
         // In most cases we would like to know whether select() returned
         // an error because of a signal being received  or for some other
-        // reasaon. This is because DHCP servers use signals to trigger
+        // reason. This is because DHCP servers use signals to trigger
         // certain actions, like reconfiguration or graceful shutdown.
         // By catching a dedicated exception the caller will know if the
         // error returned by the function is due to the reception of the
         // signal or for some other reason.
         if (errno == EINTR) {
             isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
         } else {
             isc_throw(SocketReadError, strerror(errno));
         }
@@ -961,7 +1185,7 @@ Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
         // layer access without integrating any specific features
         // in IfaceMgr
         if (s.callback_) {
-            s.callback_();
+            s.callback_(s.socket_);
         }
 
         return (Pkt4Ptr());
@@ -989,7 +1213,29 @@ Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     return (packet_filter_->receive(*iface, *candidate));
 }
 
-Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */ ) {
+Pkt6Ptr
+IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
+    if (isDHCPReceiverRunning()) {
+        return (receive6Indirect(timeout_sec, timeout_usec));
+    }
+
+    return (receive6Direct(timeout_sec, timeout_usec));
+}
+
+void
+IfaceMgr::addFDtoSet(int fd, int& maxfd, fd_set* sockets) {
+    if (!sockets) {
+        isc_throw(BadValue, "addFDtoSet: sockets can't be null");
+    }
+
+    FD_SET(fd, sockets);
+    if (maxfd < fd) {
+        maxfd = fd;
+    }
+}
+
+Pkt6Ptr
+IfaceMgr::receive6Direct(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */ ) {
     // Sanity check for microsecond timeout.
     if (timeout_usec >= 1000000) {
         isc_throw(BadValue, "fractional timeout must be shorter than"
@@ -1006,16 +1252,11 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     /// and then use its copy for select(). Please note that select() modifies
     /// provided set to indicated which sockets have something to read.
     BOOST_FOREACH(IfacePtr iface, ifaces_) {
-
         BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
             // Only deal with IPv6 addresses.
             if (s.addr_.isV6()) {
-
                 // Add this socket to listening set
-                FD_SET(s.sockfd_, &sockets);
-                if (maxfd < s.sockfd_) {
-                    maxfd = s.sockfd_;
-                }
+                addFDtoSet(s.sockfd_, maxfd, &sockets);
             }
         }
     }
@@ -1024,10 +1265,7 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     if (!callbacks_.empty()) {
         BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
             // Add it to the set as well
-            FD_SET(s.socket_, &sockets);
-            if (maxfd < s.socket_) {
-                maxfd = s.socket_;
-            }
+            addFDtoSet(s.socket_, maxfd, &sockets);
         }
     }
 
@@ -1047,13 +1285,18 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     } else if (result < 0) {
         // In most cases we would like to know whether select() returned
         // an error because of a signal being received  or for some other
-        // reasaon. This is because DHCP servers use signals to trigger
+        // reason. This is because DHCP servers use signals to trigger
         // certain actions, like reconfiguration or graceful shutdown.
         // By catching a dedicated exception the caller will know if the
         // error returned by the function is due to the reception of the
         // signal or for some other reason.
         if (errno == EINTR) {
             isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
         } else {
             isc_throw(SocketReadError, strerror(errno));
         }
@@ -1071,7 +1314,7 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
         // layer access without integrating any specific features
         // in IfaceMgr
         if (s.callback_) {
-            s.callback_();
+            s.callback_(s.socket_);
         }
 
         return (Pkt6Ptr());
@@ -1097,7 +1340,325 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     return (packet_filter6_->receive(*candidate));
 }
 
-uint16_t IfaceMgr::getSocket(const isc::dhcp::Pkt6& pkt) {
+Pkt6Ptr
+IfaceMgr::receive6Indirect(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */ ) {
+    // Sanity check for microsecond timeout.
+    if (timeout_usec >= 1000000) {
+        isc_throw(BadValue, "fractional timeout must be shorter than"
+                  " one million microseconds");
+    }
+
+    fd_set sockets;
+    int maxfd = 0;
+
+    FD_ZERO(&sockets);
+
+    // if there are any callbacks for external sockets registered...
+    if (!callbacks_.empty()) {
+        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+            // Add it to the set as well
+            addFDtoSet(s.socket_, maxfd, &sockets);
+        }
+    }
+
+    // Add Receiver ready watch socket
+    addFDtoSet(dhcp_receiver_->getWatchFd(WatchedThread::READY), maxfd, &sockets);
+
+    // Add Receiver error watch socket
+    addFDtoSet(dhcp_receiver_->getWatchFd(WatchedThread::ERROR), maxfd, &sockets);
+
+    // Set timeout for our next select() call.  If there are
+    // no DHCP packets to read, then we'll wait for a finite
+    // amount of time for an IO event.  Otherwise, we'll
+    // poll (timeout = 0 secs).  We need to poll, even if
+    // DHCP packets are waiting so we don't starve external
+    // sockets under heavy DHCP load.
+    struct timeval select_timeout;
+    if (getPacketQueue6()->empty()) {
+        select_timeout.tv_sec = timeout_sec;
+        select_timeout.tv_usec = timeout_usec;
+    } else {
+        select_timeout.tv_sec = 0;
+        select_timeout.tv_usec = 0;
+    }
+
+    // zero out the errno to be safe
+    errno = 0;
+
+    int result = select(maxfd + 1, &sockets, NULL, NULL, &select_timeout);
+
+    if ((result == 0) && getPacketQueue6()->empty()) {
+        // nothing received and timeout has been reached
+        return (Pkt6Ptr());
+    } else if (result < 0) {
+        // In most cases we would like to know whether select() returned
+        // an error because of a signal being received  or for some other
+        // reason. This is because DHCP servers use signals to trigger
+        // certain actions, like reconfiguration or graceful shutdown.
+        // By catching a dedicated exception the caller will know if the
+        // error returned by the function is due to the reception of the
+        // signal or for some other reason.
+        if (errno == EINTR) {
+            isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
+        } else {
+            isc_throw(SocketReadError, strerror(errno));
+        }
+    }
+
+    // We only check external sockets if select detected an event.
+    if (result > 0) {
+        // Check for receiver thread read errors.
+        if (dhcp_receiver_->isReady(WatchedThread::ERROR)) {
+            string msg = dhcp_receiver_->getLastError();
+            dhcp_receiver_->clearReady(WatchedThread::ERROR);
+            isc_throw(SocketReadError, msg);
+        }
+
+        // Let's find out which external socket has the data
+        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+            if (!FD_ISSET(s.socket_, &sockets)) {
+                continue;
+            }
+
+            // something received over external socket
+
+            // Calling the external socket's callback provides its service
+            // layer access without integrating any specific features
+            // in IfaceMgr
+            if (s.callback_) {
+                s.callback_(s.socket_);
+            }
+
+            return (Pkt6Ptr());
+        }
+    }
+
+    // If we're here it should only be because there are DHCP packets waiting.
+    Pkt6Ptr pkt = getPacketQueue6()->dequeuePacket();
+    if (!pkt) {
+        dhcp_receiver_->clearReady(WatchedThread::READY);
+    }
+
+    return (pkt);
+}
+
+void
+IfaceMgr::receiveDHCP4Packets() {
+    IfacePtr iface;
+    fd_set sockets;
+    int maxfd = 0;
+
+    FD_ZERO(&sockets);
+
+    // Add terminate watch socket.
+    addFDtoSet(dhcp_receiver_->getWatchFd(WatchedThread::TERMINATE), maxfd, &sockets);
+
+    // Add Interface sockets.
+    BOOST_FOREACH(iface, ifaces_) {
+        BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
+            // Only deal with IPv4 addresses.
+            if (s.addr_.isV4()) {
+                // Add this socket to listening set.
+                addFDtoSet(s.sockfd_, maxfd, &sockets);
+            }
+        }
+    }
+
+    for (;;) {
+        // Check the watch socket.
+        if (dhcp_receiver_->shouldTerminate()) {
+            return;
+        }
+
+        fd_set rd_set;
+        FD_COPY(&sockets, &rd_set);
+
+        // zero out the errno to be safe.
+        errno = 0;
+
+        // Select with null timeouts to wait indefinetly an event
+        int result = select(maxfd + 1, &rd_set, 0, 0, 0);
+
+        // Re-check the watch socket.
+        if (dhcp_receiver_->shouldTerminate()) {
+            return;
+        }
+
+        if (result == 0) {
+            // nothing received?
+            continue;
+
+        } else if (result < 0) {
+            // This thread should not get signals?
+            if (errno != EINTR) {
+                // Signal the error to receive4.
+                dhcp_receiver_->setError(strerror(errno));
+                // We need to sleep in case of the error condition to
+                // prevent the thread from tight looping when result
+                // gets negative.
+                sleep(1);
+            }
+            continue;
+        }
+
+        // Let's find out which interface/socket has data.
+        BOOST_FOREACH(iface, ifaces_) {
+            BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
+                if (FD_ISSET(s.sockfd_, &sockets)) {
+                    receiveDHCP4Packet(*iface, s);
+                    // Can take time so check one more time the watch socket.
+                    if (dhcp_receiver_->shouldTerminate()) {
+                        return;
+                    }
+                }
+            }
+        }
+    }
+
+}
+
+void
+IfaceMgr::receiveDHCP6Packets() {
+    IfacePtr iface;
+    fd_set sockets;
+    int maxfd = 0;
+
+    FD_ZERO(&sockets);
+
+    // Add terminate watch socket.
+    addFDtoSet(dhcp_receiver_->getWatchFd(WatchedThread::TERMINATE), maxfd, &sockets);
+
+    // Add Interface sockets.
+    BOOST_FOREACH(iface, ifaces_) {
+        BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
+            // Only deal with IPv6 addresses.
+            if (s.addr_.isV6()) {
+                // Add this socket to listening set.
+                addFDtoSet(s.sockfd_ , maxfd, &sockets);
+            }
+        }
+    }
+
+    for (;;) {
+        // Check the watch socket.
+        if (dhcp_receiver_->shouldTerminate()) {
+            return;
+        }
+
+        fd_set rd_set;
+        FD_COPY(&sockets, &rd_set);
+
+        // zero out the errno to be safe.
+        errno = 0;
+
+        // Note we wait until something happen.
+        int result = select(maxfd + 1, &rd_set, 0, 0, 0);
+
+        // Re-check the watch socket.
+        if (dhcp_receiver_->shouldTerminate()) {
+            return;
+        }
+
+        if (result == 0) {
+            // nothing received?
+            continue;
+        } else if (result < 0) {
+            // This thread should not get signals?
+            if (errno != EINTR) {
+                // Signal the error to receive6.
+                dhcp_receiver_->setError(strerror(errno));
+                // We need to sleep in case of the error condition to
+                // prevent the thread from tight looping when result
+                // gets negative.
+                sleep(1);
+            }
+            continue;
+        }
+
+        // Let's find out which interface/socket has data.
+        BOOST_FOREACH(iface, ifaces_) {
+            BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
+                if (FD_ISSET(s.sockfd_, &sockets)) {
+                    receiveDHCP6Packet(s);
+                    // Can take time so check one more time the watch socket.
+                    if (dhcp_receiver_->shouldTerminate()) {
+                        return;
+                    }
+                }
+            }
+        }
+    }
+}
+
+void
+IfaceMgr::receiveDHCP4Packet(Iface& iface, const SocketInfo& socket_info) {
+    int len;
+
+    int result = ioctl(socket_info.sockfd_, FIONREAD, &len);
+    if (result < 0) {
+        // Signal the error to receive4.
+        dhcp_receiver_->setError(strerror(errno));
+        return;
+    }
+    if (len == 0) {
+        // Nothing to read.
+        return;
+    }
+
+    Pkt4Ptr pkt;
+
+    try {
+        pkt = packet_filter_->receive(iface, socket_info);
+    } catch (const std::exception& ex) {
+        dhcp_receiver_->setError(strerror(errno));
+    } catch (...) {
+        dhcp_receiver_->setError("packet filter receive() failed");
+    }
+
+    if (pkt) {
+        getPacketQueue4()->enqueuePacket(pkt, socket_info);
+        dhcp_receiver_->markReady(WatchedThread::READY);
+    }
+}
+
+void
+IfaceMgr::receiveDHCP6Packet(const SocketInfo& socket_info) {
+    int len;
+
+    int result = ioctl(socket_info.sockfd_, FIONREAD, &len);
+    if (result < 0) {
+        // Signal the error to receive6.
+        dhcp_receiver_->setError(strerror(errno));
+        return;
+    }
+    if (len == 0) {
+        // Nothing to read.
+        return;
+    }
+
+    Pkt6Ptr pkt;
+
+    try {
+        pkt = packet_filter6_->receive(socket_info);
+    } catch (const std::exception& ex) {
+        dhcp_receiver_->setError(ex.what());
+    } catch (...) {
+        dhcp_receiver_->setError("packet filter receive() failed");
+    }
+
+    if (pkt) {
+        getPacketQueue6()->enqueuePacket(pkt, socket_info);
+        dhcp_receiver_->markReady(WatchedThread::READY);
+    }
+}
+
+uint16_t
+IfaceMgr::getSocket(const isc::dhcp::Pkt6& pkt) {
     IfacePtr iface = getIface(pkt.getIface());
     if (!iface) {
         isc_throw(IfaceNotFound, "Tried to find socket for non-existent interface");
@@ -1161,7 +1722,7 @@ IfaceMgr::getSocket(isc::dhcp::Pkt4 const& pkt) {
     }
 
     const Iface::SocketCollection& socket_collection = iface->getSockets();
-    // A candidate being an end of the iterator marks that it is a begining of
+    // A candidate being an end of the iterator marks that it is a beginning of
     // the socket search and that the candidate needs to be set to the first
     // socket found.
     Iface::SocketCollection::const_iterator candidate = socket_collection.end();
@@ -1186,5 +1747,42 @@ IfaceMgr::getSocket(isc::dhcp::Pkt4 const& pkt) {
     return (*candidate);
 }
 
+bool
+IfaceMgr::configureDHCPPacketQueue(uint16_t family, data::ConstElementPtr queue_control) {
+    if (isDHCPReceiverRunning()) {
+        isc_throw(InvalidOperation, "Cannot reconfigure queueing"
+                                    " while DHCP receiver thread is running");
+    }
+
+    bool enable_queue = false;
+    if (queue_control) {
+        try {
+            enable_queue = data::SimpleParser::getBoolean(queue_control, "enable-queue");
+        } catch (...) {
+            // @todo - for now swallow not found errors.
+            // if not present we assume default
+        }
+    }
+
+    if (enable_queue) {
+        // Try to create the queue as configured.
+        if (family == AF_INET) {
+            packet_queue_mgr4_->createPacketQueue(queue_control);
+        } else {
+            packet_queue_mgr6_->createPacketQueue(queue_control);
+        }
+    } else {
+        // Destroy the current queue (if one), this inherently disables threading.
+        if (family == AF_INET) {
+            packet_queue_mgr4_->destroyPacketQueue();
+        } else {
+            packet_queue_mgr6_->destroyPacketQueue();
+        }
+    }
+
+    return(enable_queue);
+}
+
+
 } // end of namespace isc::dhcp
 } // end of namespace isc
