@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,6 +9,7 @@
 
 #include <dhcp/option_definition.h>
 #include <dhcp/option_space_container.h>
+#include <dhcp/pkt4.h>
 #include <dhcp/pkt6.h>
 #include <util/buffer.h>
 #include <util/staged_value.h>
@@ -27,38 +28,35 @@ public:
     /// Map of factory functions.
     typedef std::map<unsigned short, Option::Factory*>  FactoryMap;
 
-    /// @brief Return collection of option definitions.
+    /// @brief Returns collection of option definitions.
     ///
-    /// Method returns the collection of DHCP standard DHCP
-    /// option definitions.
-    /// @todo DHCPv4 option definitions are not implemented. For now
-    /// this function will throw isc::NotImplemented in case of attempt
-    /// to get option definitions for V4 universe.
+    /// This method returns a collection of option definitions for a specified
+    /// option space.
     ///
-    /// @param u universe of the options (V4 or V6).
+    /// @param space Option space.
     ///
     /// @return Pointer to a collection of option definitions.
-    static const OptionDefContainerPtr& getOptionDefs(const Option::Universe u);
+    static const OptionDefContainerPtr& getOptionDefs(const std::string& space);
 
     /// @brief Return the first option definition matching a
     /// particular option code.
     ///
-    /// @param u universe (V4 or V6)
+    /// @param space option space.
     /// @param code option code.
     ///
     /// @return reference to an option definition being requested
     /// or NULL pointer if option definition has not been found.
-    static OptionDefinitionPtr getOptionDef(const Option::Universe u,
+    static OptionDefinitionPtr getOptionDef(const std::string& space,
                                             const uint16_t code);
 
     /// @brief Return the definition of option having a specified name.
     ///
-    /// @param u universe (v4 or V6)
+    /// @param space option space.
     /// @param name Option name.
     ///
     /// @return Pointer to the option definition or NULL pointer if option
     /// definition has not been found.
-    static OptionDefinitionPtr getOptionDef(const Option::Universe u,
+    static OptionDefinitionPtr getOptionDef(const std::string& space,
                                             const std::string& name);
 
     /// @brief Returns vendor option definition for a given vendor-id and code
@@ -115,20 +113,43 @@ public:
     static OptionDefContainerPtr
     getRuntimeOptionDefs(const std::string& space);
 
-    /// @brief Check if the specified option is a standard option.
+    /// @brief Returns last resort option definition by space and option code.
     ///
-    /// @param u universe (V4 or V6)
-    /// @param code option code.
+    /// @param space Option space name.
+    /// @param code Option code.
     ///
-    /// @return true if the specified option is a standard option.
-    /// @todo We already create option definitions for the subset if
-    /// standard options. We are aiming that this function checks
-    /// the presence of the standard option definition and if it finds
-    /// it, then the true value is returned. However, at this point
-    /// this is not doable because some of the definitions (for less
-    /// important options) are not created yet.
-    static bool isStandardOption(const Option::Universe u,
-                                 const uint16_t code);
+    /// @return Pointer to option definition or NULL if it doesn't exist.
+    static OptionDefinitionPtr getLastResortOptionDef(const std::string& space,
+                                                      const uint16_t code);
+
+    /// @brief Returns last resort option definition by space and option name.
+    ///
+    /// @param space Option space name.
+    /// @param name Option name.
+    ///
+    /// @return Pointer to option definition or NULL if it doesn't exist.
+    static OptionDefinitionPtr getLastResortOptionDef(const std::string& space,
+                                                      const std::string& name);
+
+    /// @brief Returns last resort option definitions for specified option
+    /// space name.
+    ///
+    /// @param space Option space name.
+    ///
+    /// @return Pointer to the container holding option definitions or NULL.
+    static OptionDefContainerPtr
+    getLastResortOptionDefs(const std::string& space);
+
+    /// @brief Checks if an option unpacking has to be deferred.
+    ///
+    /// DHCPv4 option 43 and 224-254 unpacking is done after classification.
+    ///
+    /// @param space Option space name.
+    /// @param code Option code.
+    ///
+    /// @return True if option processing should be deferred.
+    static bool shouldDeferOptionUnpack(const std::string& space,
+                                        const uint16_t code);
 
     /// @brief Factory function to create instance of option.
     ///
@@ -157,13 +178,22 @@ public:
     /// may be different reasons (option too large, option malformed,
     /// too many options etc.)
     ///
-    /// This is v4 specific version, which stores Relay Agent Information
-    /// option and END options last.
+    /// This is v4 specific version, which stores DHCP message type first,
+    /// and the Relay Agent Information option and END options last. This
+    /// function is initially called to pack the options for a packet in
+    /// @ref Pkt4::pack(). That call leads to it being called recursively in
+    /// @ref Option::packOptions(). Thus the logic used to output the
+    /// message type should only be executed by the top-most. This is governed
+    /// by the paramater top, below.
     ///
     /// @param buf output buffer (assembled options will be stored here)
     /// @param options collection of options to store to
+    /// @param top indicates if this is the first call to pack the options.
+    /// When true logic to emit the message type first is executed. It
+    /// defaults to false.
     static void packOptions4(isc::util::OutputBuffer& buf,
-                             const isc::dhcp::OptionCollection& options);
+                             const isc::dhcp::OptionCollection& options,
+                             bool top = false);
 
     /// @brief Stores DHCPv6 options in a buffer.
     ///
@@ -228,13 +258,19 @@ public:
     ///        to be used to parse options in the packets.
     /// @param options Reference to option container. Options will be
     ///        put here.
+    /// @param deferred Reference to an option code list. Options which
+    ///        processing is deferred will be put here.
+    /// @param flexible_pad_end Parse options 0 and 255 as PAD and END
+    ///        when they are not defined in the option space.
     /// @return offset to the first byte after the last successfully
     /// parsed option or the offset of the DHO_END option type.
     ///
     /// The unpackOptions6 note applies too.
     static size_t unpackOptions4(const OptionBuffer& buf,
                                  const std::string& option_space,
-                                 isc::dhcp::OptionCollection& options);
+                                 isc::dhcp::OptionCollection& options,
+                                 std::list<uint16_t>& deferred,
+                                 bool flexible_pad_end = false);
 
     /// Registers factory method that produces options of specific option types.
     ///
@@ -316,7 +352,7 @@ public:
     /// @brief Removes runtime option definitions.
     static void clearRuntimeOptionDefs();
 
-    /// @brief Reverts uncommited changes to runtime option definitions.
+    /// @brief Reverts uncommitted changes to runtime option definitions.
     static void revertRuntimeOptionDefs();
 
     /// @brief Commits runtime option definitions.
@@ -358,8 +394,13 @@ private:
     /// is incorrect. This is a programming error.
     static void initStdOptionDefs6();
 
+    /// Initialize last resort DHCPv4 option definitions.
+    static void initLastResortOptionDefs();
+
+    /// Initialize DOCSIS DHCPv4 option definitions.
     static void initVendorOptsDocsis4();
 
+    /// Initialize DOCSIS DHCPv6 option definitions.
     static void initVendorOptsDocsis6();
 
     /// Initialize private DHCPv6 option definitions.
@@ -377,13 +418,19 @@ private:
     /// Container with DHCPv6 option definitions.
     static OptionDefContainerPtr v6option_defs_;
 
+    /// Container that holds option definitions for various option spaces.
+    static OptionDefContainers option_defs_;
+
     /// Container for v4 vendor option definitions
     static VendorOptionDefContainers vendor4_defs_;
 
     /// Container for v6 vendor option definitions
     static VendorOptionDefContainers vendor6_defs_;
 
-    /// Container for additional option defnitions created in runtime.
+    /// Container with DHCPv4 last resort option definitions.
+    static OptionDefContainerPtr lastresort_defs_;
+
+    /// Container for additional option definitions created in runtime.
     static util::StagedValue<OptionDefSpaceContainer> runtime_option_defs_;
 };
 
