@@ -1,17 +1,22 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+#include <dhcp/dhcp4.h>
 #include <dhcp/libdhcp++.h>
 #include <dhcp/option.h>
+#include <dhcp/option_space.h>
 #include <exceptions/exceptions.h>
 #include <util/encode/hex.h>
 #include <util/io_utilities.h>
 
+#include <boost/make_shared.hpp>
+
 #include <iomanip>
+#include <list>
 #include <sstream>
 
 #include <arpa/inet.h>
@@ -34,12 +39,7 @@ Option::factory(Option::Universe u,
 
 Option::Option(Universe u, uint16_t type)
     :universe_(u), type_(type) {
-
-    // END option (type 255 is forbidden as well)
-    if ((u == V4) && ((type == 0) || (type > 254))) {
-        isc_throw(BadValue, "Can't create V4 option of type "
-                  << type << ", V4 options are in range 1..254");
-    }
+    check();
 }
 
 Option::Option(Universe u, uint16_t type, const OptionBuffer& data)
@@ -60,6 +60,16 @@ Option::Option(const Option& option)
     option.getOptionsCopy(options_);
 }
 
+OptionPtr
+Option::create(Universe u, uint16_t type) {
+    return (boost::make_shared<Option>(u, type));
+}
+
+OptionPtr
+Option::create(Universe u, uint16_t type, const OptionBuffer& data) {
+    return (boost::make_shared<Option>(u, type, data));
+}
+
 Option&
 Option::operator=(const Option& rhs) {
     if (&rhs != this) {
@@ -153,9 +163,12 @@ void Option::unpack(OptionBufferConstIter begin,
 
 void
 Option::unpackOptions(const OptionBuffer& buf) {
+    list<uint16_t> deferred;
     switch (universe_) {
     case V4:
-        LibDHCP::unpackOptions4(buf, getEncapsulatedSpace(), options_);
+        LibDHCP::unpackOptions4(buf, getEncapsulatedSpace(),
+                                options_, deferred,
+                                getType() == DHO_VENDOR_ENCAPSULATED_OPTIONS);
         return;
     case V6:
         LibDHCP::unpackOptions6(buf, getEncapsulatedSpace(), options_);
