@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,13 +10,17 @@
 #include <dhcp/option.h>
 #include <dhcp/option_data_types.h>
 #include <dhcp/option_space_container.h>
+#include <cc/stamped_element.h>
+#include <cc/user_context.h>
 
 #include <boost/multi_index/hashed_index.hpp>
 #include <boost/multi_index/mem_fun.hpp>
+#include <boost/multi_index/ordered_index.hpp>
 #include <boost/multi_index/sequenced_index.hpp>
 #include <boost/multi_index_container.hpp>
 #include <boost/shared_ptr.hpp>
 #include <map>
+#include <string>
 
 namespace isc {
 namespace dhcp {
@@ -83,15 +87,17 @@ class OptionIntArray;
 /// this is a single value the option type points to the data type of the
 /// value. For example, DHCPv6 option 8 comprises a two-byte option code, a
 /// two-byte option length and two-byte field that carries a uint16 value
-/// (RFC 3315 - http://ietf.org/rfc/rfc3315.txt).  In such a case, the option
-/// type is defined as "uint16".
+/// (RFC 8415 - http://ietf.org/rfc/rfc8415.txt).  In such a case, the option
+/// type is defined as "uint16". Length and string tuples are a length
+/// on one (DHCPv4) or two (DHCPv6) bytes followed by a string of
+/// the given length.
 ///
 /// When the option has a more complex structure, the option type may be
 /// defined as "array", "record" or even "array of records".
 ///
 /// Array types should be used when the option contains multiple contiguous
 /// data values of the same type laid. For example, DHCPv6 option 6 includes
-/// multiple fields holding uint16 codes of requested DHCPv6 options (RFC 3315).
+/// multiple fields holding uint16 codes of requested DHCPv6 options (RFC 8415).
 /// Such an option can be represented with this class by setting the option
 /// type to "uint16" and the array indicator (array_type) to true.  The number
 /// of elements in the array is effectively unlimited (although it is actually
@@ -101,7 +107,7 @@ class OptionIntArray;
 /// option type is used. In such cases the data field types within the record
 /// are specified using \ref OptionDefinition::addRecordField.
 ///
-/// When the OptionDefinition object has been sucessfully created, it can be
+/// When the OptionDefinition object has been successfully created, it can be
 /// queried to return the appropriate option factory function for the specified
 /// specified option format. There are a number of "standard" factory functions
 /// that cover well known (common) formats.  If the particular format does not
@@ -117,15 +123,19 @@ class OptionIntArray;
 /// - "uint16"
 /// - "uint32"
 /// - "ipv4-address" (IPv4 Address)
-/// - "ipv6-address" (IPV6 Address)
+/// - "ipv6-address" (IPv6 Address)
+/// - "ipv6-prefix" (IPv6 variable length prefix)
+/// - "psid" (PSID length / value)
 /// - "string"
 /// - "fqdn" (fully qualified name)
+/// - "tuple" (length and string)
 /// - "record" (set of data fields of different types)
 ///
 /// @todo Extend the comment to describe "generic factories".
 /// @todo Extend this class to use custom namespaces.
 /// @todo Extend this class with more factory functions.
-class OptionDefinition {
+/// @todo Derive from UserContext without breaking the multi index.
+class OptionDefinition : public data::StampedElement {
 public:
 
     /// List of fields within the record.
@@ -195,6 +205,81 @@ public:
                               const OptionDataType type,
                               const char* encapsulated_space);
 
+    /// @brief Factory function creating an instance of the @c OptionDefinition.
+    ///
+    /// This function should be used to create an instance of the option
+    /// definition within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param name option name.
+    /// @param code option code.
+    /// @param type option data type as string.
+    /// @param array_type array indicator, if true it indicates that the
+    /// option fields are the array.
+    ///
+    /// @return Pointer to the @c OptionDefinition instance.
+    static OptionDefinitionPtr create(const std::string& name,
+                                      const uint16_t code,
+                                      const std::string& type,
+                                      const bool array_type = false);
+
+    /// @brief Factory function creating an instance of the @c OptionDefinition.
+    ///
+    /// This function should be used to create an instance of the option
+    /// definition within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param name option name.
+    /// @param code option code.
+    /// @param type option data type.
+    /// @param array_type array indicator, if true it indicates that the
+    /// option fields are the array.
+    ///
+    /// @return Pointer to the @c OptionDefinition instance.
+    static OptionDefinitionPtr create(const std::string& name,
+                                      const uint16_t code,
+                                      const OptionDataType type,
+                                      const bool array_type = false);
+
+    /// @brief Factory function creating an instance of the @c OptionDefinition.
+    ///
+    /// This function should be used to create an instance of the option
+    /// definition within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param name option name.
+    /// @param code option code.
+    /// @param type option data type given as string.
+    /// @param encapsulated_space name of the option space being
+    /// encapsulated by this option.
+    ///
+    /// @return Pointer to the @c OptionDefinition instance.
+    static OptionDefinitionPtr create(const std::string& name,
+                                      const uint16_t code,
+                                      const std::string& type,
+                                      const char* encapsulated_space);
+
+    /// @brief Factory function creating an instance of the @c OptionDefinition.
+    ///
+    /// This function should be used to create an instance of the option
+    /// definition within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param name option name.
+    /// @param code option code.
+    /// @param type option data type.
+    /// @param encapsulated_space name of the option space being
+    /// encapsulated by this option.
+    ///
+    /// @return Pointer to the @c OptionDefinition instance.
+    static OptionDefinitionPtr create(const std::string& name,
+                                      const uint16_t code,
+                                      const OptionDataType type,
+                                      const char* encapsulated_space);
 
     /// @name Comparison functions and operators.
     ///
@@ -278,6 +363,53 @@ public:
     /// @return option data type.
     OptionDataType getType() const { return (type_); };
 
+    /// @brief Returns const pointer to the user context
+    data::ConstElementPtr getContext() const {
+        return (user_context_.getContext());
+    }
+
+    /// @brief Sets user context.
+    /// @param ctx user context to be stored.
+    void setContext(const data::ConstElementPtr& ctx) {
+        user_context_.setContext(ctx);
+    }
+
+    /// @brief Merge unparse a user_context object.
+    ///
+    /// Add user-context to map, but only if defined. Omit if it was not.
+    /// Extract comment so it will be pretty-printed first.
+    ///
+    /// @param map A pointer to map where the user context will be unparsed.
+    void contextToElement(data::ElementPtr map) const {
+        user_context_.contextToElement(map);
+    }
+
+    /// @brief Returns option space name.
+    ///
+    /// Option definitions are associated with option spaces. Typically,
+    /// such association is made when the option definition is put into
+    /// the @c CfgOptionDef structure. However, in some cases it is also
+    /// required to associate option definition with the particular option
+    /// space outside of that structure. In particular, when the option
+    /// definition is fetched from a database. The database configuration
+    /// backend will set option space upon return of the option definition.
+    /// In other cases this value won't be set.
+    ///
+    /// @return Option space name or empty string if option space
+    /// name is not set.
+    std::string getOptionSpaceName() const {
+        return (option_space_name_);
+    }
+
+    /// @brief Sets option space name for option definition.
+    ///
+    /// See @c getOptionSpaceName to learn when option space name is set.
+    ///
+    /// @param option_space_name New option space name.
+    void setOptionSpaceName(const std::string& option_space_name) {
+        option_space_name_ = option_space_name;
+    }
+
     /// @brief Check if the option definition is valid.
     ///
     /// Note that it is a responsibility of the code that created
@@ -358,6 +490,15 @@ public:
     /// @return true if option has the format of DHCPv6 Vendor Class option.
     bool haveVendorClass6Format() const;
 
+    /// @brief Check if option has format of the SLP Service Scope
+    /// %Option.
+    ///
+    /// The scope list in the SLP Service Scope option is optional
+    /// (i.e., as the error message in the DHCPv6 Status code option).
+    ///
+    /// @return true if option has the format of SLP Service Scope %Option.
+    bool haveServiceScopeFormat() const;
+
     /// @brief Check if the option has format of DHCPv6 Status Code option.
     ///
     /// @return true if option has the format of DHCPv6 Status code option.
@@ -368,6 +509,9 @@ public:
     /// @return true if option has the format of OpaqueDataTuples type options.
     bool haveOpaqueDataTuplesFormat() const;
 
+    /// @brief Check if the option has format of CompressedFqdnList options.
+    bool haveCompressedFqdnListFormat() const;
+
     /// @brief Option factory.
     ///
     /// This function creates an instance of DHCP option using
@@ -515,6 +659,21 @@ public:
                                       OptionBufferConstIter begin,
                                       OptionBufferConstIter end);
 
+    /// @brief Factory to create option with tuple list.
+    ///
+    /// @param u option universe (V4 or V6).
+    /// @param type option type.
+    /// @param begin iterator pointing to the beginning of the buffer
+    /// with a list of tuples.
+    /// @param end iterator pointing to the end of the buffer with
+    /// a list of tuples.
+    ///
+    /// @return instance of the DHCP option.
+    static OptionPtr factoryOpaqueDataTuples(Option::Universe u,
+                                             uint16_t type,
+                                             OptionBufferConstIter begin,
+                                             OptionBufferConstIter end);
+
     /// @brief Factory function to create option with integer value.
     ///
     /// @param u universe (V4 or V6).
@@ -558,6 +717,19 @@ public:
 
 private:
 
+    /// @brief Factory function to create option with a compressed FQDN list.
+    ///
+    /// @param u universe (V4 or V6).
+    /// @param type option type.
+    /// @param begin iterator pointing to the beginning of the buffer.
+    /// @param end iterator pointing to the end of the buffer.
+    ///
+    /// @return instance of the DHCP option where FQDNs are uncompressed.
+    /// @throw InvalidOptionValue if data for the option is invalid.
+    OptionPtr factoryFqdnList(Option::Universe u,
+                              OptionBufferConstIter begin,
+                              OptionBufferConstIter end) const;
+
     /// @brief Creates an instance of an option having special format.
     ///
     /// The option with special formats are encapsulated by the dedicated
@@ -621,7 +793,7 @@ private:
     /// This function performs lexical cast of a string value to integer
     /// value and checks if the resulting value is within a range of a
     /// target type. The target type should be one of the supported
-    /// integer types.
+    /// integer types. Hexadecimal input is supported.
     ///
     /// @param value_str input value given as string.
     /// @tparam T target integer type for lexical cast.
@@ -641,13 +813,14 @@ private:
     /// if it is successful it will store the data in the buffer
     /// in a binary format.
     ///
+    /// @param u option universe (V4 or V6).
     /// @param value string representation of the value to be written.
     /// @param type the actual data type to be stored.
     /// @param [in, out] buf buffer where the value is to be stored.
     ///
     /// @throw BadDataTypeCast if data write was unsuccessful.
-    void writeToBuffer(const std::string& value, const OptionDataType type,
-                       OptionBuffer& buf) const;
+    void writeToBuffer(Option::Universe u, const std::string& value,
+                       const OptionDataType type, OptionBuffer& buf) const;
 
     /// Option name.
     std::string name_;
@@ -655,12 +828,16 @@ private:
     uint16_t code_;
     /// Option data type.
     OptionDataType type_;
-    /// Indicates wheter option is a single value or array.
+    /// Indicates whether option is a single value or array.
     bool array_type_;
     /// Name of the space being encapsulated by this option.
     std::string encapsulated_space_;
     /// Collection of data fields within the record.
     RecordFieldsCollection record_fields_;
+    /// User context
+    data::UserContext user_context_;
+    /// Option space name
+    std::string option_space_name_;
 };
 
 
@@ -710,6 +887,23 @@ typedef boost::multi_index_container<
                 std::string,
                 &OptionDefinition::getName
             >
+        >,
+        // Start definition of index #3
+        boost::multi_index::ordered_non_unique<
+            // Use option definition modification time as the index key.
+            // This value is returned by the BaseStampedElement::getModificationTime
+            boost::multi_index::const_mem_fun<
+                data::BaseStampedElement,
+                boost::posix_time::ptime,
+                &data::StampedElement::getModificationTime
+            >
+        >,
+        // Start definition of index #4.
+        // Use StampedElement::getId as a key.
+        boost::multi_index::hashed_non_unique<
+            boost::multi_index::tag<OptionIdIndexTag>,
+            boost::multi_index::const_mem_fun<data::BaseStampedElement, uint64_t,
+                                              &data::BaseStampedElement::getId>
         >
     >
 > OptionDefContainer;
@@ -717,6 +911,9 @@ typedef boost::multi_index_container<
 /// Pointer to an option definition container.
 typedef boost::shared_ptr<OptionDefContainer> OptionDefContainerPtr;
 
+/// Container that holds option definitions for various option spaces.
+typedef std::map<std::string, OptionDefContainerPtr> OptionDefContainers;
+
 /// Container that holds various vendor option containers
 typedef std::map<uint32_t, OptionDefContainerPtr> VendorOptionDefContainers;
 
