@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2018 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -93,13 +93,14 @@ Pkt::delOption(uint16_t type) {
 
 bool
 Pkt::inClass(const std::string& client_class) {
-    return (classes_.find(client_class) != classes_.end());
+    return (classes_.contains(client_class));
 }
 
 void
-Pkt::addClass(const std::string& client_class) {
-    if (classes_.find(client_class) == classes_.end()) {
-        classes_.insert(client_class);
+Pkt::addClass(const std::string& client_class, bool required) {
+    ClientClasses& classes = !required ? classes_ : required_classes_;
+    if (!classes.contains(client_class)) {
+        classes.insert(client_class);
     }
 }
 
