@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,9 +13,13 @@
 #include <dhcp/option_int.h>
 #include <dhcp/option_string.h>
 #include <dhcp/option4_addrlst.h>
+#include <dhcp/option_vendor.h>
 #include <dhcp/pkt4.h>
 #include <exceptions/exceptions.h>
+#include <testutils/gtest_utils.h>
 #include <util/buffer.h>
+#include <util/encode/hex.h>
+#include <pkt_captures.h>
 
 #include <boost/shared_array.hpp>
 #include <boost/shared_ptr.hpp>
@@ -44,9 +48,9 @@ namespace {
 /// variable length data so as there are no restrictions
 /// on a length of their data.
 static uint8_t v4_opts[] = {
+    53, 1, 2, // Message Type (required to not throw exception during unpack)
     12,  3, 0,   1,  2, // Hostname
     14,  3, 10, 11, 12, // Merit Dump File
-    53, 1, 2, // Message Type (required to not throw exception during unpack)
     60,  3, 20, 21, 22, // Class Id
     128, 3, 30, 31, 32, // Vendor specific
     254, 3, 40, 41, 42, // Reserved
@@ -174,16 +178,24 @@ public:
         EXPECT_TRUE(pkt->getOption(128));
         EXPECT_TRUE(pkt->getOption(254));
 
-        boost::shared_ptr<Option> x = pkt->getOption(12);
-        ASSERT_TRUE(x); // option 1 should exist
+        // Verify the packet type is correct.
+        ASSERT_EQ(DHCPOFFER, pkt->getType());
+
+        // First option after message type starts at 3.
+        uint8_t *opt_data_ptr = v4_opts + 3;
+
         // Option 12 is represented by the OptionString class so let's do
         // the appropriate conversion.
+        boost::shared_ptr<Option> x = pkt->getOption(12);
+        ASSERT_TRUE(x); // option 1 should exist
         OptionStringPtr option12 = boost::static_pointer_cast<OptionString>(x);
+
         ASSERT_TRUE(option12);
         EXPECT_EQ(12, option12->getType());  // this should be option 12
         ASSERT_EQ(3, option12->getValue().length()); // it should be of length 3
         EXPECT_EQ(5, option12->len()); // total option length 5
-        EXPECT_EQ(0, memcmp(&option12->getValue()[0], v4_opts + 2, 3)); // data len=3
+        EXPECT_EQ(0, memcmp(&option12->getValue()[0], opt_data_ptr + 2, 2)); // data len=3
+        opt_data_ptr += x->len();
 
         x = pkt->getOption(14);
         ASSERT_TRUE(x); // option 14 should exist
@@ -194,28 +206,32 @@ public:
         EXPECT_EQ(14, option14->getType());  // this should be option 14
         ASSERT_EQ(3, option14->getValue().length()); // it should be of length 3
         EXPECT_EQ(5, option14->len()); // total option length 5
-        EXPECT_EQ(0, memcmp(&option14->getValue()[0], v4_opts + 7, 3)); // data len=3
+
+        EXPECT_EQ(0, memcmp(&option14->getValue()[0], opt_data_ptr + 2, 3)); // data len=3
+        opt_data_ptr += x->len();
 
         x = pkt->getOption(60);
         ASSERT_TRUE(x); // option 60 should exist
         EXPECT_EQ(60, x->getType());  // this should be option 60
         ASSERT_EQ(3, x->getData().size()); // it should be of length 3
         EXPECT_EQ(5, x->len()); // total option length 5
-        EXPECT_EQ(0, memcmp(&x->getData()[0], v4_opts + 15, 3)); // data len=3
+        EXPECT_EQ(0, memcmp(&x->getData()[0], opt_data_ptr + 2, 3)); // data len=3
+        opt_data_ptr += x->len();
 
         x = pkt->getOption(128);
         ASSERT_TRUE(x); // option 3 should exist
         EXPECT_EQ(128, x->getType());  // this should be option 254
         ASSERT_EQ(3, x->getData().size()); // it should be of length 3
         EXPECT_EQ(5, x->len()); // total option length 5
-        EXPECT_EQ(0, memcmp(&x->getData()[0], v4_opts + 20, 3)); // data len=3
+        EXPECT_EQ(0, memcmp(&x->getData()[0], opt_data_ptr + 2, 3)); // data len=3
+        opt_data_ptr += x->len();
 
         x = pkt->getOption(254);
         ASSERT_TRUE(x); // option 3 should exist
         EXPECT_EQ(254, x->getType());  // this should be option 254
         ASSERT_EQ(3, x->getData().size()); // it should be of length 3
         EXPECT_EQ(5, x->len()); // total option length 5
-        EXPECT_EQ(0, memcmp(&x->getData()[0], v4_opts + 25, 3)); // data len=3
+        EXPECT_EQ(0, memcmp(&x->getData()[0], opt_data_ptr + 2, 3)); // data len=3
     }
 
 };
@@ -684,7 +700,7 @@ TEST_F(Pkt4Test, unpackMalformed) {
     orig.push_back(0x53);
     orig.push_back(0x63);
 
-    orig.push_back(53); // Message Type 
+    orig.push_back(53); // Message Type
     orig.push_back(1); // length=1
     orig.push_back(2); // type=2
 
@@ -727,7 +743,7 @@ TEST_F(Pkt4Test, unpackVendorMalformed) {
     orig.push_back(0x53);
     orig.push_back(0x63);
 
-    orig.push_back(53); // Message Type 
+    orig.push_back(53); // Message Type
     orig.push_back(1); // length=1
     orig.push_back(2); // type=2
 
@@ -760,7 +776,7 @@ TEST_F(Pkt4Test, unpackVendorMalformed) {
     baddatalen.resize(orig.size() - 5);
     baddatalen[full_len_index] = 10;
     Pkt4Ptr bad_data_len_pkt(new Pkt4(&baddatalen[0], baddatalen.size()));
-    EXPECT_THROW(bad_data_len_pkt->unpack(), InvalidOptionValue);
+    EXPECT_THROW(bad_data_len_pkt->unpack(), SkipRemainingOptionsError);
 
     // A suboption must have a length byte
     vector<uint8_t> nolength = orig;
@@ -768,7 +784,7 @@ TEST_F(Pkt4Test, unpackVendorMalformed) {
     nolength[full_len_index] = 11;
     nolength[data_len_index] = 6;
     Pkt4Ptr no_length_pkt(new Pkt4(&nolength[0], nolength.size()));
-    EXPECT_THROW(no_length_pkt->unpack(), InvalidOptionValue);
+    EXPECT_THROW(no_length_pkt->unpack(), SkipRemainingOptionsError);
 
     // Truncated data is not accepted either
     vector<uint8_t> shorty = orig;
@@ -776,7 +792,7 @@ TEST_F(Pkt4Test, unpackVendorMalformed) {
     shorty[full_len_index] = 14;
     shorty[data_len_index] = 9;
     Pkt4Ptr too_short_pkt(new Pkt4(&shorty[0], shorty.size()));
-    EXPECT_THROW(too_short_pkt->unpack(), InvalidOptionValue);
+    EXPECT_THROW(too_short_pkt->unpack(), SkipRemainingOptionsError);
 }
 
 // This test verifies methods that are used for manipulating meta fields
@@ -916,13 +932,13 @@ TEST_F(Pkt4Test, clientClasses) {
     // Default values (do not belong to any class)
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_EROUTER));
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_MODEM));
-    EXPECT_TRUE(pkt.classes_.empty());
+    EXPECT_TRUE(pkt.getClasses().empty());
 
     // Add to the first class
     pkt.addClass(DOCSIS3_CLASS_EROUTER);
     EXPECT_TRUE(pkt.inClass(DOCSIS3_CLASS_EROUTER));
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_MODEM));
-    ASSERT_FALSE(pkt.classes_.empty());
+    ASSERT_FALSE(pkt.getClasses().empty());
 
     // Add to a second class
     pkt.addClass(DOCSIS3_CLASS_MODEM);
@@ -938,12 +954,40 @@ TEST_F(Pkt4Test, clientClasses) {
     EXPECT_TRUE(pkt.inClass("foo"));
 }
 
+// Tests whether a packet can be marked to evaluate later a class and
+// after check if a given class is in the collection
+TEST_F(Pkt4Test, deferredClientClasses) {
+    Pkt4 pkt(DHCPOFFER, 1234);
+
+    // Default values (do not belong to any class)
+    EXPECT_TRUE(pkt.getClasses(true).empty());
+
+    // Add to the first class
+    pkt.addClass(DOCSIS3_CLASS_EROUTER, true);
+    EXPECT_EQ(1, pkt.getClasses(true).size());
+
+    // Add to a second class
+    pkt.addClass(DOCSIS3_CLASS_MODEM, true);
+    EXPECT_EQ(2, pkt.getClasses(true).size());
+    EXPECT_TRUE(pkt.getClasses(true).contains(DOCSIS3_CLASS_EROUTER));
+    EXPECT_TRUE(pkt.getClasses(true).contains(DOCSIS3_CLASS_MODEM));
+    EXPECT_FALSE(pkt.getClasses(true).contains("foo"));
+
+    // Check that it's ok to add to the same class repeatedly
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+
+    // Check that the packet belongs to 'foo'
+    EXPECT_TRUE(pkt.getClasses(true).contains("foo"));
+}
+
 // Tests whether MAC can be obtained and that MAC sources are not
 // confused.
 TEST_F(Pkt4Test, getMAC) {
     Pkt4 pkt(DHCPOFFER, 1234);
 
-    // DHCPv4 packet by default doens't have MAC address specified.
+    // DHCPv4 packet by default doesn't have MAC address specified.
     EXPECT_FALSE(pkt.getMAC(HWAddr::HWADDR_SOURCE_ANY));
     EXPECT_FALSE(pkt.getMAC(HWAddr::HWADDR_SOURCE_RAW));
 
@@ -1093,7 +1137,7 @@ TEST_F(Pkt4Test, toText) {
     pkt.addOption(OptionPtr(new OptionUint32(Option::V4, 156, 123456)));
     pkt.addOption(OptionPtr(new OptionString(Option::V4, 87, "lorem ipsum")));
 
-    EXPECT_EQ("local_address=192.0.2.34:67, remote_adress=192.10.33.4:68, "
+    EXPECT_EQ("local_address=192.0.2.34:67, remote_address=192.10.33.4:68, "
               "msg_type=DHCPDISCOVER (1), transid=0x9ef,\n"
               "options:\n"
               "  type=053, len=001: 1 (uint8)\n"
@@ -1109,7 +1153,7 @@ TEST_F(Pkt4Test, toText) {
     pkt.delOption(87);
     pkt.delOption(53);
 
-    EXPECT_EQ("local_address=192.0.2.34:67, remote_adress=192.10.33.4:68, "
+    EXPECT_EQ("local_address=192.0.2.34:67, remote_address=192.10.33.4:68, "
               "msg_type=(missing), transid=0x9ef, "
               "message contains no options",
               pkt.toText());
@@ -1131,4 +1175,183 @@ TEST_F(Pkt4Test, getType) {
     EXPECT_TRUE(pkt.getName());
 }
 
+// Verifies that when the VIVSO option 125 has length that is too
+// short (i.e. less than sizeof(uint8_t), unpack throws a
+// SkipRemainingOptionsError exception
+TEST_F(Pkt4Test, truncatedVendorLength) {
+
+    // Build a good discover packet
+    Pkt4Ptr pkt = dhcp::test::PktCaptures::discoverWithValidVIVSO();
+
+    // Unpacking should not throw
+    ASSERT_NO_THROW(pkt->unpack());
+    ASSERT_EQ(DHCPDISCOVER, pkt->getType());
+
+    // VIVSO option should be there
+    OptionPtr x = pkt->getOption(DHO_VIVSO_SUBOPTIONS);
+    ASSERT_TRUE(x);
+    ASSERT_EQ(DHO_VIVSO_SUBOPTIONS, x->getType());
+    OptionVendorPtr vivso = boost::dynamic_pointer_cast<OptionVendor>(x);
+    ASSERT_TRUE(vivso);
+    EXPECT_EQ(133+2, vivso->len()); // data + opt code + len
+
+    // Build a bad discover packet
+    pkt = dhcp::test::PktCaptures::discoverWithTruncatedVIVSO();
+
+    // Unpack should throw Skip exception
+    ASSERT_THROW(pkt->unpack(), SkipRemainingOptionsError);
+    ASSERT_EQ(DHCPDISCOVER, pkt->getType());
+
+    // VIVSO option should not be there
+    x = pkt->getOption(DHO_VIVSO_SUBOPTIONS);
+    ASSERT_FALSE(x);
+}
+
+// Verifies that we handle text options that contain trailing
+// and embedded NULLs correctly.  Per RFC 2132, Sec 2 we should
+// be stripping trailing NULLs.  We've agreed to permit
+// embedded NULLs (for now).
+TEST_F(Pkt4Test, nullTerminatedOptions) {
+    // Construct the onwire packet.
+    vector<uint8_t> base_msg = generateTestPacket2();
+    base_msg.push_back(0x63); // magic cookie
+    base_msg.push_back(0x82);
+    base_msg.push_back(0x53);
+    base_msg.push_back(0x63);
+
+    base_msg.push_back(0x35); // message-type
+    base_msg.push_back(0x1);
+    base_msg.push_back(0x1);
+
+    int base_size = base_msg.size();
+
+    // We'll create four text options, with various combinations of NULLs.
+    vector<uint8_t> hostname = { DHO_HOST_NAME, 5, 't', 'w', 'o', 0, 0 };
+    vector<uint8_t> merit_dump = { DHO_MERIT_DUMP, 4, 'o', 'n', 'e', 0 };
+    vector<uint8_t> root_path = { DHO_ROOT_PATH, 4, 'n', 'o', 'n', 'e' };
+    vector<uint8_t> domain_name = { DHO_DOMAIN_NAME, 6, 'e', 'm', 0, 'b', 'e', 'd' };
+
+    // Add the options to the onwire packet.
+    vector<uint8_t> test_msg = base_msg;
+    test_msg.insert(test_msg.end(), hostname.begin(), hostname.end());
+    test_msg.insert(test_msg.end(), root_path.begin(), root_path.end());
+    test_msg.insert(test_msg.end(), merit_dump.begin(), merit_dump.end());
+    test_msg.insert(test_msg.end(), domain_name.begin(), domain_name.end());
+    test_msg.push_back(DHO_END);
+
+    boost::shared_ptr<Pkt4> pkt(new Pkt4(&test_msg[0], test_msg.size()));
+
+    // Unpack the onwire packet.
+    EXPECT_NO_THROW(
+        pkt->unpack()
+    );
+
+    EXPECT_EQ(DHCPDISCOVER, pkt->getType());
+
+    OptionPtr opt;
+    OptionStringPtr opstr;
+
+    // Now let's verify that each text option is as expected.
+    ASSERT_TRUE(opt = pkt->getOption(DHO_HOST_NAME));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("two", opstr->getValue());
+
+    ASSERT_TRUE(opt = pkt->getOption(DHO_MERIT_DUMP));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("one", opstr->getValue());
+
+    ASSERT_TRUE(opt = pkt->getOption(DHO_ROOT_PATH));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(4, opstr->getValue().length());
+    EXPECT_EQ("none", opstr->getValue());
+
+    ASSERT_TRUE(opt = pkt->getOption(DHO_DOMAIN_NAME));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(6, opstr->getValue().length());
+    std::string embed{"em\0bed", 6};
+    EXPECT_EQ(embed, opstr->getValue());
+
+
+    // Next we pack the packet, to make sure trailing NULLs have
+    // been eliminated, embedded NULLs are intact.
+    EXPECT_NO_THROW(
+        pkt->pack()
+    );
+
+    // Create a vector of our expectd packed option data.
+    vector<uint8_t> packed_opts =
+        {
+          DHO_HOST_NAME, 3, 't', 'w', 'o',
+          DHO_MERIT_DUMP, 3, 'o', 'n', 'e',
+          DHO_DOMAIN_NAME, 6, 'e', 'm', 0, 'b', 'e', 'd',
+          DHO_ROOT_PATH, 4, 'n', 'o', 'n', 'e',
+        };
+
+    const uint8_t* packed = static_cast<const uint8_t*>(pkt->getBuffer().getData());
+    int packed_len = pkt->getBuffer().getLength();
+
+    // Packed message options should be 3 bytes smaller than original onwire data.
+    int dif = packed_len - test_msg.size();
+    ASSERT_EQ(-3, dif);
+
+    // Make sure the packed content is as expected.
+    EXPECT_EQ(0, memcmp(&packed[base_size], &packed_opts[0], packed_opts.size()));
+}
+
+// Checks that unpacking correctly handles SkipThisOptionError by
+// omitting the offending option from the unpacked options.
+TEST_F(Pkt4Test, testSkipThisOptionError) {
+    vector<uint8_t> orig = generateTestPacket2();
+
+    orig.push_back(0x63);
+    orig.push_back(0x82);
+    orig.push_back(0x53);
+    orig.push_back(0x63);
+
+    orig.push_back(53);   // Message Type
+    orig.push_back(1);    // length=1
+    orig.push_back(2);    // type=2
+
+    orig.push_back(14);   // merit-dump
+    orig.push_back(3);    // length=3
+    orig.push_back(0x61); // data="abc"
+    orig.push_back(0x62);
+    orig.push_back(0x63);
+
+    orig.push_back(12);   // Hostname
+    orig.push_back(3);    // length=3
+    orig.push_back(0);    // data= all nulls
+    orig.push_back(0);
+    orig.push_back(0);
+
+    orig.push_back(17);   // root-path
+    orig.push_back(3);    // length=3
+    orig.push_back(0x64); // data="def"
+    orig.push_back(0x65);
+    orig.push_back(0x66);
+
+    // Unpacking should not throw.
+    Pkt4Ptr pkt(new Pkt4(&orig[0], orig.size()));
+    ASSERT_NO_THROW_LOG(pkt->unpack());
+
+    // We should have option 14 = "abc".
+    OptionPtr opt;
+    OptionStringPtr opstr;
+    ASSERT_TRUE(opt = pkt->getOption(14));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("abc", opstr->getValue());
+
+    // We should not have option 12.
+    EXPECT_FALSE(opt = pkt->getOption(12));
+
+    // We should have option 17 = "def".
+    ASSERT_TRUE(opt = pkt->getOption(17));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("def", opstr->getValue());
+}
+
 } // end of anonymous namespace
