@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,12 +13,14 @@
 #include <dhcp/option6_ia.h>
 #include <dhcp/option_int.h>
 #include <dhcp/option_int_array.h>
+#include <dhcp/option_string.h>
 #include <dhcp/option_vendor.h>
 #include <dhcp/iface_mgr.h>
 #include <dhcp/pkt6.h>
 #include <dhcp/hwaddr.h>
 #include <dhcp/docsis3_option_defs.h>
 #include <dhcp/tests/pkt_captures.h>
+#include <testutils/gtest_utils.h>
 #include <util/range_utilities.h>
 #include <boost/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -410,14 +412,14 @@ TEST_F(Pkt6Test, unpackVendorMalformed) {
     shorth.resize(orig.size() - 4);
     shorth[len_index] = 12;
     Pkt6Ptr too_short_header_pkt(new Pkt6(&shorth[0], shorth.size()));
-    EXPECT_THROW(too_short_header_pkt->unpack(), OutOfRange);
+    EXPECT_THROW(too_short_header_pkt->unpack(), SkipRemainingOptionsError);
 
     // Truncated option data is not accepted
     vector<uint8_t> shorto = orig;
     shorto.resize(orig.size() - 2);
     shorto[len_index] = 16;
     Pkt6Ptr too_short_option_pkt(new Pkt6(&shorto[0], shorto.size()));
-    EXPECT_THROW(too_short_option_pkt->unpack(), OutOfRange);
+    EXPECT_THROW(too_short_option_pkt->unpack(), SkipRemainingOptionsError);
 }
 
 // This test verifies that options can be added (addOption()), retrieved
@@ -986,7 +988,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     EXPECT_TRUE(opt->equals(relay3_opt1));
     EXPECT_FALSE(opt == relay3_opt1);
     // Test that option copy has replaced the original option within the
-    // packet. We achive that by calling a variant of the method which
+    // packet. We achieve that by calling a variant of the method which
     // retrieved non-copied option.
     relay3_opt1 = msg->getNonCopiedAnyRelayOption(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
     ASSERT_TRUE(relay3_opt1);
@@ -1049,6 +1051,38 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     EXPECT_FALSE(opt);
 }
 
+// Tests whether Pkt6::toText() properly prints out all parameters, including
+// relay options: remote-id, interface-id.
+TEST_F(Pkt6Test, toText) {
+
+    // This packet contains doubly relayed solicit. The inner-most
+    // relay-forward contains interface-id and remote-id. We will
+    // check that these are printed correctly.
+    Pkt6Ptr msg(capture2());
+    EXPECT_NO_THROW(msg->unpack());
+
+    ASSERT_EQ(2, msg->relay_info_.size());
+
+    string expected =
+        "localAddr=[ff05::1:3]:547 remoteAddr=[fe80::1234]:547\n"
+        "msgtype=1(SOLICIT), transid=0x6b4fe2\n"
+        "type=00001, len=00014: 00:01:00:01:18:b0:33:41:00:00:21:5c:18:a9\n"
+        "type=00003(IA_NA), len=00012: iaid=1, t1=4294967295, t2=4294967295\n"
+        "type=00006, len=00006: 23(uint16) 242(uint16) 243(uint16)\n"
+        "type=00008, len=00002: 0 (uint16)\n"
+        "2 relay(s):\n"
+        "relay[0]: msg-type=12(RELAY_FORWARD), hop-count=1,\n"
+        "link-address=2001:888:db8:1::, peer-address=fe80::200:21ff:fe5c:18a9, 2 option(s)\n"
+        "type=00018, len=00028: 49:53:41:4d:31:34:34:7c:32:39:39:7c:69:70:76:36:7c:6e:74:3a:76:70:3a:31:3a:31:31:30\n"
+        "type=00037, len=00018: 6527 (uint32) 0001000118B033410000215C18A9 (binary)\n"
+        "relay[1]: msg-type=12(RELAY_FORWARD), hop-count=0,\n"
+        "link-address=::, peer-address=fe80::200:21ff:fe5c:18a9, 2 option(s)\n"
+        "type=00018, len=00021: 49:53:41:4d:31:34:34:20:65:74:68:20:31:2f:31:2f:30:35:2f:30:31\n"
+        "type=00037, len=00004: 3561 (uint32)  (binary)\n";
+
+    EXPECT_EQ(expected, msg->toText());
+}
+
 // Tests whether a packet can be assigned to a class and later
 // checked if it belongs to a given class
 TEST_F(Pkt6Test, clientClasses) {
@@ -1057,13 +1091,13 @@ TEST_F(Pkt6Test, clientClasses) {
     // Default values (do not belong to any class)
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_EROUTER));
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_MODEM));
-    EXPECT_TRUE(pkt.classes_.empty());
+    EXPECT_TRUE(pkt.getClasses().empty());
 
     // Add to the first class
     pkt.addClass(DOCSIS3_CLASS_EROUTER);
     EXPECT_TRUE(pkt.inClass(DOCSIS3_CLASS_EROUTER));
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_MODEM));
-    ASSERT_FALSE(pkt.classes_.empty());
+    ASSERT_FALSE(pkt.getClasses().empty());
 
     // Add to a second class
     pkt.addClass(DOCSIS3_CLASS_MODEM);
@@ -1079,12 +1113,40 @@ TEST_F(Pkt6Test, clientClasses) {
     EXPECT_TRUE(pkt.inClass("foo"));
 }
 
+// Tests whether a packet can be marked to evaluate later a class and
+// after check if a given class is in the collection
+TEST_F(Pkt6Test, deferredClientClasses) {
+    Pkt6 pkt(DHCPV6_ADVERTISE, 1234);
+
+    // Default values (do not belong to any class)
+    EXPECT_TRUE(pkt.getClasses(true).empty());
+
+    // Add to the first class
+    pkt.addClass(DOCSIS3_CLASS_EROUTER, true);
+    EXPECT_EQ(1, pkt.getClasses(true).size());
+
+    // Add to a second class
+    pkt.addClass(DOCSIS3_CLASS_MODEM, true);
+    EXPECT_EQ(2, pkt.getClasses(true).size());
+    EXPECT_TRUE(pkt.getClasses(true).contains(DOCSIS3_CLASS_EROUTER));
+    EXPECT_TRUE(pkt.getClasses(true).contains(DOCSIS3_CLASS_MODEM));
+    EXPECT_FALSE(pkt.getClasses(true).contains("foo"));
+
+    // Check that it's ok to add to the same class repeatedly
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+
+    // Check that the packet belongs to 'foo'
+    EXPECT_TRUE(pkt.getClasses(true).contains("foo"));
+}
+
 // Tests whether MAC can be obtained and that MAC sources are not
 // confused.
 TEST_F(Pkt6Test, getMAC) {
     Pkt6 pkt(DHCPV6_ADVERTISE, 1234);
 
-    // DHCPv6 packet by default doens't have MAC address specified.
+    // DHCPv6 packet by default doesn't have MAC address specified.
     EXPECT_FALSE(pkt.getMAC(HWAddr::HWADDR_SOURCE_ANY));
     EXPECT_FALSE(pkt.getMAC(HWAddr::HWADDR_SOURCE_RAW));
 
@@ -1521,7 +1583,7 @@ TEST_F(Pkt6Test, getMACFromRemoteIdRelayOption) {
 // (Relay Supplied Options option). This test checks whether it was parsed
 // properly. See captureRelayed2xRSOO() description for details.
 TEST_F(Pkt6Test, rsoo) {
-    Pkt6Ptr msg = test::PktCaptures::captureRelayed2xRSOO();
+    Pkt6Ptr msg = dhcp::test::PktCaptures::captureRelayed2xRSOO();
 
     EXPECT_NO_THROW(msg->unpack());
 
@@ -1578,7 +1640,7 @@ TEST_F(Pkt6Test, getClientId) {
     EXPECT_TRUE(duid->getDuid() == duid_vec);
 }
 
-// This test verfies that it is possible to obtain the packet
+// This test verifies that it is possible to obtain the packet
 // identifiers (DUID, HW Address, transaction id) in the textual
 // format.
 TEST_F(Pkt6Test, makeLabel) {
@@ -1657,9 +1719,97 @@ TEST_F(Pkt6Test, getLabelEmptyClientId) {
     // Create a packet.
     Pkt6 pkt(DHCPV6_SOLICIT, 0x2312);
 
-    // Add empty client idenitifier option.
+    // Add empty client identifier option.
     pkt.addOption(OptionPtr(new Option(Option::V6, D6O_CLIENTID)));
     EXPECT_EQ("duid=[no info], tid=0x2312", pkt.getLabel());
 }
 
+// Verifies that when the VIVSO, 17, has length that is too
+// short (i.e. less than sizeof(uint8_t), unpack throws a
+// SkipRemainingOptionsError exception
+TEST_F(Pkt6Test, truncatedVendorLength) {
+
+    // Build a good Solicit packet
+    Pkt6Ptr pkt = dhcp::test::PktCaptures::captureSolicitWithVIVSO();
+
+    // Unpacking should not throw
+    ASSERT_NO_THROW(pkt->unpack());
+    ASSERT_EQ(DHCPV6_SOLICIT, pkt->getType());
+
+    // VIVSO option should be there
+    OptionPtr x = pkt->getOption(D6O_VENDOR_OPTS);
+    ASSERT_TRUE(x);
+    ASSERT_EQ(D6O_VENDOR_OPTS, x->getType());
+    OptionVendorPtr vivso = boost::dynamic_pointer_cast<OptionVendor>(x);
+    ASSERT_TRUE(vivso);
+    EXPECT_EQ(8, vivso->len()); // data + opt code + len
+
+    // Build a bad Solicit packet
+    pkt = dhcp::test::PktCaptures::captureSolicitWithTruncatedVIVSO();
+
+    // Unpack should throw Skip exception
+    ASSERT_THROW(pkt->unpack(), SkipRemainingOptionsError);
+    ASSERT_EQ(DHCPV6_SOLICIT, pkt->getType());
+
+    // VIVSO option should not be there
+    x = pkt->getOption(D6O_VENDOR_OPTS);
+    ASSERT_FALSE(x);
+}
+
+// Checks that unpacking correctly handles SkipThisOptionError by
+// omitting the offending option from the unpacked options.
+TEST_F(Pkt6Test, testSkipThisOptionError) {
+    // Get a packet. We're really interested in its on-wire
+    // representation only.
+    Pkt6Ptr donor(capture1());
+
+    // That's our original content. It should be sane.
+    OptionBuffer orig = donor->data_;
+
+    orig.push_back(0);
+    orig.push_back(41);   // new-posix-timezone
+    orig.push_back(0);
+    orig.push_back(3);    // length=3
+    orig.push_back(0x61); // data="abc"
+    orig.push_back(0x62);
+    orig.push_back(0x63);
+
+    orig.push_back(0);
+    orig.push_back(59);   // bootfile-url
+    orig.push_back(0);
+    orig.push_back(3);    // length=3
+    orig.push_back(0);    // data= all nulls
+    orig.push_back(0);
+    orig.push_back(0);
+
+    orig.push_back(0);
+    orig.push_back(42);   // new-tzdb-timezone
+    orig.push_back(0);
+    orig.push_back(3);    // length=3
+    orig.push_back(0x64); // data="def"
+    orig.push_back(0x65);
+    orig.push_back(0x66);
+
+    // Unpacking should not throw.
+    Pkt6Ptr pkt(new Pkt6(&orig[0], orig.size()));
+    ASSERT_NO_THROW_LOG(pkt->unpack());
+
+    // We should have option 41 = "abc".
+    OptionPtr opt;
+    OptionStringPtr opstr;
+    ASSERT_TRUE(opt = pkt->getOption(41));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("abc", opstr->getValue());
+
+    // We should not have option 59.
+    EXPECT_FALSE(opt = pkt->getOption(59));
+
+    // We should have option 42 = "def".
+    ASSERT_TRUE(opt = pkt->getOption(42));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("def", opstr->getValue());
+}
+
 }
