@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,14 +10,11 @@
 #include <dhcp/libdhcp++.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/dhcpsrv_log.h>
-#include <dhcpsrv/subnet_id.h>
-#include <stats/stats_mgr.h>
 #include <sstream>
 #include <string>
 
 using namespace isc::asiolink;
 using namespace isc::util;
-using namespace isc::stats;
 
 namespace isc {
 namespace dhcp {
@@ -30,59 +27,29 @@ CfgMgr::instance() {
     return (cfg_mgr);
 }
 
-void
-CfgMgr::addOptionSpace4(const OptionSpacePtr& space) {
-    if (!space) {
-        isc_throw(InvalidOptionSpace,
-                  "provided option space object is NULL.");
-    }
-    OptionSpaceCollection::iterator it = spaces4_.find(space->getName());
-    if (it != spaces4_.end()) {
-        isc_throw(InvalidOptionSpace, "option space " << space->getName()
-                  << " already added.");
-    }
-    spaces4_.insert(make_pair(space->getName(), space));
-}
-
-void
-CfgMgr::addOptionSpace6(const OptionSpacePtr& space) {
-    if (!space) {
-        isc_throw(InvalidOptionSpace,
-                  "provided option space object is NULL.");
-    }
-    OptionSpaceCollection::iterator it = spaces6_.find(space->getName());
-    if (it != spaces6_.end()) {
-        isc_throw(InvalidOptionSpace, "option space " << space->getName()
-                  << " already added.");
-    }
-    spaces6_.insert(make_pair(space->getName(), space));
-}
-
-
-std::string CfgMgr::getDataDir() const {
+Optional<std::string>
+CfgMgr::getDataDir() const {
     return (datadir_);
 }
 
 void
-CfgMgr::setDataDir(const std::string& datadir) {
-    datadir_ = datadir;
+CfgMgr::setDataDir(const std::string& datadir, bool unspecified) {
+    datadir_ = Optional<std::string>(datadir, unspecified);
 }
 
-bool
-CfgMgr::isDuplicate(const Subnet6& subnet) const {
-    for (Subnet6Collection::const_iterator subnet_it = subnets6_.begin();
-         subnet_it != subnets6_.end(); ++subnet_it) {
-        if ((*subnet_it)->getID() == subnet.getID()) {
-            return (true);
-        }
-    }
-    return (false);
-}
-
-
 void
 CfgMgr::setD2ClientConfig(D2ClientConfigPtr& new_config) {
+    ensureCurrentAllocated();
+    // Note that D2ClientMgr::setD2Config() actually attempts to apply the
+    // configuration by stopping its sender and opening a new one and so
+    // forth per the new configuration.
     d2_client_mgr_.setD2ClientConfig(new_config);
+
+    // Manager will throw if the set fails, if it succeeds
+    // we'll update our SrvConfig, configuration_, with the D2ClientConfig
+    // used. This is largely bookkeeping in case we ever want to compare
+    // configuration_ to another SrvConfig.
+    configuration_->setD2ClientConfig(new_config);
 }
 
 bool
@@ -114,6 +81,7 @@ CfgMgr::clear() {
         configuration_->removeStatistics();
     }
     configs_.clear();
+    external_configs_.clear();
     ensureCurrentAllocated();
 }
 
@@ -184,7 +152,7 @@ CfgMgr::revert(const size_t index) {
     commit();
 }
 
-ConstSrvConfigPtr
+SrvConfigPtr
 CfgMgr::getCurrentCfg() {
     ensureCurrentAllocated();
     return (configuration_);
@@ -200,9 +168,54 @@ CfgMgr::getStagingCfg() {
     return (configs_.back());
 }
 
+SrvConfigPtr
+CfgMgr::createExternalCfg() {
+    uint32_t seq = 0;
+
+    if (!external_configs_.empty()) {
+        seq = external_configs_.rbegin()->second->getSequence() + 1;
+    }
+
+    SrvConfigPtr srv_config(new SrvConfig(seq));
+    external_configs_[seq] = srv_config;
+    return (srv_config);
+}
+
+void
+CfgMgr::mergeIntoStagingCfg(const uint32_t seq) {
+    mergeIntoCfg(getStagingCfg(), seq);
+}
+
+void
+CfgMgr::mergeIntoCurrentCfg(const uint32_t seq) {
+    try {
+        // First we need to remove statistics.
+        getCurrentCfg()->removeStatistics();
+        mergeIntoCfg(getCurrentCfg(), seq);
+
+    } catch (...) {
+        // Make sure the statistics is updated even if the merge failed.
+        getCurrentCfg()->updateStatistics();
+        throw;
+    }
+    getCurrentCfg()->updateStatistics();
+}
+
+void
+CfgMgr::mergeIntoCfg(const SrvConfigPtr& target_config, const uint32_t seq) {
+    auto source_config = external_configs_.find(seq);
+    if (source_config != external_configs_.end()) {
+        target_config->merge(*source_config->second);
+        external_configs_.erase(source_config);
+
+    } else {
+        isc_throw(BadValue, "the external configuration with the sequence number "
+                  "of " << seq << " was not found");
+    }
+}
+
 CfgMgr::CfgMgr()
-    : datadir_(DHCP_DATA_DIR), echo_v4_client_id_(true),
-      d2_client_mgr_(), verbose_mode_(false) {
+    : datadir_(DHCP_DATA_DIR, true), d2_client_mgr_(), family_(AF_INET) {
     // DHCP_DATA_DIR must be set set with -DDHCP_DATA_DIR="..." in Makefile.am
     // Note: the definition of DHCP_DATA_DIR needs to include quotation marks
     // See AM_CPPFLAGS definition in Makefile.am
