@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,9 +11,14 @@
 /// This file defines the classes Kea uses to manage configuration needed to
 /// act as a client of the kea-dhcp-ddns module (aka D2).
 ///
+
 #include <asiolink/io_address.h>
+#include <cc/cfg_to_element.h>
+#include <cc/user_context.h>
 #include <dhcp_ddns/ncr_io.h>
 #include <exceptions/exceptions.h>
+#include <util/strutil.h>
+#include <util/optional.h>
 
 #include <boost/shared_ptr.hpp>
 
@@ -24,7 +29,6 @@
 namespace isc {
 namespace dhcp {
 
-
 /// An exception that is thrown if an error occurs while configuring
 /// the D2 DHCP DDNS client.
 class D2ClientError : public isc::Exception {
@@ -39,6 +43,10 @@ public:
         : isc::Exception(file, line, what) {}
 };
 
+/// @brief Callback function for @c D2ClientConfig that retrieves globally
+/// configured parameters.
+typedef std::function<data::ConstElementPtr()> FetchNetworkGlobalsFn;
+
 
 /// @brief Acts as a storage vault for D2 client configuration
 ///
@@ -46,11 +54,9 @@ public:
 /// parameters associated with DHCP-DDNS and acting as a client of D2.
 /// Instances of this class may be constructed through configuration parsing.
 ///
-class D2ClientConfig {
+class D2ClientConfig : public data::UserContext, public isc::data::CfgToElement {
 public:
     /// @brief Default configuration constants.
-    /// @todo For now these are hard-coded as configuraiton layer cannot
-    /// readily provide them (see Trac #3358).
     static const char* DFT_SERVER_IP;
     static const size_t DFT_SERVER_PORT;
     static const char* DFT_V4_SENDER_IP;
@@ -59,11 +65,12 @@ public:
     static const size_t DFT_MAX_QUEUE_SIZE;
     static const char* DFT_NCR_PROTOCOL;
     static const char* DFT_NCR_FORMAT;
-    static const bool DFT_ALWAYS_INCLUDE_FQDN;
     static const bool DFT_OVERRIDE_NO_UPDATE;
     static const bool DFT_OVERRIDE_CLIENT_UPDATE;
     static const char* DFT_REPLACE_CLIENT_NAME_MODE;
     static const char* DFT_GENERATED_PREFIX;
+    static const char* DFT_HOSTNAME_CHAR_SET;
+    static const char* DFT_HOSTNAME_CHAR_REPLACEMENT;
 
     /// @brief Defines the client name replacement modes.
     enum ReplaceClientNameMode  {
@@ -85,8 +92,6 @@ public:
     /// Currently only UDP is supported.
     /// @param ncr_format Format of the kea-dhcp-ddns requests.
     /// Currently only JSON format is supported.
-    /// @param always_include_fqdn Enables always including the FQDN option in
-    /// DHCP responses.
     /// @param override_no_update Enables updates, even if clients request no
     /// updates.
     /// @param override_client_update Perform updates, even if client requested
@@ -95,6 +100,10 @@ public:
     /// supplied by the client with a generated name.
     /// @param generated_prefix Prefix to use when generating domain-names.
     /// @param qualifying_suffix Suffix to use to qualify partial domain-names.
+    /// @param hostname_char_set regular expression string which describes invalid
+    /// characters to be scrubbed from client host names
+    /// @param hostname_char_replacement string of zero or more characters to
+    /// replace invalid chars when sanitizing client host names
     ///
     /// @c enable_updates is mandatory, @c qualifying_suffix is mandatory
     /// when updates are enabled, other parameters are optional.
@@ -108,12 +117,14 @@ public:
                    const size_t max_queue_size,
                    const dhcp_ddns::NameChangeProtocol& ncr_protocol,
                    const dhcp_ddns::NameChangeFormat& ncr_format,
-                   const bool always_include_fqdn,
                    const bool override_no_update,
                    const bool override_client_update,
                    const ReplaceClientNameMode replace_client_name_mode,
                    const std::string& generated_prefix,
-                   const std::string& qualifying_suffix);
+                   const std::string& qualifying_suffix,
+                   util::Optional<std::string> hostname_char_set,
+                   util::Optional<std::string> hostname_char_replacement);
+
 
     /// @brief Default constructor
     /// The default constructor creates an instance that has updates disabled.
@@ -147,7 +158,7 @@ public:
         return(sender_port_);
     }
 
-    /// @brief Return Maximun sender queue size
+    /// @brief Return Maximum sender queue size
     size_t getMaxQueueSize() const {
         return(max_queue_size_);
     }
@@ -162,11 +173,6 @@ public:
         return(ncr_format_);
     }
 
-    /// @brief Return whether or not FQDN is always included in DHCP responses.
-    bool getAlwaysIncludeFqdn() const {
-        return(always_include_fqdn_);
-    }
-
     /// @brief Return if updates are done even if clients request no updates.
     bool getOverrideNoUpdate() const {
         return(override_no_update_);
@@ -192,6 +198,60 @@ public:
         return(qualifying_suffix_);
     }
 
+    /// @brief Return the char set regexp used to sanitize client hostnames.
+    util::Optional<std::string> getHostnameCharSet() const {
+        if (hostname_char_set_.unspecified() && fetch_globals_fn_) {
+            data::ConstElementPtr globals = fetch_globals_fn_();
+            if (globals && (globals->getType() == data::Element::map)) {
+                data::ConstElementPtr global_param =
+                    globals->get("hostname-char-set");
+                if (global_param) {
+                    return (global_param->stringValue());
+                }
+            }
+        }
+
+        return (hostname_char_set_);
+        }
+
+    /// @brief Return the invalid char replacement used to sanitize client hostnames.
+    util::Optional<std::string> getHostnameCharReplacement() const {
+        if (hostname_char_replacement_.unspecified() && fetch_globals_fn_) {
+            data::ConstElementPtr globals = fetch_globals_fn_();
+            if (globals && (globals->getType() == data::Element::map)) {
+                data::ConstElementPtr global_param =
+                    globals->get("hostname-char-replacement");
+                if (global_param) {
+                    return (global_param->stringValue());
+                }
+            }
+        }
+
+        return (hostname_char_replacement_);
+    }
+
+    /// @brief Return pointer to compiled regular expression string sanitizer
+    /// Will be empty if hostname-char-set is empty.
+    util::str::StringSanitizerPtr getHostnameSanitizer() const {
+        return(hostname_sanitizer_);
+    }
+
+    /// @brief Sets the optional callback function used to fetch globally
+    /// configured parameters.
+    ///
+    /// @param fetch_globals_fn Pointer to the function.
+    void setFetchGlobalsFn(FetchNetworkGlobalsFn fetch_globals_fn) {
+        fetch_globals_fn_ = fetch_globals_fn;
+    }
+
+    /// @brief Checks if the D2 client config is associated with a function
+    /// used to fetch globally configured parameters.
+    ///
+    /// @return true if it is associated, false otherwise.
+    bool hasFetchGlobalsFn() const {
+        return (static_cast<bool>(fetch_globals_fn_));
+    }
+
     /// @brief Compares two D2ClientConfigs for equality
     bool operator == (const D2ClientConfig& other) const;
 
@@ -214,7 +274,7 @@ public:
     ///
     /// @param mode_str text to convert to an enum.
     /// Valid string values: "never", "always", "when-present",
-    /// "when-not-present" (case insensistive)
+    /// "when-not-present" (case-insensitive)
     ///
     /// @return NameChangeFormat value which maps to the given string.
     ///
@@ -230,10 +290,16 @@ public:
     /// "unknown" if not.
     static std::string replaceClientNameModeToString(const ReplaceClientNameMode& mode);
 
-protected:
+    /// @brief Unparse a configuration object
+    ///
+    /// @return a pointer to unparsed configuration
+    virtual isc::data::ElementPtr toElement() const;
+
     /// @brief Validates member values.
     ///
     /// Method is used by the constructor to validate member contents.
+    /// Should be called when parsing is complete to (re)compute
+    /// the hostname sanitizer.
     ///
     /// @throw D2ClientError if given an invalid protocol or format.
     virtual void validateContents();
@@ -254,7 +320,7 @@ private:
     /// @brief IP port on which the client should send
     size_t sender_port_;
 
-    /// @brief Maxium number of NCRs allowed to queue waiting to send
+    /// @brief Maximum number of NCRs allowed to queue waiting to send
     size_t max_queue_size_;
 
     /// @brief The socket protocol to use with kea-dhcp-ddns.
@@ -265,9 +331,6 @@ private:
     /// Currently only JSON format is supported.
     dhcp_ddns::NameChangeFormat ncr_format_;
 
-    /// @brief Should Kea always include the FQDN option in its response.
-    bool always_include_fqdn_;
-
     /// @brief Should Kea perform updates, even if client requested no updates.
     /// Overrides the client request for no updates via the N flag.
     bool override_no_update_;
@@ -283,6 +346,21 @@ private:
 
     /// @brief Suffix Kea should use when to qualify partial domain-names.
     std::string qualifying_suffix_;
+
+    /// @brief Regular expression describing invalid characters for client hostnames.
+    /// If empty, host name scrubbing is not done.
+    util::Optional<std::string> hostname_char_set_;
+
+    /// @brief A string to replace invalid characters when scrubbing hostnames.
+    /// Meaningful only if hostname_char_set_ is not empty.
+    util::Optional<std::string> hostname_char_replacement_;
+
+    /// @brief Pointer to compiled regular expression string sanitizer
+    util::str::StringSanitizerPtr hostname_sanitizer_;
+
+    /// @brief Pointer to the optional callback used to fetch globally
+    /// configured parameters inherited to the @c D2ClientConfig object.
+    FetchNetworkGlobalsFn fetch_globals_fn_;
 };
 
 std::ostream&
