@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,6 +8,8 @@
 #define HOST_H
 
 #include <asiolink/io_address.h>
+#include <cc/data.h>
+#include <cc/user_context.h>
 #include <dhcp/classify.h>
 #include <dhcp/duid.h>
 #include <dhcp/hwaddr.h>
@@ -22,9 +24,131 @@
 namespace isc {
 namespace dhcp {
 
-/// @brief HostID (used only when storing in MySQL or Postgres)
+/// @brief Maximum size of an IPv6 address represented as a text string.
+///
+/// This is 32 hexadecimal characters written in 8 groups of four, plus seven
+/// colon separators.
+const size_t ADDRESS6_TEXT_MAX_LEN = 39;
+
+/// @brief Maximum length of classes stored in a dhcp4/6_client_classes
+/// columns.
+const size_t CLIENT_CLASSES_MAX_LEN = 255;
+
+/// @brief Maximum length of the hostname stored in DNS.
+///
+/// This length is restricted by the length of the domain-name carried
+/// in the Client FQDN %Option (see RFC4702 and RFC4704).
+const size_t HOSTNAME_MAX_LEN = 255;
+
+/// @brief Maximum length of option value.
+const size_t OPTION_VALUE_MAX_LEN = 4096;
+
+/// @brief Maximum length of option value specified in textual format.
+const size_t OPTION_FORMATTED_VALUE_MAX_LEN = 8192;
+
+/// @brief Maximum length of option space name.
+const size_t OPTION_SPACE_MAX_LEN = 128;
+
+/// @brief Maximum length of user context.
+const size_t USER_CONTEXT_MAX_LEN = 8192;
+
+/// @brief Maximum length of the server hostname.
+const size_t SERVER_HOSTNAME_MAX_LEN = 64;
+
+/// @brief Maximum length of the boot file name.
+const size_t BOOT_FILE_NAME_MAX_LEN = 128;
+
+/// @brief Maximum length of authentication keys - 128 bits.
+const uint8_t AUTH_KEY_LEN = 16;
+
+/// @brief Maximum length of authentication keys (coded in hexadecimal).
+const size_t TEXT_AUTH_KEY_LEN = AUTH_KEY_LEN * 2;
+
+/// @brief HostID (used only when storing in MySQL, PostgreSQL or Cassandra)
 typedef uint64_t HostID;
 
+/// @brief Authentication keys.
+///
+/// This class represents authentication keys to be used for
+/// calculating HMAC in the authentication field of the reconfigure message.
+class AuthKey {
+public:
+    /// @brief Constructor.
+    ///
+    /// Constructor for assigning auth keys in host reservation.
+    /// Ensures the key length is not greater than 16 bytes.
+    /// @param key auth key in binary to be stored.
+    AuthKey(const std::vector<uint8_t>& key);
+
+    /// @brief Constructor.
+    ///
+    /// Constructor for assigning auth keys in host reservation.
+    /// Ensures the key length is not greater than AUTH_KEY_LEN (16) bytes
+    /// so TEXT_AUTH_KEY_LEN (32) hexadecimal digits.
+    /// See @c setKey for constraints on its input format.
+    ///
+    /// @param key auth key in hexadecimal to be stored.
+    AuthKey(const std::string& key);
+
+    /// @brief Constructor.
+    ///
+    /// Constructor for generating auth keys, with no argument.
+    /// shall use the internal function for generationg random keys.
+    AuthKey();
+
+    // @brief Get random string.
+    ///
+    /// Random string is generated by default will be used for
+    /// the keys to be used for signing Reconfigure Message.
+    /// @return Random binary string of 16 bytes.
+    static std::vector<uint8_t> getRandomKeyString();
+
+    /// @brief Set auth key value from binary.
+    ///
+    /// Set the key value.
+    //  If the size is greater than 16 bytes, we resize to 16 bytes.
+    /// Doesnt throw an exception.
+    /// @param key auth key in binary to be stored
+    void setAuthKey(const std::vector<uint8_t>& key);
+
+    /// @brief Set auth key value from hexadecimal.
+    ///
+    /// Set the key value.
+    /// If the size is greater than 16 bytes, we resize to 16 bytes.
+    /// @param key auth key in hexadecimal to be stored.
+    /// @throw BadValue if the string is not a valid hexadecimal encoding,
+    /// for instance has a not hexadecimal or odd number of digits.
+    void setAuthKey(const std::string& key);
+
+    /// @brief Return auth key.
+    ///
+    /// @return auth key in binary.
+    const std::vector<uint8_t>& getAuthKey() const {
+        return authKey_;
+    }
+
+    /// @brief Return text format for keys.
+    ///
+    /// @return auth key as a string of hexadecimal digits.
+    std::string toText() const;
+
+    ///
+    /// @brief Equality operator.
+    ///
+    /// equality operator to compare two AuthKey objects.
+    /// @param other Authkey to be compared against.
+    bool operator==(const AuthKey& other) const;
+
+    /// @brief Inequality operator.
+    ///
+    /// inequality operator to compare two AuthKey objects.
+    /// @param other Authkey to be compared against.
+    bool operator!=(const AuthKey& other) const;
+
+private:
+    std::vector<uint8_t> authKey_;
+};
+
 /// @brief IPv6 reservation for a host.
 ///
 /// This class represents a reservation for a host of a single IPv6
@@ -109,7 +233,6 @@ private:
     Type type_;                  ///< Reservation type.
     asiolink::IOAddress prefix_; ///< Prefix
     uint8_t prefix_len_;         ///< Prefix length.
-
 };
 
 /// @brief Collection of IPv6 reservations for the host.
@@ -170,7 +293,7 @@ typedef std::pair<IPv6ResrvIterator, IPv6ResrvIterator> IPv6ResrvRange;
 /// - disable IPv4 reservation without a need to set it to the 0.0.0.0 address
 /// Note that the last three operations are mainly required for managing
 /// host reservations which will be implemented later.
-class Host {
+class Host : public data::UserContext {
 public:
 
     /// @brief Type of the host identifier.
@@ -184,12 +307,13 @@ public:
         IDENT_HWADDR,
         IDENT_DUID,
         IDENT_CIRCUIT_ID,
-        IDENT_CLIENT_ID
+        IDENT_CLIENT_ID,
+        IDENT_FLEX, ///< Flexible host identifier.
     };
 
     /// @brief Constant pointing to the last identifier of the
     /// @ref IdentifierType enumeration.
-    static const IdentifierType LAST_IDENTIFIER_TYPE = IDENT_CLIENT_ID;
+    static const IdentifierType LAST_IDENTIFIER_TYPE = IDENT_FLEX;
 
     /// @brief Constructor.
     ///
@@ -217,6 +341,7 @@ public:
     /// @param next_server IPv4 address of next server (siaddr).
     /// @param server_host_name Server host name (a.k.a. sname).
     /// @param boot_file_name Boot file name (a.k.a. file).
+    /// @param auth_key Authentication key.
     ///
     /// @throw BadValue if the provided values are invalid. In particular,
     /// if the identifier is invalid.
@@ -229,7 +354,8 @@ public:
          const std::string& dhcp6_client_classes = "",
          const asiolink::IOAddress& next_server = asiolink::IOAddress::IPV4_ZERO_ADDRESS(),
          const std::string& server_host_name = "",
-         const std::string& boot_file_name = "");
+         const std::string& boot_file_name = "",
+         const AuthKey& auth_key = AuthKey(""));
 
     /// @brief Constructor.
     ///
@@ -240,7 +366,7 @@ public:
     /// - "yy:yy:yy:yy:yy:yy"
     /// - "yyyyyyyyyy",
     /// - "0xyyyyyyyyyy",
-    /// - "'some identfier'".
+    /// - "'some identifier'".
     /// where y is a hexadecimal digit.
     ///
     /// Note that it is possible to use textual representation, e.g. 'some identifier',
@@ -267,6 +393,7 @@ public:
     /// @param next_server IPv4 address of next server (siaddr).
     /// @param server_host_name Server host name (a.k.a. sname).
     /// @param boot_file_name Boot file name (a.k.a. file).
+    /// @param auth_key Authentication key.
     ///
     /// @throw BadValue if the provided values are invalid. In particular,
     /// if the identifier is invalid.
@@ -278,7 +405,8 @@ public:
          const std::string& dhcp6_client_classes = "",
          const asiolink::IOAddress& next_server = asiolink::IOAddress::IPV4_ZERO_ADDRESS(),
          const std::string& server_host_name = "",
-         const std::string& boot_file_name = "");
+         const std::string& boot_file_name = "",
+         const AuthKey& auth_key = AuthKey(""));
 
     /// @brief Replaces currently used identifier with a new identifier.
     ///
@@ -501,7 +629,7 @@ public:
     /// @brief Returns pointer to the DHCPv4 option data configuration for
     /// this host.
     ///
-    /// Returned pointer can be used to add, remove and udate options
+    /// Returned pointer can be used to add, remove and update options
     /// reserved for a host.
     CfgOptionPtr getCfgOption4() {
         return (cfg_option4_);
@@ -516,7 +644,7 @@ public:
     /// @brief Returns pointer to the DHCPv6 option data configuration for
     /// this host.
     ///
-    /// Returned pointer can be used to add, remove and udate options
+    /// Returned pointer can be used to add, remove and update options
     /// reserved for a host.
     CfgOptionPtr getCfgOption6() {
         return (cfg_option6_);
@@ -531,18 +659,57 @@ public:
     /// @brief Returns information about the host in the textual format.
     std::string toText() const;
 
-    /// @brief Sets Host ID (primary key in MySQL and Postgres backends)
+    /// @brief Sets Host ID (primary key in MySQL, PostgreSQL and Cassandra backends)
     /// @param id HostId value
     void setHostId(HostID id) {
         host_id_ = id;
     }
 
-    /// @brief Returns Host ID (primary key in MySQL and Postgres backends)
+    /// @brief Returns Host ID (primary key in MySQL, PostgreSQL and Cassandra backends)
     /// @return id HostId value (or 0 if not set)
     HostID getHostId() const {
         return (host_id_);
     }
 
+    /// @brief Sets the negative cached flag.
+    ///
+    /// @param negative sets whether this is a negative cached host,
+    /// i.e. a fake host in the cache which indicates non-existence
+    /// and avoids to lookup in a slow backend.
+    void setNegative(bool negative) {
+        negative_ = negative;
+    }
+
+    /// @brief Return the negative cache flag value.
+    /// When true standard lookup methods return null host pointer instead.
+    bool getNegative() const {
+        return (negative_);
+    }
+
+    /// @brief Unparses (converts to Element representation) IPv4 host
+    ///
+    /// @return Element representation of the host
+    isc::data::ElementPtr toElement4() const;
+
+    /// @brief Unparses (converts to Element representation) IPv6 host
+    ///
+    /// @return Element representation of the host
+    isc::data::ElementPtr toElement6() const;
+
+    /// @brief sets key.
+    ///
+    /// Keys are used for signing the Reconfigure Message.
+    void setKey(const AuthKey& key) {
+        key_ = key;
+    }
+
+    /// @brief Returns the key.
+    ///
+    /// Keys are used for signing the Reconfigure Message.
+    AuthKey getKey() const {
+        return(key_);
+    }
+
 private:
 
     /// @brief Adds new client class for DHCPv4 or DHCPv6.
@@ -584,13 +751,28 @@ private:
     std::string boot_file_name_;
 
     /// @brief HostID (a unique identifier assigned when the host is stored in
-    ///                MySQL or Pgsql)
+    ///     MySQL, PostgreSQL or Cassandra)
     uint64_t host_id_;
 
     /// @brief Pointer to the DHCPv4 option data configuration for this host.
     CfgOptionPtr cfg_option4_;
     /// @brief Pointer to the DHCPv6 option data configuration for this host.
     CfgOptionPtr cfg_option6_;
+
+    /// @brief Negative cached flag.
+    ///
+    /// This flag determines whether this object is a negative cache, i.e.
+    /// we queried other backends for specific host and there was no
+    /// entry for it.
+    bool negative_;
+
+    /// @brief key for authentication .
+    ///
+    /// key is a 16 byte value to be used in the authentication field.
+    /// Server replies will contain the below key in authentication field
+    /// as specified in the RFC 8415. While sending reconfigure message
+    /// authentication field shall contain MD5 hash computed using this key.
+    AuthKey key_;
 };
 
 /// @brief Pointer to the @c Host object.
