@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,6 +11,8 @@
 #include <dhcp/duid.h>
 #include <dhcp/option.h>
 #include <dhcp/hwaddr.h>
+#include <cc/user_context.h>
+#include <cc/cfg_to_element.h>
 
 namespace isc {
 namespace dhcp {
@@ -21,11 +23,16 @@ namespace dhcp {
 /// because subnet.h needs Lease::Type, so it includes lease.h
 typedef uint32_t SubnetID;
 
+struct Lease;
+
+/// @brief Pointer to the lease object.
+typedef boost::shared_ptr<Lease> LeasePtr;
+
 /// @brief a common structure for IPv4 and IPv6 leases
 ///
 /// This structure holds all information that is common between IPv4 and IPv6
 /// leases.
-struct Lease {
+struct Lease : public isc::data::UserContext, public isc::data::CfgToElement {
 
     /// @brief Type of lease or pool
     typedef enum {
@@ -37,9 +44,16 @@ struct Lease {
 
     /// @brief returns text representation of a lease type
     /// @param type lease or pool type to be converted
-    /// @return text decription
+    /// @return text description
     static std::string typeToText(Type type);
 
+    /// @brief Converts type name to the actual type.
+    ///
+    /// @param text lease type as text.
+    /// @return converted type.
+    /// @throw BadValue if the text contains unsupported value.
+    static Type textToType(const std::string& text);
+
     /// @name Common lease states constants.
     //@{
     ///
@@ -64,8 +78,6 @@ struct Lease {
     /// @brief Constructor
     ///
     /// @param addr IP address
-    /// @param t1 renewal time
-    /// @param t2 rebinding time
     /// @param valid_lft Lifetime of the lease
     /// @param subnet_id Subnet identification
     /// @param cltt Client last transmission time
@@ -73,7 +85,7 @@ struct Lease {
     /// @param fqdn_rev If true, reverse DNS update is performed for a lease.
     /// @param hostname FQDN of the client which gets the lease.
     /// @param hwaddr Hardware/MAC address
-    Lease(const isc::asiolink::IOAddress& addr, uint32_t t1, uint32_t t2,
+    Lease(const isc::asiolink::IOAddress& addr,
           uint32_t valid_lft, SubnetID subnet_id, time_t cltt,
           const bool fqdn_fwd, const bool fqdn_rev,
           const std::string& hostname,
@@ -87,24 +99,6 @@ struct Lease {
     /// IPv4, IPv6 address or, in the case of a prefix delegation, the prefix.
     isc::asiolink::IOAddress addr_;
 
-    /// @brief Renewal timer
-    ///
-    /// Specifies renewal time. Although technically it is a property of the
-    /// IA container and not the address itself, since our data model does not
-    /// define a separate IA entity, we are keeping it in the lease. In the
-    /// case of multiple addresses/prefixes for the same IA, each must have
-    /// consistent T1 and T2 values. This is specified in seconds since cltt.
-    uint32_t t1_;
-
-    /// @brief Rebinding timer
-    ///
-    /// Specifies rebinding time. Although technically it is a property of the
-    /// IA container and not the address itself, since our data model does not
-    /// define a separate IA entity, we are keeping it in the lease. In the
-    /// case of multiple addresses/prefixes for the same IA, each must have
-    /// consistent T1 and T2 values. This is specified in seconds since cltt.
-    uint32_t t2_;
-
     /// @brief Valid lifetime
     ///
     /// Expressed as number of seconds since cltt.
@@ -144,7 +138,7 @@ struct Lease {
     /// @brief Holds the lease state(s).
     ///
     /// This is the field that holds the lease state(s). Typically, a
-    /// lease remains in a single states. However, it is posible to
+    /// lease remains in a single states. However, it is possible to
     /// define a value for state which indicates that the lease remains
     /// in multiple logical states.
     ///
@@ -174,6 +168,8 @@ struct Lease {
 
     /// @brief Returns true if the other lease has equal FQDN data.
     ///
+    /// The comparison of the hostname is case insensitive.
+    ///
     /// @param other Lease which FQDN data is to be compared with our lease.
     ///
     /// @return Boolean value which indicates whether FQDN data of the other
@@ -199,16 +195,38 @@ struct Lease {
     /// @brief Sets lease to DECLINED state.
     ///
     /// All client identifying parameters will be stripped off (HWaddr,
-    /// client_id, hostname), timers set to 0 (t1, t2), cltt will be set
-    /// to current time and valid_lft to parameter specified as probation
-    /// period. Note that This method only sets fields in the structure.
+    /// client_id, hostname), cltt will be set to current time and
+    /// valid_lft to parameter specified as probation period.
+    /// Note that This method only sets fields in the structure.
     /// It is caller's responsibility to clean up DDNS, bump up stats,
     /// log, call hooks ets.
     ///
     /// @param probation_period lease lifetime will be set to this value
     virtual void decline(uint32_t probation_period) = 0;
+
+    /// Avoid a clang spurious error
+    using isc::data::CfgToElement::toElement;
+
+protected:
+
+    /// @brief Sets common (for v4 and v6) properties of the lease object.
+    ///
+    /// This method is called by the @c fromElement methods of the @c Lease
+    /// class derivations.
+    ///
+    /// @param [out] lease pointer to the lease object for which common
+    /// properties should be set.
+    /// @param element pointer to the data element object to be parsed.
+    static void fromElementCommon(const LeasePtr& lease,
+                                  const data::ConstElementPtr& element);
+
 };
 
+struct Lease4;
+
+/// @brief Pointer to a Lease4 structure.
+typedef boost::shared_ptr<Lease4> Lease4Ptr;
+
 /// @brief Structure that holds a lease for IPv4 address
 ///
 /// For performance reasons it is a simple structure, not a class. If we chose
@@ -230,8 +248,6 @@ struct Lease4 : public Lease {
     /// @param clientid Client identification buffer
     /// @param clientid_len Length of client identification buffer
     /// @param valid_lft Lifetime of the lease
-    /// @param t1 renewal time
-    /// @param t2 rebinding time
     /// @param cltt Client last transmission time
     /// @param subnet_id Subnet identification
     /// @param fqdn_fwd If true, forward DNS update is performed for a lease.
@@ -239,10 +255,10 @@ struct Lease4 : public Lease {
     /// @param hostname FQDN of the client which gets the lease.
     Lease4(const isc::asiolink::IOAddress& addr, const HWAddrPtr& hwaddr,
            const uint8_t* clientid, size_t clientid_len, uint32_t valid_lft,
-           uint32_t t1, uint32_t t2, time_t cltt, uint32_t subnet_id,
+           time_t cltt, uint32_t subnet_id,
            const bool fqdn_fwd = false, const bool fqdn_rev = false,
            const std::string& hostname = "")
-        : Lease(addr, t1, t2, valid_lft, subnet_id, cltt, fqdn_fwd, fqdn_rev,
+        : Lease(addr, valid_lft, subnet_id, cltt, fqdn_fwd, fqdn_rev,
                 hostname, hwaddr) {
         if (clientid_len) {
             client_id_.reset(new ClientId(clientid, clientid_len));
@@ -255,8 +271,6 @@ struct Lease4 : public Lease {
     /// @param hw_address Pointer to client's HW addresss.
     /// @param client_id  pointer to the client id structure.
     /// @param valid_lifetime Valid lifetime value.
-    /// @param t1 Renew timer.
-    /// @param t2 Rebind timer.
     /// @param cltt Timestamp when the lease is acquired, renewed.
     /// @param subnet_id Subnet identifier.
     /// @param fqdn_fwd Forward DNS update performed.
@@ -266,8 +280,6 @@ struct Lease4 : public Lease {
            const HWAddrPtr& hw_address,
            const ClientIdPtr& client_id,
            const uint32_t valid_lifetime,
-           const uint32_t t1,
-           const uint32_t t2,
            const time_t cltt,
            const SubnetID subnet_id,
            const bool fqdn_fwd = false,
@@ -278,7 +290,7 @@ struct Lease4 : public Lease {
     /// @brief Default constructor
     ///
     /// Initialize fields that don't have a default constructor.
-    Lease4() : Lease(0, 0, 0, 0, 0, 0, false, false, "", HWAddrPtr())
+    Lease4() : Lease(0, 0, 0, 0, false, false, "", HWAddrPtr())
     {
     }
 
@@ -390,7 +402,7 @@ struct Lease4 : public Lease {
 
     /// @brief Convert lease to printable form
     ///
-    /// @return Textual represenation of lease data
+    /// @return Textual representation of lease data
     virtual std::string toText() const;
 
     /// @brief Sets IPv4 lease to declined state.
@@ -400,15 +412,30 @@ struct Lease4 : public Lease {
     /// @param probation_period valid lifetime will be set to this value
     void decline(uint32_t probation_period);
 
+    /// @brief Return the JSON representation of a lease
+    virtual isc::data::ElementPtr toElement() const;
+
+    /// @brief Returns pointer to the IPv4 lease created from JSON
+    /// representation.
+    ///
+    /// @param element pointer to the data element object to be parsed.
+    /// @return Pointer to the created lease.
+    static Lease4Ptr fromElement(const data::ConstElementPtr& element);
+
     /// @todo: Add DHCPv4 failover related fields here
 };
 
-/// @brief Pointer to a Lease4 structure.
-typedef boost::shared_ptr<Lease4> Lease4Ptr;
-
 /// @brief A collection of IPv4 leases.
 typedef std::vector<Lease4Ptr> Lease4Collection;
 
+/// @brief A shared pointer to the collection of IPv4 leases.
+typedef boost::shared_ptr<Lease4Collection> Lease4CollectionPtr;
+
+struct Lease6;
+
+/// @brief Pointer to a Lease6 structure.
+typedef boost::shared_ptr<Lease6> Lease6Ptr;
+
 /// @brief Structure that holds a lease for IPv6 address and/or prefix
 ///
 /// For performance reasons it is a simple structure, not a class. If we chose
@@ -452,14 +479,12 @@ struct Lease6 : public Lease {
     /// @param iaid IAID.
     /// @param preferred Preferred lifetime.
     /// @param valid Valid lifetime.
-    /// @param t1 A value of the T1 timer.
-    /// @param t2 A value of the T2 timer.
     /// @param subnet_id A Subnet identifier.
     /// @param hwaddr hardware/MAC address (optional)
     /// @param prefixlen An address prefix length (optional, defaults to 128)
     Lease6(Lease::Type type, const isc::asiolink::IOAddress& addr, DuidPtr duid,
-           uint32_t iaid, uint32_t preferred, uint32_t valid, uint32_t t1,
-           uint32_t t2, SubnetID subnet_id, const HWAddrPtr& hwaddr = HWAddrPtr(),
+           uint32_t iaid, uint32_t preferred, uint32_t valid,
+           SubnetID subnet_id, const HWAddrPtr& hwaddr = HWAddrPtr(),
            uint8_t prefixlen = 128);
 
     /// @brief Constructor, including FQDN data.
@@ -470,8 +495,6 @@ struct Lease6 : public Lease {
     /// @param iaid IAID.
     /// @param preferred Preferred lifetime.
     /// @param valid Valid lifetime.
-    /// @param t1 A value of the T1 timer.
-    /// @param t2 A value of the T2 timer.
     /// @param subnet_id A Subnet identifier.
     /// @param fqdn_fwd If true, forward DNS update is performed for a lease.
     /// @param fqdn_rev If true, reverse DNS update is performed for a lease.
@@ -479,8 +502,8 @@ struct Lease6 : public Lease {
     /// @param hwaddr hardware address (MAC), may be NULL
     /// @param prefixlen An address prefix length (optional, defaults to 128)
     Lease6(Lease::Type type, const isc::asiolink::IOAddress& addr, DuidPtr duid,
-           uint32_t iaid, uint32_t preferred, uint32_t valid, uint32_t t1,
-           uint32_t t2, SubnetID subnet_id, const bool fqdn_fwd,
+           uint32_t iaid, uint32_t preferred, uint32_t valid,
+           SubnetID subnet_id, const bool fqdn_fwd,
            const bool fqdn_rev, const std::string& hostname,
            const HWAddrPtr& hwaddr = HWAddrPtr(), uint8_t prefixlen = 128);
 
@@ -532,10 +555,17 @@ struct Lease6 : public Lease {
     ///
     /// @return String form of the lease
     virtual std::string toText() const;
-};
 
-/// @brief Pointer to a Lease6 structure.
-typedef boost::shared_ptr<Lease6> Lease6Ptr;
+    /// @brief Return the JSON representation of a lease
+    virtual isc::data::ElementPtr toElement() const;
+
+    /// @brief Returns pointer to the IPv6 lease created from JSON
+    /// representation.
+    ///
+    /// @param element pointer to the data element object to be parsed.
+    /// @return Pointer to the created lease.
+    static Lease6Ptr fromElement(const data::ConstElementPtr& element);
+};
 
 /// @brief Pointer to a const Lease6 structure.
 typedef boost::shared_ptr<const Lease6> ConstLease6Ptr;
@@ -543,12 +573,16 @@ typedef boost::shared_ptr<const Lease6> ConstLease6Ptr;
 /// @brief A collection of IPv6 leases.
 typedef std::vector<Lease6Ptr> Lease6Collection;
 
+
+/// @brief A shared pointer to the collection of IPv6 leases.
+typedef boost::shared_ptr<Lease6Collection> Lease6CollectionPtr;
+
 /// @brief Stream output operator.
 ///
 /// Dumps the output of Lease::toText to the given stream.
 /// @param os output stream to which the output is
 /// @param lease reference to Lease object to dump
-/// @return a reference to the output stream paramater
+/// @return a reference to the output stream parameter
 std::ostream&
 operator<<(std::ostream& os, const Lease& lease);
 
