@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,7 @@
 #ifndef DHCPSRV_CONFIG_H
 #define DHCPSRV_CONFIG_H
 
+#include <cc/cfg_to_element.h>
 #include <dhcpsrv/cfg_db_access.h>
 #include <dhcpsrv/cfg_duid.h>
 #include <dhcpsrv/cfg_expiration.h>
@@ -16,12 +17,18 @@
 #include <dhcpsrv/cfg_option.h>
 #include <dhcpsrv/cfg_option_def.h>
 #include <dhcpsrv/cfg_rsoo.h>
+#include <dhcpsrv/cfg_shared_networks.h>
 #include <dhcpsrv/cfg_subnets4.h>
 #include <dhcpsrv/cfg_subnets6.h>
 #include <dhcpsrv/cfg_mac_source.h>
+#include <dhcpsrv/cfg_consistency.h>
 #include <dhcpsrv/client_class_def.h>
-#include <dhcpsrv/logging_info.h>
+#include <dhcpsrv/d2_client_cfg.h>
+#include <process/config_base.h>
+#include <hooks/hooks_config.h>
 #include <cc/data.h>
+#include <cc/user_context.h>
+#include <cc/simple_parser.h>
 #include <boost/shared_ptr.hpp>
 #include <vector>
 #include <stdint.h>
@@ -35,7 +42,7 @@ class CfgMgr;
 /// @brief Specifies current DHCP configuration
 ///
 /// @todo Migrate all other configuration parameters from cfgmgr.h here
-class SrvConfig {
+class SrvConfig : public process::ConfigBase {
 public:
     /// @name Constants for selection of parameters returned by @c getConfigSummary
     ///
@@ -54,10 +61,14 @@ public:
     static const uint32_t CFGSEL_DDNS    = 0x00000010;
     /// Number of all subnets
     static const uint32_t CFGSEL_SUBNET  = 0x00000003;
+    /// Configured globals
+    static const uint32_t CFGSEL_GLOBALS = 0x00000020;
+    /// Config control info
+    static const uint32_t CFGSEL_CFG_CTL = 0x00000040;
     /// IPv4 related config
-    static const uint32_t CFGSEL_ALL4    = 0x00000015;
+    static const uint32_t CFGSEL_ALL4    = 0x00000035;
     /// IPv6 related config
-    static const uint32_t CFGSEL_ALL6    = 0x0000001A;
+    static const uint32_t CFGSEL_ALL6    = 0x0000003A;
     /// Whole config
     static const uint32_t CFGSEL_ALL     = 0xFFFFFFFF;
     //@}
@@ -109,25 +120,6 @@ public:
     /// @return true if sequence numbers are equal.
     bool sequenceEquals(const SrvConfig& other);
 
-    /// @name Modifiers and accesors for the configuration objects.
-    ///
-    /// @warning References to the objects returned by accessors are only
-    /// valid during the lifetime of the @c SrvConfig object which
-    /// returned them.
-    ///
-    //@{
-    /// @brief Returns logging specific configuration.
-    const LoggingInfoStorage& getLoggingInfo() const {
-        return (logging_info_);
-    }
-
-    /// @brief Sets logging specific configuration.
-    ///
-    /// @param logging_info New logging configuration.
-    void addLoggingInfo(const LoggingInfo& logging_info) {
-        logging_info_.push_back(logging_info);
-    }
-
     /// @brief Returns non-const pointer to interface configuration.
     ///
     /// This function returns a non-const pointer to the interface
@@ -198,6 +190,24 @@ public:
         return (cfg_subnets4_);
     }
 
+    /// @brief Returns pointer to non-const object holding configuration of
+    /// shared networks in DHCPv4;
+    ///
+    /// @return Pointer to the object holding shared networks configuration
+    /// for DHCPv4.
+    CfgSharedNetworks4Ptr getCfgSharedNetworks4() const {
+        return (cfg_shared_networks4_);
+    }
+
+    /// @brief Returns pointer to non-const object holding configuration of
+    /// shared networks in DHCPv6.
+    ///
+    /// @return Pointer to the object holding shared networks configuration
+    /// for DHCPv6.
+    CfgSharedNetworks6Ptr getCfgSharedNetworks6() const {
+        return (cfg_shared_networks6_);
+    }
+
     /// @brief Returns pointer to const object holding subnets configuration for
     /// DHCPv4.
     ///
@@ -316,6 +326,11 @@ public:
         return (cfg_host_operations6_);
     }
 
+    /// @brief Returns const pointer to object holding sanity checks flags
+    CfgConsistencyPtr getConsistency() {
+        return (cfg_consist_);
+    }
+
     //@}
 
     /// @brief Returns non-const reference to an array that stores
@@ -346,6 +361,18 @@ public:
         control_socket_ = control_socket;
     }
 
+    /// @brief Returns DHCP queue control information
+    /// @return pointer to the DHCP queue control information
+    const isc::data::ConstElementPtr getDHCPQueueControl() const {
+        return (dhcp_queue_control_);
+    }
+
+    /// @brief Sets information about the dhcp queue control
+    /// @param dhcp_queue_control new dhcp queue control information
+    void setDHCPQueueControl(const isc::data::ConstElementPtr dhcp_queue_control) {
+        dhcp_queue_control_ = dhcp_queue_control;
+    }
+
     /// @brief Returns pointer to the dictionary of global client
     /// class definitions
     ClientClassDictionaryPtr getClientClassDictionary() {
@@ -364,7 +391,21 @@ public:
         class_dictionary_ = dictionary;
     }
 
-    /// @brief Copies the currnet configuration to a new configuration.
+    /// @brief Returns non-const reference to configured hooks libraries.
+    ///
+    /// @return non-const reference to configured hooks libraries.
+    isc::hooks::HooksConfig& getHooksConfig() {
+        return (hooks_config_);
+    }
+
+    /// @brief Returns const reference to configured hooks libraries.
+    ///
+    /// @return const reference to configured hooks libraries.
+    const isc::hooks::HooksConfig& getHooksConfig() const {
+        return (hooks_config_);
+    }
+
+    /// @brief Copies the current configuration to a new configuration.
     ///
     /// This method copies the parameters stored in the configuration to
     /// an object passed as parameter. The configuration sequence is not
@@ -376,15 +417,12 @@ public:
     /// the default copy constructors can't be used. Implementing this
     /// requires quite a lot of time so this is left as is for now.
     /// The lack of ability to copy the entire configuration makes
-    /// revert function of the @c CfgMgr unsuable.
+    /// revert function of the @c CfgMgr unusable.
     ///
     /// @param [out] new_config An object to which the configuration will
     /// be copied.
     void copy(SrvConfig& new_config) const;
 
-    /// @brief Apply logging configuration to log4cplus.
-    void applyLoggingCfg() const;
-
     /// @name Methods and operators used to compare configurations.
     ///
     //@{
@@ -423,7 +461,7 @@ public:
         return (equals(other));
     }
 
-    /// @param other An object to be compared with this object.
+    /// @brief other An object to be compared with this object.
     ///
     /// It ignores the configuration sequence number when checking for
     /// inequality of objects.
@@ -437,6 +475,54 @@ public:
 
     //@}
 
+    /// @brief Merges the configuration specified as a parameter into
+    /// this configuration.
+    ///
+    /// This method is used when two or more configurations held in the
+    /// @c SrvConfig objects need to be combined into a single configuration.
+    /// Specifically, when the configuration backend is used, then part of
+    /// the server configuration comes from the configuration file and
+    /// stored in the staging configuration. The other part of the
+    /// configuration comes from the database. The configuration fetched
+    /// from the database is stored in a separate @c SrvConfig instance
+    /// and then merged into the staging configuration prior to commiting
+    /// it.
+    ///
+    /// The merging strategy depends on the underlying data being merged.
+    /// For example: subnets are merged using the algorithm implemented
+    /// in the @c CfgSubnets4. Other data structures are merged using the
+    /// algorithms implemented in their respective configuration
+    /// containers.
+    ///
+    /// The general rule is that the configuration data from the @c other
+    /// object replaces configuration data held in this object instance.
+    /// The data that do not overlap between the two objects is simply
+    /// inserted into this configuration.
+    ///
+    /// @warning The call to @c merge may modify the data in the @c other
+    /// object. Therefore, the caller must not rely on the data held
+    /// in the @c other object after the call to @c merge. Also, the
+    /// data held in @c other must not be modified after the call to
+    /// @c merge because it may affect the merged configuration.
+    ///
+    /// The @c other parameter must be a @c SrvConfig or its derivation.
+    ///
+    /// This method calls either @c merge4 or @c merge6 based on
+    ///
+    /// Currently, the following parts of the configuration are merged:
+    /// - globals
+    /// - option definitions
+    /// - options
+    /// - via @c merge4 or @c merge6 depending on @c CfgMgr::family_:
+    ///     - shared networks
+    ///     - subnets
+    ///
+    /// @todo Add support for merging other configuration elements.
+    ///
+    /// @param other An object holding the configuration to be merged
+    /// into this configuration.
+    virtual void merge(ConfigBase& other);
+
     /// @brief Updates statistics.
     ///
     /// This method calls appropriate methods in child objects that update
@@ -469,32 +555,136 @@ public:
         return (decline_timer_);
     }
 
+    /// @brief Sets whether server should send back client-id in DHCPv4
+    ///
+    /// This is a compatibility flag. The default (true) is compliant with
+    /// RFC6842. False is for backward compatibility.
+    ///
+    /// @param echo should the client-id be sent or not
+    void setEchoClientId(const bool echo) {
+        echo_v4_client_id_ = echo;
+    }
+
+    /// @brief Returns whether server should send back client-id in DHCPv4.
+    /// @return true if client-id should be returned, false otherwise.
+    bool getEchoClientId() const {
+        return (echo_v4_client_id_);
+    }
+
     /// @brief Sets DHCP4o6 IPC port
     ///
     /// DHCPv4-over-DHCPv6 uses a UDP socket for interserver communication,
     /// this socket is bound and connected to this port and port + 1
     ///
     /// @param port port and port + 1 to use
-    void setDhcp4o6Port(uint32_t port) {
+    void setDhcp4o6Port(uint16_t port) {
         dhcp4o6_port_ = port;
     }
 
     /// @brief Returns DHCP4o6 IPC port
     ///
-    /// See @ref setDhcp4o6Port or brief discussion.                         
+    /// See @ref setDhcp4o6Port for brief discussion.
     /// @return value of DHCP4o6 IPC port
-    uint32_t getDhcp4o6Port() {
+    uint16_t getDhcp4o6Port() const {
         return (dhcp4o6_port_);
     }
 
+    /// @brief Returns pointer to the D2 client configuration
+    D2ClientConfigPtr getD2ClientConfig() {
+        return (d2_client_config_);
+    }
+
+    /// @brief Returns pointer to const D2 client configuration
+    const D2ClientConfigPtr getD2ClientConfig() const {
+        return (d2_client_config_);
+    }
+
+    /// @brief Sets the D2 client configuration
+    /// @param d2_client_config pointer to the new D2 client configuration
+    void setD2ClientConfig(const D2ClientConfigPtr& d2_client_config) {
+        d2_client_config_ = d2_client_config;
+    }
+
+    /// @brief Returns pointer to configured global parameters
+    isc::data::ConstElementPtr getConfiguredGlobals() const {
+        return (isc::data::ConstElementPtr(configured_globals_));
+    }
+
+    /// @brief Removes all configured global parameters.
+    /// @note This removes the default values too so either
+    /// @c applyDefaultsConfiguredGlobals and @c mergeGlobals,
+    /// or @c isc::data::SimpleParser::setDefaults and
+    /// @c extractConfiguredGlobals should be called after.
+    void clearConfiguredGlobals();
+
+    /// @brief Applies defaults to global parameters.
+    /// @param defaults vector of (name, type, value) defaults to apply.
+    void applyDefaultsConfiguredGlobals(const isc::data::SimpleDefaults& defaults);
+
+    /// @brief Saves scalar elements from the global scope of a configuration
+    void extractConfiguredGlobals(isc::data::ConstElementPtr config);
+
+    /// @brief Adds a parameter to the collection configured globals
+    /// @param name std::string name of the global to add
+    /// @param value ElementPtr containing the value of the global
+    void addConfiguredGlobal(const std::string& name, isc::data::ConstElementPtr value) {
+        configured_globals_->set(name, value);
+    }
+
+    /// @brief Unparse a configuration object
+    ///
+    /// @return a pointer to unparsed configuration
+    virtual isc::data::ElementPtr toElement() const;
+
 private:
 
+    /// @brief Merges the DHCPv4 configuration specified as a parameter into
+    /// this configuration.
+    ///
+    /// This is called by @c merge() to handle v4 specifics, such as
+    /// networks and subnets.
+    ///
+    /// @param other An object holding the configuration to be merged
+    /// into this configuration.
+    void merge4(SrvConfig& other);
+
+    /// @brief Merges the DHCPv6 configuration specified as a parameter into
+    /// this configuration.
+    ///
+    /// This is called by @c merge() to handle v4 specifics, such as
+    /// networks and subnets.
+    ///
+    /// @param other An object holding the configuration to be merged
+    /// into this configuration.
+    void merge6(SrvConfig& other);
+
+    /// @brief Merges the globals specified in the given configuration
+    /// into this configuration.
+    ///
+    /// Configurable global values may be specified either via JSON
+    /// configuration (e.g. "echo-client-id":true) or as global parameters
+    /// within a configuration back end.  Regardless of the source, these
+    /// values once provided, are stored in @c SrvConfig::configured_globals_.
+    /// Any such value that does not have an explicit specification should be
+    /// considered "unspecified" at the global scope.
+    ///
+    /// This function adds the configured globals from the "other" config
+    /// into this config's configured globals.  If a value already exists
+    /// in this config, it will be overwritten with the value from the
+    /// "other" config.
+    ///
+    /// It then iterates over this merged list of globals, setting
+    /// any of the corresponding SrvConfig members that map to a
+    /// a configurable parameter (e.g. c@ SrvConfig::echo_client_id_,
+    /// @c SrvConfig::server_tag_).
+    ///
+    /// @param other An object holding the configuration to be merged
+    /// into this configuration.
+    void mergeGlobals(SrvConfig& other);
+
     /// @brief Sequence number identifying the configuration.
     uint32_t sequence_;
 
-    /// @brief Logging specific information.
-    LoggingInfoStorage logging_info_;
-
     /// @brief Interface configuration.
     ///
     /// Used to select interfaces on which the DHCP server will listen to
@@ -519,6 +709,12 @@ private:
     /// @brief Pointer to subnets configuration for IPv6.
     CfgSubnets6Ptr cfg_subnets6_;
 
+    /// @brief Pointer to IPv4 shared networks configuration.
+    CfgSharedNetworks4Ptr cfg_shared_networks4_;
+
+    /// @brief Pointer to IPv4 shared networks configuration.
+    CfgSharedNetworks6Ptr cfg_shared_networks6_;
+
     /// @brief Pointer to the configuration for hosts reservation.
     ///
     /// This object holds a list of @c Host objects representing host
@@ -556,20 +752,38 @@ private:
     /// @brief Pointer to the control-socket information
     isc::data::ConstElementPtr control_socket_;
 
+    /// @brief Pointer to the dhcp-queue-control information
+    isc::data::ConstElementPtr dhcp_queue_control_;
+
     /// @brief Pointer to the dictionary of global client class definitions
     ClientClassDictionaryPtr class_dictionary_;
 
+    /// @brief Configured hooks libraries.
+    isc::hooks::HooksConfig hooks_config_;
+
     /// @brief Decline Period time
     ///
     /// This timer specifies decline probation period, the time after a declined
     /// lease is recovered back to available state. Expressed in seconds.
     uint32_t decline_timer_;
 
+    /// @brief Indicates whether v4 server should send back client-id
+    bool echo_v4_client_id_;
+
     /// @brief DHCP4o6 IPC port
     ///
     /// DHCPv4-over-DHCPv6 uses a UDP socket for interserver communication,
     /// this socket is bound and connected to this port and port + 1
-    uint32_t dhcp4o6_port_;
+    uint16_t dhcp4o6_port_;
+
+    /// @brief Stores D2 client configuration
+    D2ClientConfigPtr d2_client_config_;
+
+    /// @brief Stores the global parameters specified via configuration
+    isc::data::ElementPtr configured_globals_;
+
+    /// @brief Pointer to the configuration consistency settings
+    CfgConsistencyPtr cfg_consist_;
 };
 
 /// @name Pointers to the @c SrvConfig object.
