@@ -1,7 +1,8 @@
-// Copyright (C) 2015 - 2016 Deutsche Telekom AG.
+// Copyright (C) 2016-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2017 Deutsche Telekom AG.
 //
-// Author: Razvan Becheriu <razvan.becheriu@qualitance.com>
-// Author: Andrei Pavel <andrei.pavel@qualitance.com>
+// Authors: Razvan Becheriu <razvan.becheriu@qualitance.com>
+//          Andrei Pavel <andrei.pavel@qualitance.com>
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -18,12 +19,12 @@
 #include <config.h>
 
 #include <asiolink/io_address.h>
+#include <cql/cql_connection.h>
+#include <cql/testutils/cql_schema.h>
 #include <dhcpsrv/lease_mgr_factory.h>
-#include <dhcpsrv/cql_connection.h>
 #include <dhcpsrv/cql_lease_mgr.h>
 #include <dhcpsrv/tests/test_utils.h>
 #include <dhcpsrv/tests/generic_lease_mgr_unittest.h>
-#include <dhcpsrv/testutils/cql_schema.h>
 #include <exceptions/exceptions.h>
 
 #include <gtest/gtest.h>
@@ -36,12 +37,15 @@
 
 using namespace isc;
 using namespace isc::asiolink;
+using namespace isc::db;
+using namespace isc::db::test;
 using namespace isc::dhcp;
 using namespace isc::dhcp::test;
 using namespace std;
 
 namespace {
 
+
 /// @brief Test fixture class for testing Cassandra Lease Manager
 ///
 /// Opens the database prior to each test and closes it afterwards.
@@ -49,14 +53,10 @@ namespace {
 
 class CqlLeaseMgrTest : public GenericLeaseMgrTest {
 public:
-    /// @brief Constructor
-    ///
-    /// Deletes everything from the database and opens it.
-    CqlLeaseMgrTest() {
-
-        // Ensure schema is the correct one.
-        destroyCqlSchema(false, true);
-        createCqlSchema(false, true);
+    /// @brief Clears the database and opens connection to it.
+    void initializeTest() {
+        // Ensure we have the proper schema with no transient data.
+        createCqlSchema();
 
         // Connect to the database
         try {
@@ -69,59 +69,78 @@ public:
                          "*** accompanying exception output.\n";
             throw;
         }
+
         lmptr_ = &(LeaseMgrFactory::instance());
     }
 
+    /// @brief Destroys the LM and the schema.
+    void destroyTest() {
+        try {
+            lmptr_->rollback();
+        } catch (...) {
+            // Rollback may fail if backend is in read only mode. That's ok.
+        }
+        LeaseMgrFactory::destroy();
+        // If data wipe enabled, delete transient data otherwise destroy the schema
+        destroyCqlSchema();
+    }
+
+    /// @brief Constructor
+    ///
+    /// Deletes everything from the database and opens it.
+    CqlLeaseMgrTest() {
+        initializeTest();
+    }
+
     /// @brief Destructor
     ///
-    /// Rolls back all pending transactions. The deletion of lmptr_ will close
-    /// the database. Then reopen it and delete everything created by the test.
+    /// Rolls back all pending transactions.  The deletion of lmptr_ will close
+    /// the database.  Then reopen it and delete everything created by the test.
     virtual ~CqlLeaseMgrTest() {
-        lmptr_->rollback();
-        LeaseMgrFactory::destroy();
-        destroyCqlSchema(false, true);
+        destroyTest();
     }
 
     /// @brief Reopen the database
     ///
-    /// Closes the database and re-open it. Anything committed should be
+    /// Closes the database and re-open it.  Anything committed should be
     /// visible.
     ///
-    /// Parameter is ignored for CQL backend as the v4 and v6 leases share the
-    /// same keyspace.
+    /// Parameter is ignored for CQL backend as the v4 and v6 leases share
+    /// the same database.
     void reopen(Universe) {
         LeaseMgrFactory::destroy();
         LeaseMgrFactory::create(validCqlConnectionString());
         lmptr_ = &(LeaseMgrFactory::instance());
     }
 
-    // This is the CQL implementation for GenericLeaseMgrTest::testGetExpiredLeases4().
+    // This is the CQL implementation for
+    // GenericLeaseMgrTest::testGetExpiredLeases4().
     // The GenericLeaseMgrTest implementation checks for the order of expired
     // leases to be from the most expired to the least expired. Cassandra
     // doesn't support ORDER BY without imposing a EQ / IN restriction on the
     // columns. Because of that, the order check has been excluded.
-    void
-    testCqlGetExpiredLeases4() {
+    void testCqlGetExpiredLeases4() {
         // Get the leases to be used for the test.
-        vector<Lease4Ptr> leases = createLeases4();
+        std::vector<Lease4Ptr> leases = createLeases4();
         // Make sure we have at least 6 leases there.
-        ASSERT_GE(leases.size(), 6U);
+        ASSERT_GE(leases.size(), 6u);
 
         // Use the same current time for all leases.
         time_t current_time = time(NULL);
 
         // Add them to the database
-        for (size_t i = 0U; i < leases.size(); ++i) {
+        for (size_t i = 0u; i < leases.size(); ++i) {
             // Mark every other lease as expired.
-            if (i % 2U == 0U) {
+            if (i % 2u == 0u) {
                 // Set client last transmission time to the value older than the
                 // valid lifetime to make it expired. The expiration time also
                 // depends on the lease index, so as we can later check that the
                 // leases are ordered by the expiration time.
-                leases[i]->cltt_ = current_time - leases[i]->valid_lft_ - 10 - i;
-
+                leases[i]->cltt_ =
+                    current_time - leases[i]->valid_lft_ - 10 - i;
             } else {
-                // Set current time as cltt for remaining leases. These leases are
+                // Set current time as cltt for remaining leases. These leases
+                // are
                 // not expired.
                 leases[i]->cltt_ = current_time;
             }
@@ -131,20 +150,24 @@ public:
         // Retrieve at most 1000 expired leases.
         Lease4Collection expired_leases;
         ASSERT_NO_THROW(lmptr_->getExpiredLeases4(expired_leases, 1000));
+
         // Leases with even indexes should be returned as expired.
-        ASSERT_EQ(static_cast<size_t>(leases.size() / 2U), expired_leases.size());
+        ASSERT_EQ(static_cast<size_t>(leases.size() / 2u),
+                  expired_leases.size());
 
         // Update current time for the next test.
         current_time = time(NULL);
         // Also, remove expired leases collected during the previous test.
         expired_leases.clear();
 
-        // This time let's reverse the expiration time and see if they will be returned
+        // This time let's reverse the expiration time and see if they will be
+        // returned
         // in the correct order.
-        for (size_t i = 0U; i < leases.size(); ++i) {
+        for (size_t i = 0u; i < leases.size(); ++i) {
             // Update the time of expired leases with even indexes.
-            if (i % 2U == 0U) {
-                leases[i]->cltt_ = current_time - leases[i]->valid_lft_ - 1000 + i;
+            if (i % 2u == 0u) {
+                leases[i]->cltt_ =
+                    current_time - leases[i]->valid_lft_ - 1000 + i;
             } else {
                 // Make sure remaining leases remain unexpired.
                 leases[i]->cltt_ = current_time + 100;
@@ -152,11 +175,13 @@ public:
             ASSERT_NO_THROW(lmptr_->updateLease4(leases[i]));
         }
 
-        // Retrieve expired leases again. The limit of 0 means return all expired
-        // leases.
+        // Retrieve expired leases again. The limit of 0 means return all
+        // expired leases.
         ASSERT_NO_THROW(lmptr_->getExpiredLeases4(expired_leases, 0));
+
         // The same leases should be returned.
-        ASSERT_EQ(static_cast<size_t>(leases.size() / 2U), expired_leases.size());
+        ASSERT_EQ(static_cast<size_t>(leases.size() / 2u),
+                  expired_leases.size());
 
         // Remember expired leases returned.
         std::vector<Lease4Ptr> saved_expired_leases = expired_leases;
@@ -168,12 +193,13 @@ public:
         ASSERT_NO_THROW(lmptr_->getExpiredLeases4(expired_leases, 2));
 
         // Make sure we have exactly 2 leases returned.
-        ASSERT_EQ(2U, expired_leases.size());
+        ASSERT_EQ(2u, expired_leases.size());
 
         // Mark every other expired lease as reclaimed.
-        for (size_t i = 0U; i < saved_expired_leases.size(); ++i) {
-            if (i % 2U != 0U) {
-                saved_expired_leases[i]->state_ = Lease::STATE_EXPIRED_RECLAIMED;
+        for (size_t i = 0u; i < saved_expired_leases.size(); ++i) {
+            if (i % 2u != 0u) {
+                saved_expired_leases[i]->state_ =
+                    Lease::STATE_EXPIRED_RECLAIMED;
             }
             ASSERT_NO_THROW(lmptr_->updateLease4(saved_expired_leases[i]));
         }
@@ -182,46 +208,48 @@ public:
 
         // This the returned leases should exclude reclaimed ones. So the number
         // of returned leases should be roughly half of the expired leases.
-        ASSERT_NO_THROW(lmptr_->getExpiredLeases4(expired_leases, 0U));
-        ASSERT_EQ(static_cast<size_t>(saved_expired_leases.size() / 2U),
+        ASSERT_NO_THROW(lmptr_->getExpiredLeases4(expired_leases, 0u));
+        ASSERT_EQ(static_cast<size_t>(saved_expired_leases.size() / 2u),
                   expired_leases.size());
 
         // Make sure that returned leases are those that are not reclaimed, i.e.
         // those that have even index.
         for (Lease4Collection::iterator lease = expired_leases.begin();
              lease != expired_leases.end(); ++lease) {
-            int index = static_cast<int>(std::distance(expired_leases.begin(), lease));
+            int32_t index = static_cast<int32_t>(
+                std::distance(expired_leases.begin(), lease));
             EXPECT_EQ(saved_expired_leases[2 * index]->addr_, (*lease)->addr_);
         }
     }
 
-    // This is the CQL implementation for GenericLeaseMgrTest::testGetExpiredLeases4().
+    // This is the CQL implementation for
+    // GenericLeaseMgrTest::testGetExpiredLeases6().
     // The GenericLeaseMgrTest implementation checks for the order of expired
     // leases to be from the most expired to the least expired. Cassandra
     // doesn't support ORDER BY without imposing a EQ / IN restriction on the
     // columns. Because of that, the order check has been excluded.
-    void
-    testCqlGetExpiredLeases6() {
+    void testCqlGetExpiredLeases6() {
         // Get the leases to be used for the test.
-        vector<Lease6Ptr> leases = createLeases6();
+        std::vector<Lease6Ptr> leases = createLeases6();
         // Make sure we have at least 6 leases there.
-        ASSERT_GE(leases.size(), 6U);
+        ASSERT_GE(leases.size(), 6u);
 
         // Use the same current time for all leases.
         time_t current_time = time(NULL);
 
         // Add them to the database
-        for (size_t i = 0U; i < leases.size(); ++i) {
+        for (size_t i = 0u; i < leases.size(); ++i) {
             // Mark every other lease as expired.
-            if (i % 2U == 0U) {
+            if (i % 2u == 0u) {
                 // Set client last transmission time to the value older than the
                 // valid lifetime to make it expired. The expiration time also
                 // depends on the lease index, so as we can later check that the
                 // leases are ordered by the expiration time.
-                leases[i]->cltt_ = current_time - leases[i]->valid_lft_ - 10 - i;
-
+                leases[i]->cltt_ =
+                    current_time - leases[i]->valid_lft_ - 10 - i;
             } else {
-                // Set current time as cltt for remaining leases. These leases are
+                // Set current time as cltt for remaining leases. These leases
+                // are
                 // not expired.
                 leases[i]->cltt_ = current_time;
             }
@@ -231,21 +259,24 @@ public:
         // Retrieve at most 1000 expired leases.
         Lease6Collection expired_leases;
         ASSERT_NO_THROW(lmptr_->getExpiredLeases6(expired_leases, 1000));
+
         // Leases with even indexes should be returned as expired.
-        ASSERT_EQ(static_cast<size_t>(leases.size() / 2U), expired_leases.size());
+        ASSERT_EQ(static_cast<size_t>(leases.size() / 2u),
+                  expired_leases.size());
 
         // Update current time for the next test.
         current_time = time(NULL);
         // Also, remove expired leases collected during the previous test.
         expired_leases.clear();
 
-        // This time let's reverse the expiration time and see if they will be returned
+        // This time let's reverse the expiration time and see if they will be
+        // returned
         // in the correct order.
-        for (size_t i = 0U; i < leases.size(); ++i) {
+        for (size_t i = 0u; i < leases.size(); ++i) {
             // Update the time of expired leases with even indexes.
-            if (i % 2U == 0U) {
-                leases[i]->cltt_ = current_time - leases[i]->valid_lft_ - 1000 + i;
-
+            if (i % 2u == 0u) {
+                leases[i]->cltt_ =
+                    current_time - leases[i]->valid_lft_ - 1000 + i;
             } else {
                 // Make sure remaining leases remain unexpired.
                 leases[i]->cltt_ = current_time + 100;
@@ -253,11 +284,13 @@ public:
             ASSERT_NO_THROW(lmptr_->updateLease6(leases[i]));
         }
 
-        // Retrieve expired leases again. The limit of 0 means return all expired
-        // leases.
+        // Retrieve expired leases again. The limit of 0 means return all
+        // expired leases.
         ASSERT_NO_THROW(lmptr_->getExpiredLeases6(expired_leases, 0));
+
         // The same leases should be returned.
-        ASSERT_EQ(static_cast<size_t>(leases.size() / 2U), expired_leases.size());
+        ASSERT_EQ(static_cast<size_t>(leases.size() / 2u),
+                  expired_leases.size());
 
         // Remember expired leases returned.
         std::vector<Lease6Ptr> saved_expired_leases = expired_leases;
@@ -269,12 +302,13 @@ public:
         ASSERT_NO_THROW(lmptr_->getExpiredLeases6(expired_leases, 2));
 
         // Make sure we have exactly 2 leases returned.
-        ASSERT_EQ(2U, expired_leases.size());
+        ASSERT_EQ(2u, expired_leases.size());
 
         // Mark every other expired lease as reclaimed.
-        for (size_t i = 0U; i < saved_expired_leases.size(); ++i) {
-            if (i % 2U != 0U) {
-                saved_expired_leases[i]->state_ = Lease::STATE_EXPIRED_RECLAIMED;
+        for (size_t i = 0u; i < saved_expired_leases.size(); ++i) {
+            if (i % 2u != 0u) {
+                saved_expired_leases[i]->state_ =
+                    Lease::STATE_EXPIRED_RECLAIMED;
             }
             ASSERT_NO_THROW(lmptr_->updateLease6(saved_expired_leases[i]));
         }
@@ -289,7 +323,8 @@ public:
         // those that have even index.
         for (Lease6Collection::iterator lease = expired_leases.begin();
              lease != expired_leases.end(); ++lease) {
-            int index = static_cast<int>(std::distance(expired_leases.begin(), lease));
+            int32_t index = static_cast<int32_t>(
+                std::distance(expired_leases.begin(), lease));
             EXPECT_EQ(saved_expired_leases[2 * index]->addr_, (*lease)->addr_);
         }
     }
@@ -297,23 +332,21 @@ public:
 
 /// @brief Check that database can be opened
 ///
-/// This test checks if the CqlLeaseMgr can be instantiated. This happens
-/// only if the database can be opened. Note that this is not part of the
-/// CqlLeaseMgr test fixure set. This test checks that the database can be
+/// This test checks if the CqlLeaseMgr can be instantiated.  This happens
+/// only if the database can be opened.  Note that this is not part of the
+/// CqlLeaseMgr test fixure set.  This test checks that the database can be
 /// opened: the fixtures assume that and check basic operations.
-/// Unlike other backend implementations, this one doesn't check for lacking
-/// parameters. In that scenario, Cassandra defaults to configured parameters.
+
 TEST(CqlOpenTest, OpenDatabase) {
 
     // Schema needs to be created for the test to work.
-    destroyCqlSchema(false, true);
-    createCqlSchema(false, true);
+    createCqlSchema();
 
     // Check that lease manager open the database opens correctly and tidy up.
     // If it fails, print the error message.
     try {
         LeaseMgrFactory::create(validCqlConnectionString());
-        EXPECT_NO_THROW((void) LeaseMgrFactory::instance());
+        EXPECT_NO_THROW((void)LeaseMgrFactory::instance());
         LeaseMgrFactory::destroy();
     } catch (const isc::Exception& ex) {
         FAIL() << "*** ERROR: unable to open database, reason:\n"
@@ -323,10 +356,12 @@ TEST(CqlOpenTest, OpenDatabase) {
     }
 
     // Check that lease manager open the database opens correctly with a longer
-    // timeout. If it fails, print the error message.
+    // timeout.  If it fails, print the error message.
     try {
+        // CQL specifies the timeout values in ms, not seconds. Therefore
+        // we need to add extra 000 to the "connect-timeout=10" string.
         string connection_string = validCqlConnectionString() + string(" ") +
-                                   string(VALID_TIMEOUT);
+            string(VALID_TIMEOUT) + "000";
         LeaseMgrFactory::create(connection_string);
         EXPECT_NO_THROW((void) LeaseMgrFactory::instance());
         LeaseMgrFactory::destroy();
@@ -352,8 +387,63 @@ TEST(CqlOpenTest, OpenDatabase) {
         INVALID_TYPE, VALID_NAME, VALID_HOST, VALID_USER, VALID_PASSWORD)),
         InvalidType);
 
+    // Check that invalid login data does not cause an exception, CQL should use
+    // default values.
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, INVALID_NAME, VALID_HOST, VALID_USER, VALID_PASSWORD)));
+
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, INVALID_HOST, VALID_USER, VALID_PASSWORD)));
+
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, INVALID_USER, VALID_PASSWORD)));
+
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, VALID_USER, INVALID_PASSWORD)));
+
+    // Check for invalid timeouts
+    EXPECT_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, VALID_USER, VALID_PASSWORD, INVALID_TIMEOUT_1)),
+        DbOperationError);
+
+    EXPECT_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, VALID_USER, VALID_PASSWORD, INVALID_TIMEOUT_2)),
+        DbOperationError);
+
+    // Check for missing parameters
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, NULL, VALID_HOST, INVALID_USER, VALID_PASSWORD)));
+
+    // Check that invalid login data does not cause an exception, CQL should use
+    // default values.
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, INVALID_NAME, VALID_HOST, VALID_USER, VALID_PASSWORD)));
+
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, INVALID_HOST, VALID_USER, VALID_PASSWORD)));
+
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, INVALID_USER, VALID_PASSWORD)));
+
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, VALID_USER, INVALID_PASSWORD)));
+
+    // Check that invalid timeouts throw DbOperationError.
+    EXPECT_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, VALID_USER, VALID_PASSWORD,
+        INVALID_TIMEOUT_1)),
+        DbOperationError);
+    EXPECT_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, VALID_NAME, VALID_HOST, VALID_USER, VALID_PASSWORD,
+        INVALID_TIMEOUT_2)),
+        DbOperationError);
+
+    // Check that CQL allows the hostname to not be specified.
+    EXPECT_NO_THROW(LeaseMgrFactory::create(connectionString(
+        CQL_VALID_TYPE, NULL, VALID_HOST, INVALID_USER, VALID_PASSWORD)));
+
     // Tidy up after the test
-    destroyCqlSchema(false, true);
+    destroyCqlSchema();
 }
 
 /// @brief Check the getType() method
@@ -366,7 +456,7 @@ TEST_F(CqlLeaseMgrTest, getType) {
 
 /// @brief Check conversion functions
 ///
-/// The server works using cltt and valid_filetime. In the database, the
+/// The server works using cltt and valid_filetime.  In the database, the
 /// information is stored as expire_time and valid-lifetime, which are
 /// related by
 ///
@@ -375,10 +465,10 @@ TEST_F(CqlLeaseMgrTest, getType) {
 /// This test checks that the conversion is correct.
 TEST_F(CqlLeaseMgrTest, checkTimeConversion) {
     const time_t cltt = time(NULL);
-    const uint32_t valid_lft = 86400;       // 1 day
-    uint64_t cql_expire;
+    const uint32_t valid_lft = 86400;  // 1 day
+    cass_int64_t cql_expire;
 
-    // Convert to the database time
+    // Convert to the database time.
     CqlExchange::convertToDatabaseTime(cltt, valid_lft, cql_expire);
 
     // Convert back
@@ -387,7 +477,7 @@ TEST_F(CqlLeaseMgrTest, checkTimeConversion) {
     EXPECT_EQ(cltt, converted_cltt);
 }
 
-/// @brief Check getName() returns correct keyspace name.
+/// @brief Check getName() returns correct database name
 TEST_F(CqlLeaseMgrTest, getName) {
     EXPECT_EQ(std::string("keatest"), lmptr_->getName());
 }
@@ -487,10 +577,25 @@ TEST_F(CqlLeaseMgrTest, getLease4ClientIdSubnetId) {
     testGetLease4ClientIdSubnetId();
 }
 
+// This test checks that all IPv4 leases for a specified subnet id are returned.
+TEST_F(CqlLeaseMgrTest, getLeases4SubnetId) {
+    testGetLeases4SubnetId();
+}
+
+// This test checks that all IPv4 leases are returned.
+TEST_F(CqlLeaseMgrTest, getLeases4) {
+    testGetLeases4();
+}
+
+// Test that a range of IPv4 leases is returned with paging.
+TEST_F(CqlLeaseMgrTest, getLeases4Paged) {
+    testGetLeases4Paged();
+}
+
 /// @brief Basic Lease4 Checks
 ///
-/// Checks that the addLease, getLease4(by address), getLease4(hwaddr, subnet_id),
-/// updateLease4() and deleteLease (IPv4 address) can handle NULL client-id.
+/// Checks that the addLease, getLease4(by address), getLease4(hwaddr,subnet_id),
+/// updateLease4() and deleteLease can handle NULL client-id.
 /// (client-id is optional and may not be present)
 TEST_F(CqlLeaseMgrTest, lease4NullClientId) {
     testLease4NullClientId();
@@ -504,6 +609,22 @@ TEST_F(CqlLeaseMgrTest, lease4InvalidHostname) {
     testLease4InvalidHostname();
 }
 
+/// @brief Check that the expired DHCPv4 leases can be retrieved.
+///
+/// This test adds a number of leases to the lease database and marks
+/// some of them as expired. Then it queries for expired leases and checks
+/// whether only expired leases are returned, and that they are returned in
+/// the order from most to least expired. It also checks that the lease
+/// which is marked as 'reclaimed' is not returned.
+TEST_F(CqlLeaseMgrTest, getExpiredLeases4) {
+    testCqlGetExpiredLeases4();
+}
+
+/// @brief Check that expired reclaimed DHCPv4 leases are removed.
+TEST_F(CqlLeaseMgrTest, deleteExpiredReclaimedLeases4) {
+    testDeleteExpiredReclaimedLeases4();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 /// LEASE6 /////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
@@ -511,7 +632,7 @@ TEST_F(CqlLeaseMgrTest, lease4InvalidHostname) {
 // Test checks whether simple add, get and delete operations are possible
 // on Lease6
 TEST_F(CqlLeaseMgrTest, testAddGetDelete6) {
-    testAddGetDelete6(false);
+    testAddGetDelete6();
 }
 
 /// @brief Basic Lease6 Checks
@@ -535,6 +656,14 @@ TEST_F(CqlLeaseMgrTest, lease6InvalidHostname) {
     testLease6InvalidHostname();
 }
 
+/// @brief Verify that large IAID values work correctly.
+///
+/// Adds lease with a large IAID to the database and verifies it can
+/// fetched correclty.
+TEST_F(CqlLeaseMgrTest, leases6LargeIaidCheck) {
+    testLease6LargeIaidCheck();
+}
+
 /// @brief Check GetLease6 methods - access by DUID/IAID
 ///
 /// Adds leases to the database and checks that they can be accessed via
@@ -558,6 +687,14 @@ TEST_F(CqlLeaseMgrTest, lease6LeaseTypeCheck) {
     testLease6LeaseTypeCheck();
 }
 
+/// @brief Verifies the getLeases6(DUID) method
+///
+/// Adds 3 lease and verifies fetch by DUID.
+/// Verifies retrival of non existant DUID fails
+TEST_F(CqlLeaseMgrTest, getLeases6Duid) {
+    testGetLeases6Duid();
+}
+
 /// @brief Check GetLease6 methods - access by DUID/IAID/SubnetID
 ///
 /// Adds leases to the database and checks that they can be accessed via
@@ -571,6 +708,11 @@ TEST_F(CqlLeaseMgrTest, getLease6DuidIaidSubnetIdSize) {
     testGetLease6DuidIaidSubnetIdSize();
 }
 
+// Test that a range of IPv6 leases is returned with paging.
+TEST_F(CqlLeaseMgrTest, getLeases6Paged) {
+    testGetLeases6Paged();
+}
+
 /// @brief Lease6 update tests
 ///
 /// Checks that we are able to update a lease in the database.
@@ -601,27 +743,16 @@ TEST_F(CqlLeaseMgrTest, nullDuid) {
     testNullDuid();
 }
 
-/// @brief Tests whether memfile can store and retrieve hardware addresses
+/// @brief Tests whether CQL can store and retrieve hardware addresses
 TEST_F(CqlLeaseMgrTest, testLease6Mac) {
     testLease6MAC();
 }
 
-/// @brief Tests whether memfile can store and retrieve hardware addresses
+/// @brief Tests whether CQL can store and retrieve hardware addresses
 TEST_F(CqlLeaseMgrTest, testLease6HWTypeAndSource) {
     testLease6HWTypeAndSource();
 }
 
-/// @brief Check that the expired DHCPv4 leases can be retrieved.
-///
-/// This test adds a number of leases to the lease database and marks
-/// some of them as expired. Then it queries for expired leases and checks
-/// whether only expired leases are returned, and that they are returned in
-/// the order from most to least expired. It also checks that the lease
-/// which is marked as 'reclaimed' is not returned.
-TEST_F(CqlLeaseMgrTest, getExpiredLeases4) {
-    testCqlGetExpiredLeases4();
-}
-
 /// @brief Check that the expired DHCPv6 leases can be retrieved.
 ///
 /// This test adds a number of leases to the lease database and marks
@@ -638,9 +769,38 @@ TEST_F(CqlLeaseMgrTest, deleteExpiredReclaimedLeases6) {
     testDeleteExpiredReclaimedLeases6();
 }
 
-/// @brief Check that expired reclaimed DHCPv4 leases are removed.
-TEST_F(CqlLeaseMgrTest, deleteExpiredReclaimedLeases4) {
-    testDeleteExpiredReclaimedLeases4();
+/// @brief Verifies that IPv4 lease statistics can be recalculated.
+TEST_F(CqlLeaseMgrTest, recountLeaseStats4) {
+    testRecountLeaseStats4();
 }
 
-}; // Of anonymous namespace
+/// @brief Verifies that IPv6 lease statistics can be recalculated.
+TEST_F(CqlLeaseMgrTest, recountLeaseStats6) {
+    testRecountLeaseStats6();
+}
+
+/// @brief Tests that leases from specific subnet can be removed.
+/// @todo: uncomment this once lease wipe is implemented
+/// for Cassandra (see #5485)
+TEST_F(CqlLeaseMgrTest, DISABLED_wipeLeases4) {
+    testWipeLeases4();
+}
+
+/// @brief Tests that leases from specific subnet can be removed.
+/// @todo: uncomment this once lease wipe is implemented
+/// for Cassandra (see #5485)
+TEST_F(CqlLeaseMgrTest, DISABLED_wipeLeases6) {
+    testWipeLeases6();
+}
+
+// Tests v4 lease stats query variants.
+TEST_F(CqlLeaseMgrTest, leaseStatsQuery4) {
+    testLeaseStatsQuery4();
+}
+
+// Tests v6 lease stats query variants.
+TEST_F(CqlLeaseMgrTest, leaseStatsQuery6) {
+    testLeaseStatsQuery6();
+}
+
+}  // namespace
