@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,12 +9,14 @@
 #include <dhcp/duid.h>
 #include <dhcpsrv/csv_lease_file4.h>
 #include <dhcpsrv/lease.h>
-#include <dhcpsrv/tests/lease_file_io.h>
+#include <dhcpsrv/testutils/lease_file_io.h>
 #include <gtest/gtest.h>
+#include <ctime>
 #include <sstream>
 
 using namespace isc;
 using namespace isc::asiolink;
+using namespace isc::data;
 using namespace isc::dhcp;
 using namespace isc::dhcp::test;
 using namespace isc::util;
@@ -100,12 +102,14 @@ CSVLeaseFile4Test::absolutePath(const std::string& filename) {
 void
 CSVLeaseFile4Test::writeSampleFile() const {
     io_.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state\n"
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
                   "192.0.2.1,06:07:08:09:0a:bc,,200,200,8,1,1,"
-                  "host.example.com,0\n"
-                  "192.0.2.1,,a:11:01:04,200,200,8,1,1,host.example.com,1\n"
-                  "192.0.3.15,dd:de:ba:0d:1b:2e:3e:4f,0a:00:01:04,100,100,7,"
-                  "0,0,,1\n");
+                  "host.example.com,0,\n"
+                  "192.0.2.2,,,200,200,8,1,1,host.example.com,0,\n"
+                  "192.0.2.3,dd:de:ba:0d:1b:2e:3e:4f,0a:00:01:04,100,100,7,"
+                  "0,0,,1,{ \"foobar\": true }\n"
+                  "192.0.2.4,,11:22:33:44:55:66,200,200,8,1,1,host.example.com,0,\n"
+                  "192.0.2.5,,,200,200,8,1,1,,1,\n");
 }
 
 // This test checks the capability to read and parse leases from the file.
@@ -143,12 +147,15 @@ TEST_F(CSVLeaseFile4Test, parse) {
     EXPECT_TRUE(lease->fqdn_rev_);
     EXPECT_EQ("host.example.com", lease->hostname_);
     EXPECT_EQ(Lease::STATE_DEFAULT, lease->state_);
+    EXPECT_FALSE(lease->getContext());
     }
 
-    // Second lease is malformed - HW address is empty.
+    // Second lease is malformed - has no HW address or client id and state
+    // is not declined.
     {
     SCOPED_TRACE("Second lease malformed");
     EXPECT_FALSE(lf.next(lease));
+    EXPECT_FALSE(lease);
     checkStats(lf, 2, 1, 1, 0, 0, 0);
     }
 
@@ -161,7 +168,7 @@ TEST_F(CSVLeaseFile4Test, parse) {
     checkStats(lf, 3, 2, 1, 0, 0, 0);
 
     // Verify that the third lease is correct.
-    EXPECT_EQ("192.0.3.15", lease->addr_.toText());
+    EXPECT_EQ("192.0.2.3", lease->addr_.toText());
     HWAddr hwaddr3(*lease->hwaddr_);
     EXPECT_EQ("dd:de:ba:0d:1b:2e:3e:4f", hwaddr3.toText(false));
     ASSERT_TRUE(lease->client_id_);
@@ -173,23 +180,53 @@ TEST_F(CSVLeaseFile4Test, parse) {
     EXPECT_FALSE(lease->fqdn_rev_);
     EXPECT_TRUE(lease->hostname_.empty());
     EXPECT_EQ(Lease::STATE_DECLINED, lease->state_);
+    ASSERT_TRUE(lease->getContext());
+    EXPECT_EQ("{ \"foobar\": true }", lease->getContext()->str());
+    }
+
+    // Fourth lease has no hardware address but has client id
+    {
+    SCOPED_TRACE("Fourth lease valid");
+    EXPECT_TRUE(lf.next(lease));
+    ASSERT_TRUE(lease);
+    checkStats(lf, 4, 3, 1, 0, 0, 0);
+
+    EXPECT_EQ("192.0.2.4", lease->addr_.toText());
+    ASSERT_TRUE(lease->hwaddr_);
+    EXPECT_TRUE(lease->hwaddr_->hwaddr_.empty());
+    ASSERT_TRUE(lease->client_id_);
+    EXPECT_EQ("11:22:33:44:55:66", lease->client_id_->toText());
+    }
+
+    // Fifth lease has no hardware address or client id but is declined
+    {
+    SCOPED_TRACE("Fifth lease valid");
+    EXPECT_TRUE(lf.next(lease));
+    ASSERT_TRUE(lease);
+    checkStats(lf, 5, 4, 1, 0, 0, 0);
+
+    EXPECT_EQ("192.0.2.5", lease->addr_.toText());
+    ASSERT_TRUE(lease->hwaddr_);
+    EXPECT_TRUE(lease->hwaddr_->hwaddr_.empty());
+    ASSERT_FALSE(lease->client_id_);
+    EXPECT_EQ(lease->state_, Lease::STATE_DECLINED);
     }
 
     // There are no more leases. Reading should cause no error, but the returned
     // lease pointer should be NULL.
     {
-    SCOPED_TRACE("Fifth read empty");
+    SCOPED_TRACE("Sixth read empty");
     EXPECT_TRUE(lf.next(lease));
     EXPECT_FALSE(lease);
-    checkStats(lf, 4, 2, 1, 0, 0, 0);
+    checkStats(lf, 6, 4, 1, 0, 0, 0);
     }
 
     // We should be able to do it again.
     {
-    SCOPED_TRACE("Sixth read empty");
+    SCOPED_TRACE("Seventh read empty");
     EXPECT_TRUE(lf.next(lease));
     EXPECT_FALSE(lease);
-    checkStats(lf, 5, 2, 1, 0, 0, 0);
+    checkStats(lf, 7, 4, 1, 0, 0, 0);
     }
 }
 
@@ -206,7 +243,7 @@ TEST_F(CSVLeaseFile4Test, recreate) {
     Lease4Ptr lease(new Lease4(IOAddress("192.0.3.2"),
                                hwaddr0_,
                                NULL, 0,
-                               200, 50, 80, 0, 8, true, true,
+                               200, 0, 8, true, true,
                                "host.example.com"));
     lease->state_ = Lease::STATE_EXPIRED_RECLAIMED;
     {
@@ -215,11 +252,12 @@ TEST_F(CSVLeaseFile4Test, recreate) {
     checkStats(lf, 0, 0, 0, 1, 1, 0);
     }
 
-    // Create second lease, with non-NULL client id.
+    // Create second lease, with non-NULL client id and user context.
     lease.reset(new Lease4(IOAddress("192.0.3.10"),
                            hwaddr1_,
                            CLIENTID, sizeof(CLIENTID),
-                           100, 60, 90, 0, 7));
+                           100, 0, 7));
+    lease->setContext(Element::fromJSON("{ \"foobar\": true }"));
     {
     SCOPED_TRACE("Second write");
     ASSERT_NO_THROW(lf.append(*lease));
@@ -230,10 +268,11 @@ TEST_F(CSVLeaseFile4Test, recreate) {
     lf.close();
     // Check that the contents of the csv file are correct.
     EXPECT_EQ("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-              "fqdn_fwd,fqdn_rev,hostname,state\n"
-              "192.0.3.2,00:01:02:03:04:05,,200,200,8,1,1,host.example.com,2\n"
+              "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+              "192.0.3.2,00:01:02:03:04:05,,200,200,8,1,1,host.example.com,"
+              "2,\n"
               "192.0.3.10,0d:0e:0a:0d:0b:0e:0e:0f,01:02:03:04,100,100,7,0,"
-              "0,,0\n",
+              "0,,0,{ \"foobar\": true }\n",
               io_.readFile());
 }
 
@@ -251,9 +290,9 @@ TEST_F(CSVLeaseFile4Test, mixedSchemaload) {
                   // schema 2.0 record - has state
                   "192.0.2.2,06:07:08:09:2a:bc,,200,200,8,1,1,"
                   "two.example.com,1\n"
-                  // schema 2.0 record - has state
+                  // schema 2.1 record - has state and user context
                   "192.0.2.3,06:07:08:09:3a:bc,,200,200,8,1,1,"
-                  "three.example.com,2\n"
+                  "three.example.com,2,{ \"foobar\": true }\n"
                    );
 
     // Open the lease file.
@@ -281,6 +320,7 @@ TEST_F(CSVLeaseFile4Test, mixedSchemaload) {
     EXPECT_EQ("one.example.com", lease->hostname_);
     // Verify that added state is DEFAULT
     EXPECT_EQ(Lease::STATE_DEFAULT, lease->state_);
+    EXPECT_FALSE(lease->getContext());
     }
 
     {
@@ -300,6 +340,7 @@ TEST_F(CSVLeaseFile4Test, mixedSchemaload) {
     EXPECT_TRUE(lease->fqdn_rev_);
     EXPECT_EQ("two.example.com", lease->hostname_);
     EXPECT_EQ(Lease::STATE_DECLINED, lease->state_);
+    EXPECT_FALSE(lease->getContext());
     }
 
     {
@@ -319,6 +360,8 @@ TEST_F(CSVLeaseFile4Test, mixedSchemaload) {
     EXPECT_TRUE(lease->fqdn_rev_);
     EXPECT_EQ("three.example.com", lease->hostname_);
     EXPECT_EQ(Lease::STATE_EXPIRED_RECLAIMED, lease->state_);
+    ASSERT_TRUE(lease->getContext());
+    EXPECT_EQ("{ \"foobar\": true }", lease->getContext()->str());
     }
 }
 
@@ -340,7 +383,7 @@ TEST_F(CSVLeaseFile4Test, tooFewHeaderColumns) {
 TEST_F(CSVLeaseFile4Test, invalidHeaderColumn) {
     // Create 1.0 file
     io_.writeFile("address,hwaddr,BOGUS,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state\n");
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n");
 
     // Open the lease file.
     CSVLeaseFile4 lf(filename_);
@@ -352,10 +395,10 @@ TEST_F(CSVLeaseFile4Test, invalidHeaderColumn) {
 TEST_F(CSVLeaseFile4Test, downGrade) {
     // Create 2.0 PLUS a column file
     io_.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state,FUTURE_COL\n"
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context,FUTURE_COL\n"
 
                   "192.0.2.3,06:07:08:09:3a:bc,,200,200,8,1,1,"
-                  "three.example.com,2,BOGUS\n");
+                  "three.example.com,2,,BOGUS\n");
 
     // Lease file should open and report as needing downgrade.
     CSVLeaseFile4 lf(filename_);
@@ -382,14 +425,142 @@ TEST_F(CSVLeaseFile4Test, downGrade) {
     EXPECT_TRUE(lease->fqdn_rev_);
     EXPECT_EQ("three.example.com", lease->hostname_);
     EXPECT_EQ(Lease::STATE_EXPIRED_RECLAIMED, lease->state_);
+    EXPECT_FALSE(lease->getContext());
     }
 }
 
+// Verifies that leases with no hardware address are only permitted
+// if they are in the declined state.
+TEST_F(CSVLeaseFile4Test, declinedLeaseTest) {
+    io_.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                  "192.0.2.1,,,200,200,8,1,1,host.example.com,0,\n"
+                  "192.0.2.1,,,200,200,8,1,1,host.example.com,1,\n");
+
+    CSVLeaseFile4 lf(filename_);
+    ASSERT_NO_THROW(lf.open());
+    EXPECT_FALSE(lf.needsConversion());
+    EXPECT_EQ(util::VersionedCSVFile::CURRENT, lf.getInputSchemaState());
+    Lease4Ptr lease;
+
+    {
+    SCOPED_TRACE("No hardware and not declined, invalid");
+    EXPECT_FALSE(lf.next(lease));
+    ASSERT_FALSE(lease);
+    EXPECT_EQ(lf.getReadErrs(),1);
+    }
+
+    {
+    SCOPED_TRACE("No hardware and declined, valid");
+    EXPECT_TRUE(lf.next(lease));
+    ASSERT_TRUE(lease);
+    EXPECT_EQ(lf.getReadErrs(),1);
+    }
+}
+
+// Verifies that it is possible to output a lease with very high valid
+// lifetime (infinite in RFC2131 terms) and current time, and then read
+// back this lease.
+TEST_F(CSVLeaseFile4Test, highLeaseLifetime) {
+    CSVLeaseFile4 lf(filename_);
+    ASSERT_NO_THROW(lf.recreate());
+    ASSERT_TRUE(io_.exists());
+
+    // Write lease with very high lease lifetime and current time.
+    Lease4Ptr lease(new Lease4(IOAddress("192.0.3.2"),
+                               hwaddr0_,
+                               NULL, 0,
+                               0xFFFFFFFF, time(0),
+                               8, true, true,
+                               "host.example.com"));
+    // Write this lease out to the lease file.
+    ASSERT_NO_THROW(lf.append(*lease));
+
+    // Close the lease file.
+    lf.close();
+
+    Lease4Ptr lease_read;
+
+    // Re-open the file for reading.
+    ASSERT_NO_THROW(lf.open());
+
+    // Read the lease and make sure it is successful.
+    EXPECT_TRUE(lf.next(lease_read));
+    ASSERT_TRUE(lease_read);
+
+    // The valid lifetime and the cltt should match with the original lease.
+    EXPECT_EQ(lease->valid_lft_, lease_read->valid_lft_);
+    EXPECT_EQ(lease->cltt_, lease_read->cltt_);
+}
+
+// Verifies that it is not possible to output a lease with empty hwaddr in other
+// than the declined state
+TEST_F(CSVLeaseFile4Test, emptyHWAddrDefaultStateOnly) {
+    CSVLeaseFile4 lf(filename_);
+    ASSERT_NO_THROW(lf.recreate());
+    ASSERT_TRUE(io_.exists());
+
+    HWAddrPtr hwaddr;
+
+    // Create lease with null hwaddr and default state
+    Lease4Ptr lease_null_hwaddr(new Lease4(IOAddress("192.0.3.2"),
+                                hwaddr,
+                                NULL, 0,
+                                0xFFFFFFFF, time(0),
+                                8, true, true,
+                                "host.example.com"));
+    // Try to write this lease out to the lease file.
+    ASSERT_THROW(lf.append(*lease_null_hwaddr), BadValue);
+
+    hwaddr.reset(new HWAddr());
+
+    // Create lease with empty hwaddr and default state
+    Lease4Ptr lease_empty_hwaddr(new Lease4(IOAddress("192.0.3.2"),
+                                 hwaddr,
+                                 NULL, 0,
+                                 0xFFFFFFFF, time(0),
+                                 8, true, true,
+                                 "host.example.com"));
+    // Try to write this lease out to the lease file.
+    ASSERT_THROW(lf.append(*lease_empty_hwaddr), BadValue);
+
+    // Create lease with hwaddr and current time.
+    Lease4Ptr lease(new Lease4(IOAddress("192.0.3.2"),
+                               hwaddr0_,
+                               NULL, 0,
+                               0xFFFFFFFF, time(0),
+                               8, true, true,
+                               "host.example.com"));
+
+    // Decline the lease
+    lease->decline(1000);
+    ASSERT_TRUE(lease->hwaddr_);
+    EXPECT_EQ(lease->hwaddr_->toText(false), "");
+
+    // Write this lease out to the lease file.
+    ASSERT_NO_THROW(lf.append(*lease));
+
+    // Close the lease file.
+    lf.close();
+
+    Lease4Ptr lease_read;
+
+    // Re-open the file for reading.
+    ASSERT_NO_THROW(lf.open());
+
+    // Read the lease and make sure it is successful.
+    EXPECT_TRUE(lf.next(lease_read));
+    ASSERT_TRUE(lease_read);
+
+    // The valid lifetime and the cltt should match with the original lease.
+    EXPECT_EQ(lease->valid_lft_, lease_read->valid_lft_);
+    EXPECT_EQ(lease->cltt_, lease_read->cltt_);
+}
 
 /// @todo Currently we don't check invalid lease attributes, such as invalid
 /// lease type, invalid preferred lifetime vs valid lifetime etc. The Lease6
 /// should be extended with the function that validates lease attributes. Once
 /// this is implemented we should provide more tests for malformed leases
-/// in the CSV file. See http://kea.isc.org/ticket/2405.
+/// in the CSV file. See http://oldkea.isc.org/ticket/2405.
 
 } // end of anonymous namespace
