@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -67,7 +67,8 @@ public:
     ///
     /// @param address Address to use for the initialization
     ///
-    /// @return Lease6Ptr.  This will not point to anything if the initialization
+    /// @return Lease6Ptr.  This will not point to anything if the
+    /// initialization
     ///         failed (e.g. unknown address).
     Lease6Ptr initializeLease6(std::string address);
 
@@ -209,6 +210,9 @@ public:
     /// @brief Test method which returns all IPv6 leases for Subnet ID.
     void testGetLeases6SubnetId();
 
+    /// @brief Test making/fetching leases with IAIDs > signed 32-bit max.
+    void testLease6LargeIaidCheck();
+
     /// @brief Test method which returns all IPv6 leases.
     void testGetLeases6();
 
@@ -247,14 +251,7 @@ public:
     void testLease6HWTypeAndSource();
 
     /// @brief Test that IPv6 lease can be added, retrieved and deleted.
-    ///
-    /// This method checks basic IPv6 lease operations. There's check_t1_t2
-    /// parameter that controls whether the backend supports storing T1, T2
-    /// parameters. memfile supports it, while MySQL doesn't. If T1,T2
-    /// storage is not supported, the expected values are 0.
-    ///
-    /// @param check_t1_t2 controls whether T1,T2 timers should be checked
-    void testAddGetDelete6(bool check_t1_t2);
+    void testAddGetDelete6();
 
     /// @brief Check GetLease6 methods - access by DUID/IAID
     ///
@@ -278,7 +275,7 @@ public:
     /// Adds leases to the database and checks that they can be accessed via
     /// a combination of DIUID and IAID.
     void testGetLease6DuidIaidSubnetId();
-    
+
     /// @brief verifies getLeases6 method by DUID
     ///
     /// Adds 3 leases to backend and retrieves, verifes empty
@@ -446,13 +443,13 @@ public:
                          const std::vector<std::string>& expected_addresses);
 
     /// @brief String forms of IPv4 addresses
-    std::vector<std::string>  straddress4_;
+    std::vector<std::string> straddress4_;
 
     /// @brief IOAddress forms of IPv4 addresses
     std::vector<isc::asiolink::IOAddress> ioaddress4_;
 
     /// @brief String forms of IPv6 addresses
-    std::vector<std::string>  straddress6_;
+    std::vector<std::string> straddress6_;
 
     /// @brief Types of IPv6 Leases
     std::vector<Lease::Type> leasetype6_;
@@ -502,7 +499,7 @@ public:
 
     /// @brief Verifies open failures do NOT invoke db lost callback
     ///
-    /// The db lost callback should only be invoked after succesfully
+    /// The db lost callback should only be invoked after successfully
     /// opening the DB and then subsequently losing it. Failing to
     /// open should be handled directly by the application layer.
     void testNoCallbackOnOpenFailure();
@@ -529,8 +526,8 @@ public:
 
 };
 
-}; // namespace test
-}; // namespace dhcp
-}; // namespace isc
+}  // namespace test
+}  // namespace dhcp
+}  // namespace isc
 
 #endif
