@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,6 +6,7 @@
 
 #include <config.h>
 #include <asiolink/asio_wrapper.h>
+#include <asiolink/interval_timer.h>
 #include <asiolink/io_address.h>
 #include <dhcp/duid.h>
 #include <dhcp/iface_mgr.h>
@@ -14,7 +15,7 @@
 #include <dhcpsrv/lease_mgr_factory.h>
 #include <dhcpsrv/memfile_lease_mgr.h>
 #include <dhcpsrv/timer_mgr.h>
-#include <dhcpsrv/tests/lease_file_io.h>
+#include <dhcpsrv/testutils/lease_file_io.h>
 #include <dhcpsrv/tests/test_utils.h>
 #include <dhcpsrv/tests/generic_lease_mgr_unittest.h>
 #include <util/pid_file.h>
@@ -34,6 +35,7 @@
 using namespace std;
 using namespace isc;
 using namespace isc::asiolink;
+using namespace isc::db;
 using namespace isc::dhcp;
 using namespace isc::dhcp::test;
 using namespace isc::util;
@@ -102,8 +104,11 @@ public:
     MemfileLeaseMgrTest() :
         io4_(getLeaseFilePath("leasefile4_0.csv")),
         io6_(getLeaseFilePath("leasefile6_0.csv")),
+        io_service_(new IOService()),
         timer_mgr_(TimerMgr::instance()) {
 
+        timer_mgr_->setIOService(io_service_);
+
         std::ostringstream s;
         s << KEA_LFC_BUILD_DIR << "/kea-lfc";
         setenv("KEA_LFC_EXECUTABLE", s.str().c_str(), 1);
@@ -130,7 +135,6 @@ public:
     /// destroys lease manager backend.
     virtual ~MemfileLeaseMgrTest() {
         // Stop TimerMgr worker thread if it is running.
-        timer_mgr_->stopThread();
         // Make sure there are no timers registered.
         timer_mgr_->unregisterTimers();
         LeaseMgrFactory::destroy();
@@ -143,7 +147,7 @@ public:
     /// @brief Remove files being products of Lease File Cleanup.
     ///
     /// @param base_name Path to the lease file name. This file is removed
-    /// and all files which names are crated from this name (having specific
+    /// and all files which names are created from this name (having specific
     /// suffixes used by Lease File Cleanup mechanism).
     void removeFiles(const std::string& base_name) const {
         // Generate suffixes and append them to the base name. The
@@ -185,7 +189,8 @@ public:
         std::ostringstream s;
         s << "type=memfile " << (u == V4 ? "universe=4 " : "universe=6 ")
           << "name="
-          << getLeaseFilePath(u == V4 ? "leasefile4_0.csv" : "leasefile6_0.csv");
+          << getLeaseFilePath(u == V4 ? "leasefile4_0.csv" : "leasefile6_0.csv")
+          << " lfc-interval=0";
         return (s.str());
     }
 
@@ -207,12 +212,13 @@ public:
     ///
     /// @param ms Duration in milliseconds.
     void setTestTime(const uint32_t ms) {
-        // Measure test time and exit if timeout hit.
-        Stopwatch stopwatch;
-        while (stopwatch.getTotalMilliseconds() < ms) {
-            // Block for one 1 millisecond.
-            IfaceMgr::instance().receive6(0, 1000);
-        }
+        IntervalTimer timer(*io_service_);
+        timer.setup([this]() {
+                io_service_->stop();
+        }, ms, IntervalTimer::ONE_SHOT);
+
+        io_service_->run();
+        io_service_->get_io_service().reset();
     }
 
     /// @brief Waits for the specified process to finish.
@@ -243,8 +249,6 @@ public:
     ///
     /// The following lease parameters are set to constant values:
     /// - valid lifetime = 1200,
-    /// - T1 = 600,
-    /// - T2 = 900,
     /// - DNS update forward flag = false,
     /// - DNS update reverse flag = false,
     ///
@@ -267,8 +271,6 @@ public:
         fillRandom(clientid.begin(), clientid.end());
 
         uint32_t valid_lft = 1200;
-        uint32_t t1 = 600;
-        uint32_t t2 = 900;
         time_t timestamp = time(NULL) - 86400 + random()%86400;
         bool fqdn_fwd = false;
         bool fqdn_rev = false;
@@ -279,7 +281,7 @@ public:
 
         // Return created lease.
         return (Lease4Ptr(new Lease4(address, hwaddr, &clientid[0],
-                                     clientid.size(), valid_lft, t1, t2,
+                                     clientid.size(), valid_lft,
                                      timestamp, subnet_id, fqdn_fwd,
                                      fqdn_rev, hostname.str())));
     }
@@ -297,8 +299,6 @@ public:
     /// - lease type = IA_NA
     /// - valid lifetime = 1200,
     /// - preferred lifetime = 1000
-    /// - T1 = 600,
-    /// - T2 = 900,
     /// - DNS update forward flag = false,
     /// - DNS update reverse flag = false,
     ///
@@ -319,8 +319,6 @@ public:
         uint32_t iaid = 1 + random()%100;
         uint32_t valid_lft = 1200;
         uint32_t preferred_lft = 1000;
-        uint32_t t1 = 600;
-        uint32_t t2 = 900;
         time_t timestamp = time(NULL) - 86400 + random()%86400;
         bool fqdn_fwd = false;
         bool fqdn_rev = false;
@@ -331,7 +329,7 @@ public:
 
         // Return created lease.
         Lease6Ptr lease(new Lease6(lease_type, address, duid, iaid,
-                                   preferred_lft, valid_lft, t1, t2,
+                                   preferred_lft, valid_lft,
                                    subnet_id, fqdn_fwd, fqdn_rev,
                                    hostname.str()));
         lease->cltt_ = timestamp;
@@ -344,6 +342,9 @@ public:
     /// @brief Object providing access to v6 lease IO.
     LeaseFileIO io6_;
 
+    /// @brief Pointer to the IO service used by the tests.
+    IOServicePtr io_service_;
+
     /// @brief Pointer to the instance of the @c TimerMgr.
     TimerMgrPtr timer_mgr_;
 };
@@ -361,6 +362,7 @@ TEST_F(MemfileLeaseMgrTest, constructor) {
     pmap["lfc-interval"] = "10";
     pmap["persist"] = "true";
     pmap["name"] = getLeaseFilePath("leasefile4_1.csv");
+    pmap["max-row-errors"] = "5";
     EXPECT_NO_THROW(lease_mgr.reset(new Memfile_LeaseMgr(pmap)));
 
     // Expecting that persist parameter is yes or no. Everything other than
@@ -373,6 +375,22 @@ TEST_F(MemfileLeaseMgrTest, constructor) {
     pmap["persist"] = "true";
     pmap["lfc-interval"] = "bogus";
     EXPECT_THROW(lease_mgr.reset(new Memfile_LeaseMgr(pmap)), isc::BadValue);
+
+    // The max-row-errors must be an integer.
+    pmap["persist"] = "true";
+    pmap["max-row-errors"] = "bogus";
+    EXPECT_THROW(lease_mgr.reset(new Memfile_LeaseMgr(pmap)), isc::BadValue);
+
+    // The max-row-errors must be >= 0.
+    pmap["persist"] = "true";
+    pmap["max-row-errors"] = "-1";
+    EXPECT_THROW(lease_mgr.reset(new Memfile_LeaseMgr(pmap)), isc::BadValue);
+}
+
+// Checks if there is no lease manager NoLeaseManager is thrown.
+TEST_F(MemfileLeaseMgrTest, noLeaseManager) {
+    LeaseMgrFactory::destroy();
+    EXPECT_THROW(LeaseMgrFactory::instance(), NoLeaseManager);
 }
 
 // Checks if the getType() and getName() methods both return "memfile".
@@ -403,7 +421,7 @@ TEST_F(MemfileLeaseMgrTest, getLeaseFilePath) {
     EXPECT_TRUE(lease_mgr->getLeaseFilePath(Memfile_LeaseMgr::V6).empty());
 }
 
-// Check if the persitLeases correctly checks that leases should not be written
+// Check if the persistLeases correctly checks that leases should not be written
 // to disk when disabled through configuration.
 TEST_F(MemfileLeaseMgrTest, persistLeases) {
     // Initialize IO objects, so as the test csv files get removed after the
@@ -413,6 +431,7 @@ TEST_F(MemfileLeaseMgrTest, persistLeases) {
 
     DatabaseConnection::ParameterMap pmap;
     pmap["universe"] = "4";
+    pmap["lfc-interval"] = "0";
     // Specify the names of the lease files. Leases will be written.
     pmap["name"] = getLeaseFilePath("leasefile4_1.csv");
     boost::scoped_ptr<Memfile_LeaseMgr> lease_mgr(new Memfile_LeaseMgr(pmap));
@@ -447,15 +466,9 @@ TEST_F(MemfileLeaseMgrTest, lfcTimer) {
     boost::scoped_ptr<LFCMemfileLeaseMgr>
         lease_mgr(new LFCMemfileLeaseMgr(pmap));
 
-    // Start worker thread to execute LFC periodically.
-    ASSERT_NO_THROW(timer_mgr_->startThread());
-
     // Run the test for at most 2.9 seconds.
     setTestTime(2900);
 
-    // Stop worker thread.
-    ASSERT_NO_THROW(timer_mgr_->stopThread());
-
     // Within 2.9 we should record two LFC executions.
     EXPECT_EQ(2, lease_mgr->getLFCCount());
 }
@@ -474,17 +487,9 @@ TEST_F(MemfileLeaseMgrTest, lfcTimerDisabled) {
     boost::scoped_ptr<LFCMemfileLeaseMgr>
         lease_mgr(new LFCMemfileLeaseMgr(pmap));
 
-    // Start worker thread to execute LFC periodically.
-    ASSERT_NO_THROW(timer_mgr_->startThread());
-
     // Run the test for at most 1.9 seconds.
     setTestTime(1900);
 
-    // Stop worker thread to make sure it is not running when lease
-    // manager is destroyed. The lease manager will be unable to
-    // unregster timer when the thread is active.
-    ASSERT_NO_THROW(timer_mgr_->stopThread());
-
     // There should be no LFC execution recorded.
     EXPECT_EQ(0, lease_mgr->getLFCCount());
 }
@@ -497,20 +502,20 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCleanup4) {
     // stored.
     std::string new_file_contents =
         "address,hwaddr,client_id,valid_lifetime,expire,"
-        "subnet_id,fqdn_fwd,fqdn_rev,hostname,state\n";
+        "subnet_id,fqdn_fwd,fqdn_rev,hostname,state,user_context\n";
 
     // This string contains the contents of the lease file with exactly
     // one lease, but two entries. One of the entries should be removed
     // as a result of lease file cleanup.
     std::string current_file_contents = new_file_contents +
-        "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1\n"
-        "192.0.2.2,02:02:02:02:02:02,,200,800,8,1,1,,1\n";
+        "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1,{ \"foo\": true }\n"
+        "192.0.2.2,02:02:02:02:02:02,,200,800,8,1,1,,1,\n";
     LeaseFileIO current_file(getLeaseFilePath("leasefile4_0.csv"));
     current_file.writeFile(current_file_contents);
 
     std::string previous_file_contents = new_file_contents +
-        "192.0.2.3,03:03:03:03:03:03,,200,200,8,1,1,,1\n"
-        "192.0.2.3,03:03:03:03:03:03,,200,800,8,1,1,,1\n";
+        "192.0.2.3,03:03:03:03:03:03,,200,200,8,1,1,,1,\n"
+        "192.0.2.3,03:03:03:03:03:03,,200,800,8,1,1,,1,{ \"bar\": true }\n";
     LeaseFileIO previous_file(getLeaseFilePath("leasefile4_0.csv.2"));
     previous_file.writeFile(previous_file_contents);
 
@@ -541,20 +546,21 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCleanup4) {
     // Check if we can still write to the lease file.
     std::vector<uint8_t> hwaddr_vec(6);
     HWAddrPtr hwaddr(new HWAddr(hwaddr_vec, HTYPE_ETHER));
-    Lease4Ptr new_lease(new Lease4(IOAddress("192.0.2.45"), hwaddr, 0, 0,
-                                   100, 50, 60, 0, 1));
+    Lease4Ptr new_lease(new Lease4(IOAddress("192.0.2.45"), hwaddr,
+                                   static_cast<const uint8_t*>(0), 0,
+                                   100, 0, 1));
     ASSERT_NO_THROW(lease_mgr->addLease(new_lease));
 
     std::string updated_file_contents = new_file_contents +
-        "192.0.2.45,00:00:00:00:00:00,,100,100,1,0,0,,0\n";
+        "192.0.2.45,00:00:00:00:00:00,,100,100,1,0,0,,0,\n";
     EXPECT_EQ(updated_file_contents, current_file.readFile());
 
     // This string contains the contents of the lease file we
     // expect after the LFC run.  It has two leases with one
     // entry each.
     std::string result_file_contents = new_file_contents +
-        "192.0.2.2,02:02:02:02:02:02,,200,800,8,1,1,,1\n"
-        "192.0.2.3,03:03:03:03:03:03,,200,800,8,1,1,,1\n";
+        "192.0.2.2,02:02:02:02:02:02,,200,800,8,1,1,,1,\n"
+        "192.0.2.3,03:03:03:03:03:03,,200,800,8,1,1,,1,{ \"bar\": true }\n";
 
     // The LFC should have created a file with the two leases and moved it
     // to leasefile4_0.csv.2
@@ -573,24 +579,24 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCleanup6) {
     std::string new_file_contents =
         "address,duid,valid_lifetime,expire,subnet_id,"
         "pref_lifetime,lease_type,iaid,prefix_len,fqdn_fwd,"
-        "fqdn_rev,hostname,hwaddr,state\n";
+        "fqdn_rev,hostname,hwaddr,state,user_context\n";
 
     // This string contains the contents of the lease file with exactly
     // one lease, but two entries. One of the entries should be removed
     // as a result of lease file cleanup.
     std::string current_file_contents = new_file_contents +
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,200,"
-        "8,100,0,7,0,1,1,,,1\n"
+        "8,100,0,7,0,1,1,,,1,\n"
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,800,"
-        "8,100,0,7,0,1,1,,,1\n";
+        "8,100,0,7,0,1,1,,,1,{ \"foo\": true }\n";
     LeaseFileIO current_file(getLeaseFilePath("leasefile6_0.csv"));
     current_file.writeFile(current_file_contents);
 
     std::string previous_file_contents = new_file_contents +
         "2001:db8:1::2,01:01:01:01:01:01:01:01:01:01:01:01:01,200,200,"
-        "8,100,0,7,0,1,1,,,1\n"
+        "8,100,0,7,0,1,1,,,1,{ \"bar\": true }\n"
         "2001:db8:1::2,01:01:01:01:01:01:01:01:01:01:01:01:01,200,800,"
-        "8,100,0,7,0,1,1,,,1\n";
+        "8,100,0,7,0,1,1,,,1,\n";
     LeaseFileIO previous_file(getLeaseFilePath("leasefile6_0.csv.2"));
     previous_file.writeFile(previous_file_contents);
 
@@ -622,14 +628,14 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCleanup6) {
     // Check if we can still write to the lease file.
     std::vector<uint8_t> duid_vec(13);
     DuidPtr duid(new DUID(duid_vec));
-    Lease6Ptr new_lease(new Lease6(Lease::TYPE_NA, IOAddress("3000::1"),duid,
-                                   123, 300, 400, 100, 300, 2));
+    Lease6Ptr new_lease(new Lease6(Lease::TYPE_NA, IOAddress("3000::1"), duid,
+                                   123, 300, 400, 2));
     new_lease->cltt_ = 0;
     ASSERT_NO_THROW(lease_mgr->addLease(new_lease));
 
     std::string update_file_contents = new_file_contents +
         "3000::1,00:00:00:00:00:00:00:00:00:00:00:00:00,400,"
-        "400,2,300,0,123,128,0,0,,,0\n";
+        "400,2,300,0,123,128,0,0,,,0,\n";
     EXPECT_EQ(update_file_contents, current_file.readFile());
 
     // This string contains the contents of the lease file we
@@ -637,9 +643,9 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCleanup6) {
     // entry each.
     std::string result_file_contents = new_file_contents +
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,800,"
-        "8,100,0,7,0,1,1,,,1\n"
+        "8,100,0,7,0,1,1,,,1,{ \"foo\": true }\n"
         "2001:db8:1::2,01:01:01:01:01:01:01:01:01:01:01:01:01,200,800,"
-        "8,100,0,7,0,1,1,,,1\n";
+        "8,100,0,7,0,1,1,,,1,\n";
 
     // The LFC should have created a file with the two leases and moved it
     // to leasefile6_0.csv.2
@@ -657,11 +663,11 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCleanupStartFail) {
     // stored.
     std::string new_file_contents =
         "address,hwaddr,client_id,valid_lifetime,expire,"
-        "subnet_id,fqdn_fwd,fqdn_rev,hostname,state\n";
+        "subnet_id,fqdn_fwd,fqdn_rev,hostname,state,user_context\n";
 
     // Create the lease file to be used by the backend.
     std::string current_file_contents = new_file_contents +
-        "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1\n";
+        "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1,\n";
     LeaseFileIO current_file(getLeaseFilePath("leasefile4_0.csv"));
     current_file.writeFile(current_file_contents);
 
@@ -698,15 +704,15 @@ TEST_F(MemfileLeaseMgrTest, leaseFileFinish) {
     std::string new_file_contents =
         "address,duid,valid_lifetime,expire,subnet_id,"
         "pref_lifetime,lease_type,iaid,prefix_len,fqdn_fwd,"
-        "fqdn_rev,hostname,hwaddr,state\n";
+        "fqdn_rev,hostname,hwaddr,state,user_context\n";
 
     // This string contains the contents of the current lease file.
     // It should not be moved.
     std::string current_file_contents = new_file_contents +
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,200,"
-        "8,100,0,7,0,1,1,,,1\n"
+        "8,100,0,7,0,1,1,,,1,\n"
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,800,"
-        "8,100,0,7,0,1,1,,,1\n";
+        "8,100,0,7,0,1,1,,,1,\n";
     LeaseFileIO current_file(getLeaseFilePath("leasefile6_0.csv"));
     current_file.writeFile(current_file_contents);
 
@@ -714,7 +720,7 @@ TEST_F(MemfileLeaseMgrTest, leaseFileFinish) {
     // be moved to the previous file.
     std::string finish_file_contents = new_file_contents +
         "2001:db8:1::2,01:01:01:01:01:01:01:01:01:01:01:01:01,200,800,"
-        "8,100,0,7,0,1,1,,,1\n";
+        "8,100,0,7,0,1,1,,,1,\n";
     LeaseFileIO finish_file(getLeaseFilePath("leasefile6_0.csv.completed"));
     finish_file.writeFile(finish_file_contents);
 
@@ -761,15 +767,15 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCopy) {
     std::string new_file_contents =
         "address,duid,valid_lifetime,expire,subnet_id,"
         "pref_lifetime,lease_type,iaid,prefix_len,fqdn_fwd,"
-        "fqdn_rev,hostname,hwaddr,state\n";
+        "fqdn_rev,hostname,hwaddr,state,user_context\n";
 
     // This string contains the contents of the current lease file.
     // It should not be moved.
     std::string current_file_contents = new_file_contents +
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,200,"
-        "8,100,0,7,0,1,1,,,1\n"
+        "8,100,0,7,0,1,1,,,1,\n"
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,800,"
-        "8,100,0,7,0,1,1,,,1\n";
+        "8,100,0,7,0,1,1,,,1,{ \"foo\": true }\n";
     LeaseFileIO current_file(getLeaseFilePath("leasefile6_0.csv"));
     current_file.writeFile(current_file_contents);
 
@@ -779,7 +785,7 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCopy) {
     // the same.
     std::string input_file_contents = new_file_contents +
         "2001:db8:1::2,01:01:01:01:01:01:01:01:01:01:01:01:01,200,800,"
-        "8,100,0,7,0,1,1,,,1\n";
+        "8,100,0,7,0,1,1,,,1,{ \"foo\": true }\n";
     LeaseFileIO input_file(getLeaseFilePath("leasefile6_0.csv.1"));
     input_file.writeFile(input_file_contents);
 
@@ -820,9 +826,7 @@ TEST_F(MemfileLeaseMgrTest, leaseFileCopy) {
 // Checks that adding/getting/deleting a Lease6 object works.
 TEST_F(MemfileLeaseMgrTest, addGetDelete6) {
     startBackend(V6);
-    testAddGetDelete6(true); // true - check T1,T2 values
-    // memfile is able to preserve those values, but some other
-    // backends can't do that.
+    testAddGetDelete6();
 }
 
 /// @brief Basic Lease4 Checks
@@ -872,7 +876,7 @@ TEST_F(MemfileLeaseMgrTest, getLease4ClientIdHWAddrSubnetId) {
 /// @brief Basic Lease4 Checks
 ///
 /// Checks that the addLease, getLease4(by address), getLease4(hwaddr,subnet_id),
-/// updateLease4() and deleteLease (IPv4 address) can handle NULL client-id.
+/// updateLease4() and deleteLease can handle NULL client-id.
 /// (client-id is optional and may not be present)
 TEST_F(MemfileLeaseMgrTest, lease4NullClientId) {
     startBackend(V4);
@@ -916,6 +920,49 @@ TEST_F(MemfileLeaseMgrTest, getLease4ClientIdSubnetId) {
     testGetLease4ClientIdSubnetId();
 }
 
+// This test checks that all IPv4 leases for a specified subnet id are returned.
+TEST_F(MemfileLeaseMgrTest, getLeases4SubnetId) {
+    startBackend(V4);
+    testGetLeases4SubnetId();
+}
+
+// This test checks that all IPv4 leases are returned.
+TEST_F(MemfileLeaseMgrTest, getLeases4) {
+    startBackend(V4);
+    testGetLeases4();
+}
+
+// Test that a range of IPv4 leases is returned with paging.
+TEST_F(MemfileLeaseMgrTest, getLeases4Paged) {
+    startBackend(V4);
+    testGetLeases4Paged();
+}
+
+// This test checks that all IPv6 leases for a specified subnet id are returned.
+TEST_F(MemfileLeaseMgrTest, getLeases6SubnetId) {
+    startBackend(V6);
+    testGetLeases6SubnetId();
+}
+
+// This test adds 3 leases  and verifies fetch by DUID.
+// Verifies retrival of non existant DUID fails
+TEST_F(MemfileLeaseMgrTest, getLeases6Duid) {
+    startBackend(V6);
+    testGetLeases6Duid();
+}
+
+// This test checks that all IPv6 leases are returned.
+TEST_F(MemfileLeaseMgrTest, getLeases6) {
+    startBackend(V6);
+    testGetLeases6();
+}
+
+// Test that a range of IPv6 leases is returned with paging.
+TEST_F(MemfileLeaseMgrTest, getLeases6Paged) {
+    startBackend(V6);
+    testGetLeases6Paged();
+}
+
 /// @brief Basic Lease6 Checks
 ///
 /// Checks that the addLease, getLease6 (by address) and deleteLease (with an
@@ -1106,22 +1153,22 @@ TEST_F(MemfileLeaseMgrTest, getDeclined6) {
 TEST_F(MemfileLeaseMgrTest, load4MultipleLeaseFiles) {
     LeaseFileIO io2(getLeaseFilePath("leasefile4_0.csv.2"));
     io2.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state\n"
-                  "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1\n"
-                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,200,8,1,1,,1\n");
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                  "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1,\n"
+                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,200,8,1,1,,1,\n");
 
     LeaseFileIO io1(getLeaseFilePath("leasefile4_0.csv.1"));
     io1.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state\n"
-                  "192.0.2.1,01:01:01:01:01:01,,200,200,8,1,1,,1\n"
-                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,400,8,1,1,,1\n"
-                  "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,200,8,1,1,,1\n");
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                  "192.0.2.1,01:01:01:01:01:01,,200,200,8,1,1,,1,\n"
+                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,400,8,1,1,,1,\n"
+                  "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,200,8,1,1,,1,\n");
 
     LeaseFileIO io(getLeaseFilePath("leasefile4_0.csv"));
     io.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                 "fqdn_fwd,fqdn_rev,hostname,state\n"
-                 "192.0.2.10,0a:0a:0a:0a:0a:0a,,200,200,8,1,1,,1\n"
-                 "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,400,8,1,1,,1\n");
+                 "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                 "192.0.2.10,0a:0a:0a:0a:0a:0a,,200,200,8,1,1,,1,\n"
+                 "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,400,8,1,1,,1,\n");
 
     startBackend(V4);
 
@@ -1164,27 +1211,27 @@ TEST_F(MemfileLeaseMgrTest, load4MultipleLeaseFiles) {
 TEST_F(MemfileLeaseMgrTest, load4CompletedFile) {
     LeaseFileIO io2(getLeaseFilePath("leasefile4_0.csv.2"));
     io2.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state\n"
-                  "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1\n"
-                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,200,8,1,1,,1\n");
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                  "192.0.2.2,02:02:02:02:02:02,,200,200,8,1,1,,1,\n"
+                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,200,8,1,1,,1,\n");
 
     LeaseFileIO io1(getLeaseFilePath("leasefile4_0.csv.1"));
     io1.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state\n"
-                  "192.0.2.1,01:01:01:01:01:01,,200,200,8,1,1,,1\n"
-                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,400,8,1,1,,1\n"
-                  "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,200,8,1,1,,1\n");
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                  "192.0.2.1,01:01:01:01:01:01,,200,200,8,1,1,,1,\n"
+                  "192.0.2.11,bb:bb:bb:bb:bb:bb,,200,400,8,1,1,,1,\n"
+                  "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,200,8,1,1,,1,\n");
 
     LeaseFileIO io(getLeaseFilePath("leasefile4_0.csv"));
     io.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                 "fqdn_fwd,fqdn_rev,hostname,state\n"
-                 "192.0.2.10,0a:0a:0a:0a:0a:0a,,200,200,8,1,1,,1\n"
-                 "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,400,8,1,1,,1\n");
+                 "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                 "192.0.2.10,0a:0a:0a:0a:0a:0a,,200,200,8,1,1,,1,\n"
+                 "192.0.2.12,cc:cc:cc:cc:cc:cc,,200,400,8,1,1,,1,\n");
 
     LeaseFileIO ioc(getLeaseFilePath("leasefile4_0.csv.completed"));
     ioc.writeFile("address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state\n"
-                  "192.0.2.13,ff:ff:ff:ff:ff:ff,,200,200,8,1,1,,1\n");
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n"
+                  "192.0.2.13,ff:ff:ff:ff:ff:ff,,200,200,8,1,1,,1,\n");
 
     startBackend(V4);
 
@@ -1230,7 +1277,7 @@ TEST_F(MemfileLeaseMgrTest, load4LFCInProgress) {
     ASSERT_THROW(lease_mgr.reset(new NakedMemfileLeaseMgr(pmap)),
                  DbOpenError);
 
-    // Remove the pid file, and retry. The bakckend should be created.
+    // Remove the pid file, and retry. The backend should be created.
     pid_file.deleteFile();
     ASSERT_NO_THROW(lease_mgr.reset(new NakedMemfileLeaseMgr(pmap)));
 }
@@ -1240,32 +1287,32 @@ TEST_F(MemfileLeaseMgrTest, load4LFCInProgress) {
 TEST_F(MemfileLeaseMgrTest, load6MultipleLeaseFiles) {
     LeaseFileIO io2(getLeaseFilePath("leasefile6_0.csv.2"));
     io2.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                  "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                   "2001:db8:1::1,01:01:01:01:01:01:01:01:01:01:01:01:01,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n"
+                  "200,200,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::2,02:02:02:02:02:02:02:02:02:02:02:02:02,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n");
+                  "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     LeaseFileIO io1(getLeaseFilePath("leasefile6_0.csv.1"));
     io1.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                  "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                   "2001:db8:1::3,03:03:03:03:03:03:03:03:03:03:03:03:03,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n"
+                  "200,200,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::2,02:02:02:02:02:02:02:02:02:02:02:02:02,"
-                  "300,800,8,100,0,7,0,1,1,,,1\n"
+                  "300,800,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::4,04:04:04:04:04:04:04:04:04:04:04:04:04,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n");
+                  "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     LeaseFileIO io(getLeaseFilePath("leasefile6_0.csv"));
     io.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                 "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                 "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                  "2001:db8:1::4,04:04:04:04:04:04:04:04:04:04:04:04:04,"
-                 "400,1000,8,100,0,7,0,1,1,,,1\n"
+                 "400,1000,8,100,0,7,0,1,1,,,1,\n"
                  "2001:db8:1::5,05:05:05:05:05:05:05:05:05:05:05:05:05,"
-                 "200,200,8,100,0,7,0,1,1,,,1\n");
+                 "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     startBackend(V6);
 
@@ -1306,23 +1353,23 @@ TEST_F(MemfileLeaseMgrTest, load6MultipleLeaseFiles) {
 TEST_F(MemfileLeaseMgrTest, load6MultipleNoSecondFile) {
     LeaseFileIO io1(getLeaseFilePath("leasefile6_0.csv.1"));
     io1.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                  "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                   "2001:db8:1::3,03:03:03:03:03:03:03:03:03:03:03:03:03,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n"
+                  "200,200,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::2,02:02:02:02:02:02:02:02:02:02:02:02:02,"
-                  "300,800,8,100,0,7,0,1,1,,,1\n"
+                  "300,800,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::4,04:04:04:04:04:04:04:04:04:04:04:04:04,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n");
+                  "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     LeaseFileIO io(getLeaseFilePath("leasefile6_0.csv"));
     io.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                 "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                 "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                  "2001:db8:1::4,04:04:04:04:04:04:04:04:04:04:04:04:04,"
-                 "400,1000,8,100,0,7,0,1,1,,,1\n"
+                 "400,1000,8,100,0,7,0,1,1,,,1,\n"
                  "2001:db8:1::5,05:05:05:05:05:05:05:05:05:05:05:05:05,"
-                 "200,200,8,100,0,7,0,1,1,,,1\n");
+                 "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     startBackend(V6);
 
@@ -1354,21 +1401,21 @@ TEST_F(MemfileLeaseMgrTest, load6MultipleNoSecondFile) {
 TEST_F(MemfileLeaseMgrTest, load6MultipleNoFirstFile) {
     LeaseFileIO io2(getLeaseFilePath("leasefile6_0.csv.2"));
     io2.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                  "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                   "2001:db8:1::1,01:01:01:01:01:01:01:01:01:01:01:01:01,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n"
+                  "200,200,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::2,02:02:02:02:02:02:02:02:02:02:02:02:02,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n");
+                  "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     LeaseFileIO io(getLeaseFilePath("leasefile6_0.csv"));
     io.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                 "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                 "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                  "2001:db8:1::4,04:04:04:04:04:04:04:04:04:04:04:04:04,"
-                 "400,1000,8,100,0,7,0,1,1,,,1\n"
+                 "400,1000,8,100,0,7,0,1,1,,,1,\n"
                  "2001:db8:1::5,05:05:05:05:05:05:05:05:05:05:05:05:05,"
-                 "200,200,8,100,0,7,0,1,1,,,1\n");
+                 "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     startBackend(V6);
 
@@ -1402,39 +1449,39 @@ TEST_F(MemfileLeaseMgrTest, load6MultipleNoFirstFile) {
 TEST_F(MemfileLeaseMgrTest, load6CompletedFile) {
     LeaseFileIO io2(getLeaseFilePath("leasefile6_0.csv.2"));
     io2.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                  "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                   "2001:db8:1::1,01:01:01:01:01:01:01:01:01:01:01:01:01,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n"
+                  "200,200,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::2,02:02:02:02:02:02:02:02:02:02:02:02:02,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n");
+                  "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     LeaseFileIO io1(getLeaseFilePath("leasefile6_0.csv.1"));
     io1.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                  "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                   "2001:db8:1::3,03:03:03:03:03:03:03:03:03:03:03:03:03,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n"
+                  "200,200,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::2,02:02:02:02:02:02:02:02:02:02:02:02:02,"
-                  "300,800,8,100,0,7,0,1,1,,,1\n"
+                  "300,800,8,100,0,7,0,1,1,,,1,\n"
                   "2001:db8:1::4,04:04:04:04:04:04:04:04:04:04:04:04:04,"
-                  "200,200,8,100,0,7,0,1,1,,,1\n");
+                  "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     LeaseFileIO io(getLeaseFilePath("leasefile6_0.csv"));
     io.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                 "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                 "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                  "2001:db8:1::4,04:04:04:04:04:04:04:04:04:04:04:04:04,"
-                 "400,1000,8,100,0,7,0,1,1,,,1\n"
+                 "400,1000,8,100,0,7,0,1,1,,,1,\n"
                  "2001:db8:1::5,05:05:05:05:05:05:05:05:05:05:05:05:05,"
-                 "200,200,8,100,0,7,0,1,1,,,1\n");
+                 "200,200,8,100,0,7,0,1,1,,,1,\n");
 
     LeaseFileIO ioc(getLeaseFilePath("leasefile6_0.csv.completed"));
     ioc.writeFile("address,duid,valid_lifetime,expire,subnet_id,pref_lifetime,"
-                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,hwaddr,"
-                  "state\n"
+                  "lease_type,iaid,prefix_len,fqdn_fwd,fqdn_rev,hostname,"
+                  "hwaddr,state,user_context\n"
                   "2001:db8:1::125,ff:ff:ff:ff:ff:ff:ff:ff:ff:ff:ff:ff:ff,"
-                  "400,1000,8,100,0,7,0,1,1,,,1\n");
+                  "400,1000,8,100,0,7,0,1,1,,,1,\n");
 
     startBackend(V6);
 
@@ -1480,7 +1527,7 @@ TEST_F(MemfileLeaseMgrTest, load6LFCInProgress) {
     ASSERT_THROW(lease_mgr.reset(new NakedMemfileLeaseMgr(pmap)),
                  DbOpenError);
 
-    // Remove the pid file, and retry. The bakckend should be created.
+    // Remove the pid file, and retry. The backend should be created.
     pid_file.deleteFile();
     ASSERT_NO_THROW(lease_mgr.reset(new NakedMemfileLeaseMgr(pmap)));
 }
@@ -1495,7 +1542,7 @@ TEST_F(MemfileLeaseMgrTest, leaseUpgrade4) {
 
     std::string header_2_0 =
         "address,hwaddr,client_id,valid_lifetime,expire,"
-        "subnet_id,fqdn_fwd,fqdn_rev,hostname,state\n";
+        "subnet_id,fqdn_fwd,fqdn_rev,hostname,state,user_context\n";
 
     // Create 1.0 Schema current lease file with two entries for
     // the same lease
@@ -1541,8 +1588,8 @@ TEST_F(MemfileLeaseMgrTest, leaseUpgrade4) {
 
     // Verify cleaned, converted contents
     std::string result_file_contents = header_2_0 +
-        "192.0.2.2,02:02:02:02:02:02,,200,800,8,1,1,,0\n"
-        "192.0.2.3,03:03:03:03:03:03,,200,800,8,1,1,,0\n";
+        "192.0.2.2,02:02:02:02:02:02,,200,800,8,1,1,,0,\n"
+        "192.0.2.3,03:03:03:03:03:03,,200,800,8,1,1,,0,\n";
     EXPECT_EQ(result_file_contents, input_file.readFile());
 }
 
@@ -1561,7 +1608,7 @@ TEST_F(MemfileLeaseMgrTest, leaseUpgrade6) {
     std::string header_3_0 =
         "address,duid,valid_lifetime,expire,subnet_id,"
         "pref_lifetime,lease_type,iaid,prefix_len,fqdn_fwd,"
-        "fqdn_rev,hostname,hwaddr,state\n";
+        "fqdn_rev,hostname,hwaddr,state,user_context\n";
 
     // The current lease file is schema 1.0 and has two entries for
     // the same lease
@@ -1612,9 +1659,9 @@ TEST_F(MemfileLeaseMgrTest, leaseUpgrade6) {
     // Verify cleaned, converted contents
     std::string result_file_contents = header_3_0 +
         "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,200,800,"
-        "8,100,0,7,0,1,1,,,0\n"
+        "8,100,0,7,0,1,1,,,0,\n"
         "2001:db8:1::2,01:01:01:01:01:01:01:01:01:01:01:01:01,200,800,"
-        "8,100,0,7,0,1,1,,11:22:33:44:55,0\n";
+        "8,100,0,7,0,1,1,,11:22:33:44:55,0,\n";
     EXPECT_EQ(result_file_contents, input_file.readFile());
 }
 
@@ -1634,6 +1681,7 @@ TEST_F(MemfileLeaseMgrTest, lease4ContainerIndexUpdate) {
     pmap["universe"] = "4";
     pmap["name"] = leasefile;
     pmap["persist"] = "true";
+    pmap["lfc-interval"] = "0";
 
     srand(seed);
 
@@ -1647,7 +1695,7 @@ TEST_F(MemfileLeaseMgrTest, lease4ContainerIndexUpdate) {
     // pick a lease.
     std::vector<IOAddress> lease_addresses;
 
-    // Generarate random leases. We remember their addresses in
+    // Generate random leases. We remember their addresses in
     // lease_addresses.
     for (uint32_t i = 0; i < leases_cnt; ++i) {
         Lease4Ptr lease = initiateRandomLease4(addr);
@@ -1773,6 +1821,7 @@ TEST_F(MemfileLeaseMgrTest, lease6ContainerIndexUpdate) {
     pmap["universe"] = "6";
     pmap["name"] = leasefile;
     pmap["persist"] = "true";
+    pmap["lfc-interval"] = "0";
 
     srand(seed);
 
@@ -1786,7 +1835,7 @@ TEST_F(MemfileLeaseMgrTest, lease6ContainerIndexUpdate) {
     // pick a lease.
     std::vector<IOAddress> lease_addresses;
 
-    // Generarate random leases. We remember their addresses in
+    // Generate random leases. We remember their addresses in
     // lease_addresses.
     for (uint32_t i = 0; i < leases_cnt; ++i) {
         Lease6Ptr lease = initiateRandomLease6(addr);
@@ -1895,4 +1944,29 @@ TEST_F(MemfileLeaseMgrTest, recountLeaseStats6) {
     testRecountLeaseStats6();
 }
 
-}; // end of anonymous namespace
+// Tests that leases from specific subnet can be removed.
+TEST_F(MemfileLeaseMgrTest, wipeLeases4) {
+    startBackend(V4);
+    testWipeLeases4();
+}
+
+// Tests that leases from specific subnet can be removed.
+TEST_F(MemfileLeaseMgrTest, wipeLeases6) {
+    startBackend(V6);
+    testWipeLeases6();
+}
+
+// Tests v4 lease stats query variants.
+TEST_F(MemfileLeaseMgrTest, leaseStatsQuery4) {
+    startBackend(V4);
+    testLeaseStatsQuery4();
+}
+
+// Tests v6 lease stats query variants.
+TEST_F(MemfileLeaseMgrTest, leaseStatsQuery6) {
+    startBackend(V6);
+    testLeaseStatsQuery6();
+}
+
+
+}  // namespace
