@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -34,6 +34,21 @@ detailCompareLease(const Lease6Ptr& first, const Lease6Ptr& second);
 void
 detailCompareLease(const Lease4Ptr& first, const Lease4Ptr& second);
 
+/// @brief Function that finds the last open socket descriptor
+///
+/// This function is used to attempt lost connectivity
+/// with backends, notably MySQL and Postgresql.
+///
+/// The theory being, that in a confined test environment the last
+/// such descriptor is the SQL client socket descriptor.  This allows
+/// us to the close that descriptor and simulate a loss of server
+/// connectivity.
+///
+/// @return the descriptor of the last open socket or -1 if there
+/// are none.
+int findLastSocketFd();
+
+
 }; // namespace test
 }; // namespace dhcp
 }; // namespace isc
