@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -151,7 +151,7 @@ public:
     /// @param expected_code expected option code
     /// @param expected_repr expected representation (text, hex, exists)
     void checkTokenRelay6Option(const TokenPtr& token,
-                                uint8_t expected_level,
+                                int8_t expected_level,
                                 uint16_t expected_code,
                                 TokenOption::RepresentationType expected_repr) {
         ASSERT_TRUE(token);
@@ -174,7 +174,7 @@ public:
     /// @param exp_repr expected representation to be parsed
     /// @param exp_tokens expected number of tokens
     void testRelay6Option(const std::string& expr,
-                         uint8_t exp_level,
+                         int8_t exp_level,
                          uint16_t exp_code,
                          TokenOption::RepresentationType exp_repr,
                          int exp_tokens) {
@@ -189,13 +189,13 @@ public:
             return;
         }
 
-        // Parsing should succed and return a token.
+        // Parsing should succeed and return a token.
         EXPECT_TRUE(parsed_);
 
         // There should be the expected number of tokens.
         ASSERT_EQ(exp_tokens, eval.expression.size());
 
-        // checkt that the first token is TokenRelay6Option and that
+        // checked that the first token is TokenRelay6Option and that
         // is has the correct attributes
         checkTokenRelay6Option(eval.expression.at(0), exp_level, exp_code, exp_repr);
     }
@@ -212,7 +212,7 @@ public:
         EXPECT_EQ(type, pkt->getType());
     }
 
-    /// @brief Test that verifies access to the DHCP packet metadatas.
+    /// @brief Test that verifies access to the DHCP packet metadata.
     ///
     /// This test attempts to parse the expression, will check if the number
     /// of tokens is exactly as expected and then will try to verify if the
@@ -345,7 +345,7 @@ public:
     /// @param expected_code expected option code
     /// @param expected_repr expected representation (text, hex, exists)
     void checkTokenRelay6Field(const TokenPtr& token,
-                               uint8_t expected_level,
+                               int8_t expected_level,
                                TokenRelay6Field::FieldType expected_type) {
         ASSERT_TRUE(token);
         boost::shared_ptr<TokenRelay6Field> opt =
@@ -365,7 +365,7 @@ public:
     /// @param exp_type expected field type to be parsed
     /// @param exp_tokens expected number of tokens
     void testRelay6Field(const std::string& expr,
-                         uint8_t exp_level,
+                         int8_t exp_level,
                          TokenRelay6Field::FieldType exp_type,
                          int exp_tokens) {
         EvalContext eval(Option::V6);
@@ -379,17 +379,65 @@ public:
             return;
         }
 
-        // Parsing should succed and return a token.
+        // Parsing should succeed and return a token.
         EXPECT_TRUE(parsed_);
 
         // There should be the expected number of tokens.
         ASSERT_EQ(exp_tokens, eval.expression.size());
 
-        // checkt that the first token is TokenRelay6Field and that
+        // checked that the first token is TokenRelay6Field and that
         // is has the correct attributes
         checkTokenRelay6Field(eval.expression.at(0), exp_level, exp_type);
     }
 
+    /// @brief checks if the given token is a TokenMember with the
+    /// correct client class name.
+    /// @param token token to be checked
+    /// @param expected_client_class expected client class name
+    void checkTokenMember(const TokenPtr& token,
+                          const std::string& expected_client_class) {
+        ASSERT_TRUE(token);
+        boost::shared_ptr<TokenMember> member =
+            boost::dynamic_pointer_cast<TokenMember>(token);
+        ASSERT_TRUE(member);
+
+        EXPECT_EQ(expected_client_class, member->getClientClass());
+    }
+
+    /// @rief This tests attempts to parse the expression then checks
+    /// if the number of tokens is correct and the TokenMember is as
+    /// expected.
+    ///
+    /// @param expr expression to be parsed
+    /// @param check_defined closure checking if the client class is defined
+    /// @param exp_client_class expected client class name to be parsed
+    /// @param exp_tokens expected number of tokens
+    void testMember(const std::string& expr,
+                    EvalContext::CheckDefined check_defined,
+                    const std::string& exp_client_class,
+                    int exp_tokens) {
+        EvalContext eval(Option::V6, check_defined);
+
+        // parse the expression
+        try {
+            parsed_ = eval.parseString(expr);
+        }
+        catch (const EvalParseError& ex) {
+            FAIL() <<"Exception thrown: " << ex.what();
+            return;
+        }
+
+        // Parsing should succeed and return a token.
+        EXPECT_TRUE(parsed_);
+
+        // There should be the expected number of tokens.
+        ASSERT_EQ(exp_tokens, eval.expression.size());
+
+        // checked that the first token is TokenRelay6Field and that
+        // is has the correct attributes
+        checkTokenMember(eval.expression.at(0), exp_client_class);
+    }
+
     /// @brief checks if the given token is a substring operator
     void checkTokenSubstring(const TokenPtr& token) {
         ASSERT_TRUE(token);
@@ -406,6 +454,22 @@ public:
         EXPECT_TRUE(conc);
     }
 
+    /// @brief checks if the given token is an ifelse operator
+    void checkTokenIfElse(const TokenPtr& token) {
+        ASSERT_TRUE(token);
+        boost::shared_ptr<TokenIfElse> alt =
+            boost::dynamic_pointer_cast<TokenIfElse>(token);
+        EXPECT_TRUE(alt);
+    }
+
+    /// @brief checks if the given token is a hexstring operator
+    void checkTokenToHexString(const TokenPtr& token) {
+        ASSERT_TRUE(token);
+        boost::shared_ptr<TokenToHexString> tohex =
+            boost::dynamic_pointer_cast<TokenToHexString>(token);
+        EXPECT_TRUE(tohex);
+    }
+
     /// @brief checks if the given expression raises the expected message
     /// when it is parsed.
     void checkError(const string& expr, const string& msg) {
@@ -615,6 +679,26 @@ public:
         EXPECT_EQ(field, vendor->getField());
     }
 
+    /// @brief checks if the given token is a sub-option with the expected
+    /// parent option and sub-option codes and representation type
+    /// @param token token to be checked
+    /// @param expected_code expected option code
+    /// @param expected_sub_code expected sub-option code
+    /// @param expected_repr expected representation (text, hex, exists)
+    void checkTokenSubOption(const TokenPtr& token,
+                             uint16_t expected_code,
+                             uint16_t expected_sub_code,
+                             TokenOption::RepresentationType expected_repr) {
+        ASSERT_TRUE(token);
+        boost::shared_ptr<TokenSubOption> sub =
+            boost::dynamic_pointer_cast<TokenSubOption>(token);
+        ASSERT_TRUE(sub);
+
+        EXPECT_EQ(expected_code, sub->getCode());
+        EXPECT_EQ(expected_sub_code, sub->getSubCode());
+        EXPECT_EQ(expected_repr, sub->getRepresentation());
+    }
+
     Option::Universe universe_; ///< Universe (V4 or V6)
     bool parsed_; ///< Parsing status
 
@@ -934,7 +1018,15 @@ TEST_F(EvalContextTest, relay6OptionHex) {
                      2, 85, TokenOption::HEXADECIMAL, 3);
 }
 
-// Test the nest level of a relay6 option should be in [0..32[
+// Test the parsing of a relay6 option in reverse order
+TEST_F(EvalContextTest, relay6OptionReverse) {
+    EvalContext eval(Option::V6);
+
+    testRelay6Option("relay6[-1].option[123].text == 'foo'",
+                     -1, 123, TokenOption::TEXTUAL, 3);
+}
+
+// Test the nest level of a relay6 option should be in [-32..32[
 TEST_F(EvalContextTest, relay6OptionLimits) {
     EvalContext eval(Option::V6);
 
@@ -946,11 +1038,14 @@ TEST_F(EvalContextTest, relay6OptionLimits) {
 
     checkError("relay6[32].option[123].text == 'foo'",
                "<string>:1.8-9: Nest level has invalid value in 32. "
-               "Allowed range: 0..31");
+               "Allowed range: -32..31");
 
-    // next level must be a positive number
-    checkError("relay6[-1].option[123].text == 'foo'",
-               "<string>:1.8-9: Invalid value in -1. Allowed range: 0..255");
+    // min nest level is minus hop count limit
+    testRelay6Option("relay6[-32].option[123].text == 'foo'",
+                     -32, 123, TokenOption::TEXTUAL, 3);
+
+    checkError("relay6[-33].option[123].text == 'foo'",
+               "<string>:1.8-10: Nest level has invalid value in -33. Allowed range: -32..31");
 }
 
 // Verify that relay6[13].option is not usable in v4
@@ -989,6 +1084,12 @@ TEST_F(EvalContextTest, pkt4FieldChaddr) {
     testPkt4Field("pkt4.mac == 0x000102030405", TokenPkt4::CHADDR, 3);
 }
 
+// Tests whether chaddr field in DHCPv4 can be accessed and converted.
+TEST_F(EvalContextTest, pkt4FieldChaddrHexa) {
+    testPkt4Field("hexstring(pkt4.mac, ':') == '00:01:02:03:04:05'",
+                  TokenPkt4::CHADDR, 5);
+}
+
 // Tests whether hlen field in DHCPv4 can be accessed.
 TEST_F(EvalContextTest, pkt4FieldHlen) {
     testPkt4Field("pkt4.hlen == 0x6", TokenPkt4::HLEN, 3);
@@ -1041,7 +1142,7 @@ TEST_F(EvalContextTest, relay6FieldPeerAddr) {
                     1, TokenRelay6Field::PEERADDR, 3);
 }
 
-// Verify that relay6[13].<field> is not usable in v4
+// Verify that relay6[0].<field> is not usable in v4
 TEST_F(EvalContextTest, relay6FieldError) {
     universe_ = Option::V4;
 
@@ -1052,6 +1153,31 @@ TEST_F(EvalContextTest, relay6FieldError) {
                "<string>:1.8: Nest level invalid for DHCPv4 packets");
 }
 
+// Tests parsing of member with defined class
+TEST_F(EvalContextTest, member) {
+    auto check_defined = [](const ClientClass& cc) { return (cc == "foo"); };
+    testMember("member('foo')", check_defined, "foo", 1);
+}
+
+// Test parsing of member with not defined class
+TEST_F(EvalContextTest, memberError) {
+    auto check_defined = [](const ClientClass& cc) { return (cc == "foo"); };
+    EvalContext eval(Option::V6, check_defined);
+    parsed_ = false;
+    try {
+        parsed_ = eval.parseString("member('bar')");
+        FAIL() << "Expected EvalParseError but nothing was raised";
+    }
+    catch (const EvalParseError& ex) {
+        EXPECT_EQ("<string>:1.8-12: Not defined client class 'bar'",
+                  std::string(ex.what()));
+        EXPECT_FALSE(parsed_);
+    }
+    catch (...) {
+        FAIL() << "Expected EvalParseError but something else was raised";
+    }
+}
+
 // Test parsing of logical operators
 TEST_F(EvalContextTest, logicalOps) {
     // option.exists
@@ -1198,6 +1324,45 @@ TEST_F(EvalContextTest, concat) {
     checkTokenConcat(tmp3);
 }
 
+// Test the parsing of an ifelse expression
+TEST_F(EvalContextTest, ifElse) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ =
+        eval.parseString("ifelse('foo' == 'bar', 'us', 'them') == 'you'"));
+
+    ASSERT_EQ(8, eval.expression.size());
+
+    TokenPtr tmp1 = eval.expression.at(2);
+    TokenPtr tmp2 = eval.expression.at(3);
+    TokenPtr tmp3 = eval.expression.at(4);
+    TokenPtr tmp4 = eval.expression.at(5);
+
+    checkTokenEq(tmp1);
+    checkTokenString(tmp2, "us");
+    checkTokenString(tmp3, "them");
+    checkTokenIfElse(tmp4);
+}
+
+// Test the parsing of a hexstring expression
+TEST_F(EvalContextTest, toHexString) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ =
+        eval.parseString("hexstring(0x666f,'-') == '66-6f'"));
+    EXPECT_TRUE(parsed_);
+
+    ASSERT_EQ(5, eval.expression.size());
+
+    TokenPtr tmp1 = eval.expression.at(0);
+    TokenPtr tmp2 = eval.expression.at(1);
+    TokenPtr tmp3 = eval.expression.at(2);
+
+    checkTokenHexString(tmp1, "fo");
+    checkTokenString(tmp2, "-");
+    checkTokenToHexString(tmp3);
+}
+
 //
 // Test some scanner error cases
 TEST_F(EvalContextTest, scanErrors) {
@@ -1207,11 +1372,14 @@ TEST_F(EvalContextTest, scanErrors) {
     checkError("0x123h", "<string>:1.6: Invalid character: h");
     checkError(":1", "<string>:1.1: Invalid character: :");
     checkError("=", "<string>:1.1: Invalid character: =");
+
+    // Typo should be handled as well.
     checkError("subtring", "<string>:1.1: Invalid character: s");
     checkError("foo", "<string>:1.1: Invalid character: f");
     checkError(" bar", "<string>:1.2: Invalid character: b");
     checkError("relay[12].hex == 'foo'", "<string>:1.1: Invalid character: r");
     checkError("pkt4.ziaddr", "<string>:1.6: Invalid character: z");
+    checkError("members('foo'", "<string>:1.7: Invalid character: s");
 }
 
 // Tests some scanner/parser error cases
@@ -1340,10 +1508,21 @@ TEST_F(EvalContextTest, parseErrors) {
                "expecting integer");
     checkError("substring('foobar',1,a) == 'foo'",
                "<string>:1.22: Invalid character: a");
+    string long_text = "substring('foobar',1,65535) == ";
+    for (int i = 0; i < (1 << 16); ++i) {
+        long_text += "0";
+    }
+    long_text += "'";
+    checkError(long_text,
+               "<string>:1.65568: Invalid character: '");
     checkError("concat('foobar') == 'f'",
                "<string>:1.16: syntax error, unexpected ), expecting \",\"");
     checkError("concat('foo','bar','') == 'foobar'",
                "<string>:1.19: syntax error, unexpected \",\", expecting )");
+    checkError("ifelse('foo'=='bar','foo')",
+               "<string>:1.26: syntax error, unexpected ), expecting \",\"");
+    checkError("ifelse('foo'=='bar','foo','bar','')",
+               "<string>:1.32: syntax error, unexpected \",\", expecting )");
 }
 
 // Tests some type error cases
@@ -1359,7 +1538,7 @@ TEST_F(EvalContextTest, typeErrors) {
                "hexstring, expecting integer");
 
     // With the #4483 addition, all integers are treated as 4 byte strings,
-    // so those checks no longer makes sense. Commeting it out.
+    // so those checks no longer makes sense. Commenting it out.
     // checkError("concat('foo',3) == 'foo3'",
     //            "<string>:1.14: syntax error, unexpected integer");
     // checkError("concat(3,'foo') == '3foo'",
@@ -1374,6 +1553,22 @@ TEST_F(EvalContextTest, typeErrors) {
                "<string>:1.8-10: syntax error, unexpected and, expecting ==");
     checkError("'true' or 'false'",
                "<string>:1.8-9: syntax error, unexpected or, expecting ==");
+    // Ifelse requires a boolean condition and string branches.
+    checkError("ifelse('foobar','foo','bar')",
+               "<string>:1.16: syntax error, unexpected \",\", expecting ==");
+    checkError("ifelse('foo'=='bar','foo'=='foo','bar')",
+               "<string>:1.26-27: syntax error, unexpected ==, "
+               "expecting \",\"");
+    checkError("ifelse('foo'=='bar','foo','bar'=='bar')",
+               "<string>:1.32-33: syntax error, unexpected ==, expecting )");
+
+    // Member uses quotes around the client class name.
+    checkError("member(foo)", "<string>:1.8: Invalid character: f");
+
+    // sub-option by name is not supported.
+    checkError("option[123].option[host-name].exists",
+               "<string>:1.20-28: syntax error, unexpected option name, "
+               "expecting integer");
 }
 
 
@@ -1467,6 +1662,46 @@ TEST_F(EvalContextTest, vendorClass6DataIndex) {
     testVendorClass("vendor-class[4491].data[3] == 0x1234", Option::V6, 4491, 3);
 }
 
+// Test the parsing of a sub-option with perent by code.
+TEST_F(EvalContextTest, subOptionWithCode) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ = eval.parseString("option[123].option[234].text == 'foo'"));
+    EXPECT_TRUE(parsed_);
+    ASSERT_EQ(3, eval.expression.size());
+    checkTokenSubOption(eval.expression.at(0), 123, 234, TokenOption::TEXTUAL);
+}
+
+// Test the parsing of a sub-option with perent by name.
+TEST_F(EvalContextTest, subOptionWithName) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ = eval.parseString("option[host-name].option[123].text == 'foo'"));
+    EXPECT_TRUE(parsed_);
+    ASSERT_EQ(3, eval.expression.size());
+    checkTokenSubOption(eval.expression.at(0), 12, 123, TokenOption::TEXTUAL);
+}
+
+// Test the parsing of a sub-option existence
+TEST_F(EvalContextTest, subOptionExists) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ = eval.parseString("option[100].option[200].exists"));
+    EXPECT_TRUE(parsed_);
+    ASSERT_EQ(1, eval.expression.size());
+    checkTokenSubOption(eval.expression.at(0), 100, 200, TokenOption::EXISTS);
+}
+
+// Test parsing of a sub-option represented as hexadecimal string.
+TEST_F(EvalContextTest, subOptionHex) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ = eval.parseString("option[123].option[234].hex == 0x666F6F"));
+    EXPECT_TRUE(parsed_);
+    ASSERT_EQ(3, eval.expression.size());
+    checkTokenSubOption(eval.expression.at(0), 123, 234, TokenOption::HEXADECIMAL);
+}
+
 // Checks if integer expressions can be parsed and checked for equality.
 TEST_F(EvalContextTest, integer1) {
 
