@@ -1,9 +1,11 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <hooks/callout_handle.h>
 #include <hooks/callout_manager.h>
 #include <hooks/callout_manager.h>
@@ -48,19 +50,44 @@ HooksManager::calloutsPresent(int index) {
     return (getHooksManager().calloutsPresentInternal(index));
 }
 
+bool
+HooksManager::commandHandlersPresentInternal(const std::string& command_name) {
+    conditionallyInitialize();
+    return (callout_manager_->commandHandlersPresent(command_name));
+}
+
+bool
+HooksManager::commandHandlersPresent(const std::string& command_name) {
+    return (getHooksManager().commandHandlersPresentInternal(command_name));
+}
+
 // Call the callouts
 
 void
 HooksManager::callCalloutsInternal(int index, CalloutHandle& handle) {
     conditionallyInitialize();
-    return (callout_manager_->callCallouts(index, handle));
+    callout_manager_->callCallouts(index, handle);
 }
 
 void
 HooksManager::callCallouts(int index, CalloutHandle& handle) {
-    return (getHooksManager().callCalloutsInternal(index, handle));
+    getHooksManager().callCalloutsInternal(index, handle);
 }
 
+void
+HooksManager::callCommandHandlersInternal(const std::string& command_name,
+                                          CalloutHandle& handle) {
+    conditionallyInitialize();
+    callout_manager_->callCommandHandlers(command_name, handle);
+}
+
+void
+HooksManager::callCommandHandlers(const std::string& command_name,
+                                  CalloutHandle& handle) {
+    getHooksManager().callCommandHandlersInternal(command_name, handle);
+}
+
+
 // Load the libraries.  This will delete the previously-loaded libraries
 // (if present) and load new ones.
 
@@ -103,6 +130,7 @@ HooksManager::unloadLibrariesInternal() {
     // ease debugging.
     lm_collection_.reset();
     callout_manager_.reset();
+    ServerHooks::getServerHooks().getParkingLotsPtr()->clear();
 }
 
 void HooksManager::unloadLibraries() {
@@ -199,5 +227,11 @@ HooksManager::validateLibraries(const std::vector<std::string>& libraries) {
     return (LibraryManagerCollection::validateLibraries(libraries));
 }
 
+// Shared callout manager
+boost::shared_ptr<CalloutManager>&
+HooksManager::getSharedCalloutManager() {
+    return (getHooksManager().shared_callout_manager_);
+}
+
 } // namespace util
 } // namespace isc
