@@ -1,13 +1,17 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <hooks/callout_manager.h>
 #include <hooks/library_handle.h>
 #include <hooks/hooks_manager.h>
 
+#include <iostream>
+
 namespace isc {
 namespace hooks {
 
@@ -33,6 +37,16 @@ LibraryHandle::registerCallout(const std::string& name, CalloutPtr callout) {
     }
 }
 
+void
+LibraryHandle::registerCommandCallout(const std::string& command_name,
+                                      CalloutPtr callout) {
+    // Register hook point for this command, if one doesn't exist.
+    callout_manager_->registerCommandHook(command_name);
+    // Register the command handler as a callout.
+    registerCallout(ServerHooks::commandToHookName(command_name), callout);
+}
+
+
 bool
 LibraryHandle::deregisterCallout(const std::string& name, CalloutPtr callout) {
     int saved_index = callout_manager_->getLibraryIndex();
@@ -66,7 +80,7 @@ LibraryHandle::deregisterAllCallouts(const std::string& name) {
 }
 
 isc::data::ConstElementPtr
-LibraryHandle::getParameter(const std::string& name) {
+LibraryHandle::getParameters() {
     HookLibsCollection libinfo = HooksManager::getHooksManager().getLibraryInfo();
 
     int index = index_;
@@ -89,8 +103,13 @@ LibraryHandle::getParameter(const std::string& name) {
     // * 1..numlib - indexes for actual libraries
     // * INT_MAX - post-user library callout
 
+    return (libinfo[index - 1].second);
+}
+
+isc::data::ConstElementPtr
+LibraryHandle::getParameter(const std::string& name) {
     // Try to find appropriate parameter. May return null pointer
-    isc::data::ConstElementPtr params = libinfo[index - 1].second;
+    isc::data::ConstElementPtr params = getParameters();
     if (!params) {
         return (isc::data::ConstElementPtr());
     }
@@ -99,5 +118,23 @@ LibraryHandle::getParameter(const std::string& name) {
     return (params->get(name));
 }
 
+std::vector<std::string>
+LibraryHandle::getParameterNames() {
+    std::vector<std::string> names;
+    // Find all parameter names.
+    isc::data::ConstElementPtr params = getParameters();
+    if (!params ||
+        (params->getType() != isc::data::Element::map) ||
+        (params->size() == 0)) {
+        return (names);
+    }
+    auto map = params->mapValue();
+    for (auto elem = map.begin(); elem != map.end(); ++elem) {
+        names.push_back(elem->first);
+    }
+    return (names);
+}
+
+
 } // namespace util
 } // namespace isc
