@@ -4,6 +4,8 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <cctype>
 #include <cstddef>
 #include <fstream>
@@ -35,8 +37,6 @@ using namespace std;
 using namespace isc::log;
 using namespace isc::util;
 
-static const char* VERSION = "1.0-0";
-
 /// \file log/compiler/message.cc
 /// \brief Message Compiler
 ///
