@@ -1,4 +1,4 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -28,7 +28,7 @@ namespace stats {
 /// As of May 2015, Tomek ran performance benchmarks (see unit-tests in
 /// stats_mgr_unittest.cc with performance in their names) and it seems
 /// the code is able to register ~2.5-3 million observations per second, even
-/// with 1000 different statistics recored. That seems sufficient for now,
+/// with 1000 different statistics recorded. That seems sufficient for now,
 /// so there is no immediate need to develop any multi-threading solutions
 /// for now. However, should this decision be revised in the future, the
 /// best place for it would to be modify @ref addObservation method here.
@@ -47,7 +47,7 @@ namespace stats {
 ///   significantly. While it's possible to log on sufficiently high debug
 ///   level, such a log would be not that useful)
 /// - dependency (statistics are intended to be a lightweight library, adding
-///   dependency on libkea-log, which has its own dependecies, including
+///   dependency on libkea-log, which has its own dependencies, including
 ///   external log4cplus, is against 'lightweight' design)
 /// - if logging of specific statistics is warranted, it is recommended to
 ///   add log entries in the code that calls StatsMgr.
@@ -134,26 +134,37 @@ class StatsMgr : public boost::noncopyable {
     /// approach. For sample count constrained approach, see @ref
     /// setMaxSampleCount() below.
     ///
-    /// @todo: Not implemented.
     ///
-    /// Example: to set a statistic to keep observations for the last 5 minutes,
-    /// call setMaxSampleAge("incoming-packets", time_duration(0,5,0,0));
+    /// @param name name of the observation
+    /// @param duration determines maximum age of samples
+    /// @return true if successful, false if there's no such statistic
+    /// Example:
+    /// To set a statistic to keep observations for the last 5 minutes, call:
+    /// setMaxSampleAge("incoming-packets", time_duration(0, 5, 0, 0));
     /// to revert statistic to a single value, call:
-    /// setMaxSampleAge("incoming-packets" time_duration(0,0,0,0))
-    void setMaxSampleAge(const std::string& name, const StatsDuration& duration);
+    /// setMaxSampleAge("incoming-packets", time_duration(0, 0, 0, 0));
+    bool setMaxSampleAge(const std::string& name, const StatsDuration& duration);
 
     /// @brief Determines how many samples of a given statistic should be kept.
     ///
     /// Specifies that statistic name should be stored not as single value, but
     /// rather as a set of values. In this form, at most max_samples will be kept.
-    /// When adding max_samples+1 sample, the oldest sample will be discarded.
+    /// When adding max_samples + 1 sample, the oldest sample will be discarded.
     ///
-    /// @todo: Not implemented.
     ///
+    /// @param name name of the observation
+    /// @param max_samples how many samples of a given statistic should be kept
+    /// @return true if successful, false if there's no such statistic
     /// Example:
     /// To set a statistic to keep the last 100 observations, call:
     /// setMaxSampleCount("incoming-packets", 100);
-    void setMaxSampleCount(const std::string& name, uint32_t max_samples);
+    bool setMaxSampleCount(const std::string& name, uint32_t max_samples);
+
+    /// @brief Set duration limit for all collected statistics.
+    void setMaxSampleAgeAll(const StatsDuration& duration);
+
+    /// @brief Set count limit for all collected statistics.
+    void setMaxSampleCountAll(uint32_t max_samples);
 
     /// @}
 
@@ -183,6 +194,12 @@ class StatsMgr : public boost::noncopyable {
     /// @brief Removes all collected statistics.
     void removeAll();
 
+    /// @brief Returns size of specified statistic.
+    ///
+    /// @param name name of the statistic which size should be return.
+    /// @return size of specified statistic, 0 means lack of given statistic.
+    size_t getSize(const std::string& name) const;
+
     /// @brief Returns number of available statistics.
     ///
     /// @return number of recorded statistics.
@@ -223,7 +240,7 @@ class StatsMgr : public boost::noncopyable {
     /// @return returns full statistic name in form context[index].stat_name
     template<typename Type>
     static std::string generateName(const std::string& context, Type index,
-                             const std::string& stat_name) {
+                                    const std::string& stat_name) {
         std::stringstream name;
         name << context << "[" << index << "]." << stat_name;
         return (name.str());
@@ -239,7 +256,7 @@ class StatsMgr : public boost::noncopyable {
     ///
     /// This method handles statistic-get command, which returns value
     /// of a given statistic). It expects one parameter stored in params map:
-    /// name: name-of-the-statistic
+    /// name: name of the statistic
     ///
     /// Example params structure:
     /// {
@@ -257,7 +274,7 @@ class StatsMgr : public boost::noncopyable {
     ///
     /// This method handles statistic-reset command, which resets value
     /// of a given statistic. It expects one parameter stored in params map:
-    /// name: name-of-the-statistic
+    /// name: name of the statistic
     ///
     /// Example params structure:
     /// {
@@ -275,7 +292,7 @@ class StatsMgr : public boost::noncopyable {
     ///
     /// This method handles statistic-reset command, which removes a given
     /// statistic completely. It expects one parameter stored in params map:
-    /// name: name-of-the-statistic
+    /// name: name of the statistic
     ///
     /// Example params structure:
     /// {
@@ -289,6 +306,50 @@ class StatsMgr : public boost::noncopyable {
     statisticRemoveHandler(const std::string& name,
                            const isc::data::ConstElementPtr& params);
 
+    /// @brief Handles statistic-sample-age-set command
+    ///
+    /// This method handles statistic-sample-age-set command,
+    /// which set max_sample_age_ limit of a given statistic
+    /// and leaves max_sample_count_ disabled.
+    /// It expects two parameters stored in params map:
+    /// name: name of the statistic
+    /// duration: time limit expressed as a number of seconds
+    ///
+    /// Example params structure:
+    /// {
+    ///     "name": "packets-received",
+    ///     "duration": 1245
+    /// }
+    ///
+    /// @param name name of the command (ignored, should be "statistic-sample-age-set")
+    /// @param params structure containing a map that contains "name" and "duration"
+    /// @return answer containing information about successfully setup limit of statistic
+    static isc::data::ConstElementPtr
+    statisticSetMaxSampleAgeHandler(const std::string& name,
+                                    const isc::data::ConstElementPtr& params);
+
+    /// @brief Handles statistic-sample-count-set command
+    ///
+    /// This method handles statistic-sample-count-set command,
+    /// which set max_sample_count_ limit of a given statistic
+    /// and leaves max_sample_age_ disabled.
+    /// It expects two parameters stored in params map:
+    /// name: name of the statistic
+    /// max-samples: count limit
+    ///
+    /// Example params structure:
+    /// {
+    ///     "name": "packets-received",
+    ///     "max-samples": 15
+    /// }
+    ///
+    /// @param name name of the command (ignored, should be "statistic-sample-count-set")
+    /// @param params structure containing a map that contains "name" and "max-samples"
+    /// @return answer containing information about successfully setup limit of statistic
+    static isc::data::ConstElementPtr
+    statisticSetMaxSampleCountHandler(const std::string& name,
+                                      const isc::data::ConstElementPtr& params);
+
     /// @brief Handles statistic-get-all command
     ///
     /// This method handles statistic-get-all command, which returns values
@@ -325,9 +386,47 @@ class StatsMgr : public boost::noncopyable {
     statisticRemoveAllHandler(const std::string& name,
                               const isc::data::ConstElementPtr& params);
 
+    /// @brief Handles statistic-sample-age-set-all command
+    ///
+    /// This method handles statistic-sample-age-set-all command,
+    /// which set max_sample_age_ limit to all statistics.
+    /// It expects one parameter stored in params map:
+    /// duration: limit expressed as a number of seconds
+    ///
+    /// Example params structure:
+    /// {
+    ///     "duration": 1245
+    /// }
+    ///
+    /// @param name name of the command (ignored, should be "statistic-sample-age-set-all")
+    /// @param params structure containing a map that contains "duration"
+    /// @return answer confirming success of this operation
+    static isc::data::ConstElementPtr
+    statisticSetMaxSampleAgeAllHandler(const std::string& name,
+                                       const isc::data::ConstElementPtr& params);
+
+    /// @brief Handles statistic-sample-count-set-all command
+    ///
+    /// This method handles statistic-sample-count-set-all command,
+    /// which set max_sample_count_ limit of all statistics.
+    /// It expects one parameter stored in params map:
+    /// max-samples: count limit
+    ///
+    /// Example params structure:
+    /// {
+    ///     "max-samples": 15
+    /// }
+    ///
+    /// @param name name of the command (ignored, should be "statistic-sample-count-set-all")
+    /// @param params structure containing a map that contains "max-samples"
+    /// @return answer confirming success of this operation
+    static isc::data::ConstElementPtr
+    statisticSetMaxSampleCountAllHandler(const std::string& name,
+                                         const isc::data::ConstElementPtr& params);
+
     /// @}
 
- private:
+private:
 
     /// @brief Private constructor.
     /// StatsMgr is a singleton. It should be accessed using @ref instance
@@ -422,6 +521,45 @@ class StatsMgr : public boost::noncopyable {
                             std::string& name,
                             std::string& reason);
 
+    /// @brief Utility method that attempts to extract duration limit for
+    /// a given statistic
+    ///
+    /// This method attempts to extract duration limit for a given statistic
+    /// from the params structure.
+    /// It is expected to be a map that contains four 'duration' elements: 'hours',
+    /// 'minutes', 'seconds' and 'milliseconds'
+    /// all are of type int. If present as expected, statistic duration
+    /// limit is set and true is returned.
+    /// If any of these four parameters is missing or is of incorrect type,
+    /// the reason is specified in reason parameter and false is returned.
+    ///
+    /// @param params parameters structure received in command
+    /// @param duration [out] duration limit for the statistic (if no error detected)
+    /// @param reason [out] failure reason (if error is detected)
+    /// @return true (if everything is ok), false otherwise
+    static bool getStatDuration(const isc::data::ConstElementPtr& params,
+                                StatsDuration& duration,
+                                std::string& reason);
+
+    /// @brief Utility method that attempts to extract count limit for
+    /// a given statistic
+    ///
+    /// This method attempts to extract count limit for a given statistic
+    /// from the params structure.
+    /// It is expected to be a map that contains 'max-samples' element,
+    /// that is of type int. If present as expected, statistic count
+    /// limit (max_samples) is set and true is returned.
+    /// If missing or is of incorrect type, the reason is specified in reason
+    /// parameter and false is returned.
+    ///
+    /// @param params parameters structure received in command
+    /// @param max_samples [out] count limit for the statistic (if no error detected)
+    /// @param reason [out] failure reason (if error is detected)
+    /// @return true (if everything is ok), false otherwise
+    static bool getStatMaxSamples(const isc::data::ConstElementPtr& params,
+                                  uint32_t& max_samples,
+                                  std::string& reason);
+
     // This is a global context. All statistics will initially be stored here.
     StatContextPtr global_;
 };
