@@ -1,4 +1,4 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -48,7 +48,6 @@ public:
 
 // Basic test for statistics manager interface.
 TEST_F(StatsMgrTest, basic) {
-
     // Getting an instance
     EXPECT_NO_THROW(StatsMgr::instance());
 
@@ -66,8 +65,8 @@ TEST_F(StatsMgrTest, integerStat) {
     EXPECT_NO_THROW(alpha = StatsMgr::instance().getObservation("alpha"));
     ASSERT_TRUE(alpha);
 
-    std::string exp = "{ \"alpha\": [ [ 1234, \""
-        + isc::util::ptimeToText(alpha->getInteger().second) + "\" ] ] }";
+    std::string exp = "{ \"alpha\": [ [ 1234, \"" +
+        isc::util::ptimeToText(alpha->getInteger().second) + "\" ] ] }";
 
     EXPECT_EQ(exp, StatsMgr::instance().get("alpha")->str());
 }
@@ -81,8 +80,8 @@ TEST_F(StatsMgrTest, floatStat) {
     EXPECT_NO_THROW(beta = StatsMgr::instance().getObservation("beta"));
     ASSERT_TRUE(beta);
 
-    std::string exp = "{ \"beta\": [ [ 12.34, \""
-        + isc::util::ptimeToText(beta->getFloat().second) + "\" ] ] }";
+    std::string exp = "{ \"beta\": [ [ 12.34, \"" +
+        isc::util::ptimeToText(beta->getFloat().second) + "\" ] ] }";
 
     EXPECT_EQ(exp, StatsMgr::instance().get("beta")->str());
 }
@@ -91,14 +90,14 @@ TEST_F(StatsMgrTest, floatStat) {
 // a duration statistic.
 TEST_F(StatsMgrTest, durationStat) {
     EXPECT_NO_THROW(StatsMgr::instance().setValue("gamma",
-                                                  microsec::time_duration(1,2,3,4)));
+                                                  microsec::time_duration(1, 2, 3, 4)));
 
     ObservationPtr gamma;
     EXPECT_NO_THROW(gamma = StatsMgr::instance().getObservation("gamma"));
     ASSERT_TRUE(gamma);
 
-    std::string exp = "{ \"gamma\": [ [ \"01:02:03.000004\", \""
-        + isc::util::ptimeToText(gamma->getDuration().second) + "\" ] ] }";
+    std::string exp = "{ \"gamma\": [ [ \"01:02:03.000004\", \"" +
+        isc::util::ptimeToText(gamma->getDuration().second) + "\" ] ] }";
 
     EXPECT_EQ(exp, StatsMgr::instance().get("gamma")->str());
 }
@@ -113,37 +112,136 @@ TEST_F(StatsMgrTest, stringStat) {
     EXPECT_NO_THROW(delta = StatsMgr::instance().getObservation("delta"));
     ASSERT_TRUE(delta);
 
-    std::string exp = "{ \"delta\": [ [ \"Lorem ipsum\", \""
-        + isc::util::ptimeToText(delta->getString().second) + "\" ] ] }";
+    std::string exp = "{ \"delta\": [ [ \"Lorem ipsum\", \"" +
+        isc::util::ptimeToText(delta->getString().second) + "\" ] ] }";
 
     EXPECT_EQ(exp, StatsMgr::instance().get("delta")->str());
 }
 
-// Setting limits is currently not implemented, so those methods should
-// throw.
-TEST_F(StatsMgrTest, setLimits) {
-    EXPECT_THROW(StatsMgr::instance().setMaxSampleAge("foo",
-                                                      time_duration(1,0,0,0)),
-                 NotImplemented);
+// Basic test of getSize function.
+TEST_F(StatsMgrTest, getSize) {
+    StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
+    StatsMgr::instance().setValue("beta", 12.34);
+    StatsMgr::instance().setValue("gamma", microsec::time_duration(1, 2, 3, 4));
+    StatsMgr::instance().setValue("delta", "Lorem ipsum");
 
-    EXPECT_THROW(StatsMgr::instance().setMaxSampleCount("foo", 100),
-                 NotImplemented);
+    EXPECT_NO_THROW(StatsMgr::instance().getSize("alpha"));
+    EXPECT_NO_THROW(StatsMgr::instance().getSize("beta"));
+    EXPECT_NO_THROW(StatsMgr::instance().getSize("gamma"));
+    EXPECT_NO_THROW(StatsMgr::instance().getSize("delta"));
+
+    EXPECT_EQ(StatsMgr::instance().getSize("alpha"), 1);
+    EXPECT_EQ(StatsMgr::instance().getSize("beta"), 1);
+    EXPECT_EQ(StatsMgr::instance().getSize("gamma"), 1);
+    EXPECT_EQ(StatsMgr::instance().getSize("delta"), 1);
+}
+
+// Test checks whether setting age limit and count limit works properly.
+TEST_F(StatsMgrTest, setLimits) {
+    // Initializing of an integer type observation
+    StatsMgr::instance().setValue("foo", static_cast<int64_t>(1));
+
+    EXPECT_NO_THROW(StatsMgr::instance().setMaxSampleAge("foo",
+                                                         time_duration(0, 0, 1, 0)));
+
+    for (uint32_t i = 0; i < 10; ++i) {
+        if (i == 5) {
+            sleep(1); // wait one second to force exceeding the time limit
+        }
+        StatsMgr::instance().setValue("foo", static_cast<int64_t>(i));
+    }
+
+    EXPECT_EQ(StatsMgr::instance().getSize("foo"), 5);
+    EXPECT_NO_THROW(StatsMgr::instance().setMaxSampleCount("foo", 100));
+
+    for (int64_t i = 0; i < 200; ++i) {
+        StatsMgr::instance().setValue("foo", i);
+    }
+
+    EXPECT_EQ(StatsMgr::instance().getSize("foo"), 100);
+}
+
+// Test checks whether setting age limit and count limit to existing
+// statistics works properly.
+TEST_F(StatsMgrTest, setLimitsAll) {
+    // Set a couple of statistics
+    StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
+    StatsMgr::instance().setValue("beta", 12.34);
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
+    StatsMgr::instance().setValue("delta", "Lorem ipsum");
+
+    // check the setting of time limit to existing statistics
+    EXPECT_NO_THROW(StatsMgr::instance().setMaxSampleAgeAll(time_duration(0, 0, 1, 0)));
+
+    // check if time limit was set properly and whether count limit is disabled
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().second,
+              time_duration(0, 0, 1, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleAge().second,
+              time_duration(0, 0, 1, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleCount().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleAge().second,
+              time_duration(0, 0, 1, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleCount().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleAge().second,
+              time_duration(0, 0, 1, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleCount().first, false);
+
+    // check the setting of count limit to existing statistics
+    EXPECT_NO_THROW(StatsMgr::instance().setMaxSampleCountAll(1200));
+
+    // check if count limit was set properly and whether count limit is disabled
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().second, 1200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleCount().second, 1200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleAge().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleCount().second, 1200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleAge().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleCount().second, 1200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleAge().first, false);
 }
 
 // This test checks whether a single (get("foo")) and all (getAll())
 // statistics are reported properly.
 TEST_F(StatsMgrTest, getGetAll) {
-
     // Set a couple of statistics
     StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
     StatsMgr::instance().setValue("beta", 12.34);
-    StatsMgr::instance().setValue("gamma", time_duration(1,2,3,4));
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
     StatsMgr::instance().setValue("delta", "Lorem");
 
+    // The string's representation of firstly added statistics
+    std::string alpha_first = ", [ 1234, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("alpha")
+                                   ->getInteger().second) + "\" ] ]";
+    std::string beta_first = ", [ 12.34, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("beta")
+                                   ->getFloat().second) + "\" ] ]";
+    std::string gamma_first = ", [ \"01:02:03.000004\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("gamma")
+                                   ->getDuration().second) + "\" ] ]";
+    std::string delta_first = ", [ \"Lorem\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("delta")
+                                   ->getString().second) + "\" ] ]";
+
     // Now add some values to them
     StatsMgr::instance().addValue("alpha", static_cast<int64_t>(5678));
     StatsMgr::instance().addValue("beta", 56.78);
-    StatsMgr::instance().addValue("gamma", time_duration(5,6,7,8));
+    StatsMgr::instance().addValue("gamma", time_duration(5, 6, 7, 8));
     StatsMgr::instance().addValue("delta", " ipsum");
 
     // There should be 4 statistics reported
@@ -160,18 +258,18 @@ TEST_F(StatsMgrTest, getGetAll) {
     ASSERT_TRUE(rep_gamma);
     ASSERT_TRUE(rep_delta);
 
-    std::string exp_str_alpha = "[ [ 6912, \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("alpha")
-                                   ->getInteger().second) + "\" ] ]";
-    std::string exp_str_beta = "[ [ 69.12, \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("beta")
-                                   ->getFloat().second) + "\" ] ]";
-    std::string exp_str_gamma = "[ [ \"06:08:10.000012\", \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("gamma")
-                                   ->getDuration().second) + "\" ] ]";
-    std::string exp_str_delta = "[ [ \"Lorem ipsum\", \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("delta")
-                                   ->getString().second) + "\" ] ]";
+    std::string exp_str_alpha = "[ [ 6912, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("alpha")
+                                   ->getInteger().second) + "\" ]" + alpha_first;
+    std::string exp_str_beta = "[ [ 69.12, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("beta")
+                                   ->getFloat().second) + "\" ]" + beta_first;
+    std::string exp_str_gamma = "[ [ \"06:08:10.000012\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("gamma")
+                                   ->getDuration().second) + "\" ]" + gamma_first;
+    std::string exp_str_delta = "[ [ \"Lorem ipsum\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("delta")
+                                   ->getString().second) + "\" ]" + delta_first;
 
     // Check that individual stats are reported properly
     EXPECT_EQ("{ \"alpha\": " + exp_str_alpha + " }", rep_alpha->str());
@@ -203,21 +301,21 @@ TEST_F(StatsMgrTest, getGetAll) {
 
 // This test checks whether existing statistics can be reset.
 TEST_F(StatsMgrTest, reset) {
-
     // Set a couple of statistics
     StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
     StatsMgr::instance().setValue("beta", 12.34);
-    StatsMgr::instance().setValue("gamma", time_duration(1,2,3,4));
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
     StatsMgr::instance().setValue("delta", "Lorem ipsum");
 
     // This should reset alpha to 0
     EXPECT_NO_THROW(StatsMgr::instance().reset("alpha"));
-    EXPECT_EQ(0, StatsMgr::instance().getObservation("alpha")->getInteger().first);
+    EXPECT_EQ(0,
+              StatsMgr::instance().getObservation("alpha")->getInteger().first);
 
     // The other stats should remain untouched
     EXPECT_EQ(12.34,
               StatsMgr::instance().getObservation("beta")->getFloat().first);
-    EXPECT_EQ(time_duration(1,2,3,4),
+    EXPECT_EQ(time_duration(1, 2, 3, 4),
               StatsMgr::instance().getObservation("gamma")->getDuration().first);
     EXPECT_EQ("Lorem ipsum",
               StatsMgr::instance().getObservation("delta")->getString().first);
@@ -228,7 +326,7 @@ TEST_F(StatsMgrTest, reset) {
     EXPECT_NO_THROW(StatsMgr::instance().reset("delta"));
     EXPECT_EQ(0.0,
               StatsMgr::instance().getObservation("beta")->getFloat().first);
-    EXPECT_EQ(time_duration(0,0,0,0),
+    EXPECT_EQ(time_duration(0, 0, 0, 0),
               StatsMgr::instance().getObservation("gamma")->getDuration().first);
     EXPECT_EQ("",
               StatsMgr::instance().getObservation("delta")->getString().first);
@@ -239,19 +337,19 @@ TEST_F(StatsMgrTest, reset) {
 
 // This test checks whether existing statistics can be reset.
 TEST_F(StatsMgrTest, resetAll) {
-
     // Set a couple of statistics
     StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
     StatsMgr::instance().setValue("beta", 12.34);
-    StatsMgr::instance().setValue("gamma", time_duration(1,2,3,4));
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
     StatsMgr::instance().setValue("delta", "Lorem ipsum");
 
     // This should reset alpha to 0
     EXPECT_NO_THROW(StatsMgr::instance().resetAll());
-    EXPECT_EQ(0, StatsMgr::instance().getObservation("alpha")->getInteger().first);
+    EXPECT_EQ(0,
+              StatsMgr::instance().getObservation("alpha")->getInteger().first);
     EXPECT_EQ(0.0,
               StatsMgr::instance().getObservation("beta")->getFloat().first);
-    EXPECT_EQ(time_duration(0,0,0,0),
+    EXPECT_EQ(time_duration(0, 0, 0, 0),
               StatsMgr::instance().getObservation("gamma")->getDuration().first);
     EXPECT_EQ("",
               StatsMgr::instance().getObservation("delta")->getString().first);
@@ -262,11 +360,10 @@ TEST_F(StatsMgrTest, resetAll) {
 
 // This test checks whether statistics can be removed.
 TEST_F(StatsMgrTest, removeAll) {
-
     // Set a couple of statistics
     StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
     StatsMgr::instance().setValue("beta", 12.34);
-    StatsMgr::instance().setValue("gamma", time_duration(1,2,3,4));
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
     StatsMgr::instance().setValue("delta", "Lorem ipsum");
 
     // This should reset alpha to 0
@@ -300,7 +397,7 @@ TEST_F(StatsMgrTest, DISABLED_performanceSingleAdd) {
 
     ptime before = microsec_clock::local_time();
     for (uint32_t i = 0; i < cycles; ++i) {
-        StatsMgr::instance().addValue("metric1", 0.1*i);
+        StatsMgr::instance().addValue("metric1", 0.1 * i);
     }
     ptime after = microsec_clock::local_time();
 
@@ -322,7 +419,7 @@ TEST_F(StatsMgrTest, DISABLED_performanceSingleSet) {
 
     ptime before = microsec_clock::local_time();
     for (uint32_t i = 0; i < cycles; ++i) {
-        StatsMgr::instance().setValue("metric1", 0.1*i);
+        StatsMgr::instance().setValue("metric1", 0.1 * i);
     }
     ptime after = microsec_clock::local_time();
 
@@ -422,8 +519,8 @@ TEST_F(StatsMgrTest, commandStatisticGet) {
     EXPECT_NO_THROW(alpha = StatsMgr::instance().getObservation("alpha"));
     ASSERT_TRUE(alpha);
 
-    std::string exp = "{ \"alpha\": [ [ 1234, \""
-        + isc::util::ptimeToText(alpha->getInteger().second) + "\" ] ] }";
+    std::string exp = "{ \"alpha\": [ [ 1234, \"" +
+        isc::util::ptimeToText(alpha->getInteger().second) + "\" ] ] }";
 
     EXPECT_EQ("{ \"arguments\": " + exp + ", \"result\": 0 }", rsp->str());
 }
@@ -433,7 +530,6 @@ TEST_F(StatsMgrTest, commandStatisticGet) {
 // - a request with missing statistic name
 // - a request for non-existing statistic.
 TEST_F(StatsMgrTest, commandStatisticGetNegative) {
-
     // Case 1: a request without parameters
     ConstElementPtr rsp = StatsMgr::instance().statisticGetHandler("statistic-get",
                                                                    ElementPtr());
@@ -453,14 +549,13 @@ TEST_F(StatsMgrTest, commandStatisticGetNegative) {
     EXPECT_EQ("{ \"arguments\": {  }, \"result\": 0 }", rsp->str());
 }
 
-// This test checks whether statistc-get-all command returns all statistics
+// This test checks whether statistic-get-all command returns all statistics
 // correctly.
 TEST_F(StatsMgrTest, commandGetAll) {
-
     // Set a couple of statistics
     StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
     StatsMgr::instance().setValue("beta", 12.34);
-    StatsMgr::instance().setValue("gamma", time_duration(1,2,3,4));
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
     StatsMgr::instance().setValue("delta", "Lorem ipsum");
 
     // Now get them. They're used to generate expected output
@@ -474,17 +569,17 @@ TEST_F(StatsMgrTest, commandGetAll) {
     ASSERT_TRUE(rep_gamma);
     ASSERT_TRUE(rep_delta);
 
-    std::string exp_str_alpha = "[ [ 1234, \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("alpha")
+    std::string exp_str_alpha = "[ [ 1234, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("alpha")
                                    ->getInteger().second) + "\" ] ]";
-    std::string exp_str_beta = "[ [ 12.34, \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("beta")
+    std::string exp_str_beta = "[ [ 12.34, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("beta")
                                    ->getFloat().second) + "\" ] ]";
-    std::string exp_str_gamma = "[ [ \"01:02:03.000004\", \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("gamma")
+    std::string exp_str_gamma = "[ [ \"01:02:03.000004\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("gamma")
                                    ->getDuration().second) + "\" ] ]";
-    std::string exp_str_delta = "[ [ \"Lorem ipsum\", \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("delta")
+    std::string exp_str_delta = "[ [ \"Lorem ipsum\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("delta")
                                    ->getString().second) + "\" ] ]";
 
     // Check that all of them can be reported at once
@@ -537,7 +632,6 @@ TEST_F(StatsMgrTest, commandStatisticReset) {
 // - a request with missing statistic name
 // - a request for non-existing statistic.
 TEST_F(StatsMgrTest, commandStatisticResetNegative) {
-
     // Case 1: a request without parameters
     ConstElementPtr rsp =
         StatsMgr::instance().statisticResetHandler("statistic-reset", ElementPtr());
@@ -561,11 +655,10 @@ TEST_F(StatsMgrTest, commandStatisticResetNegative) {
 // This test checks whether statistic-reset-all command really resets all
 // statistics correctly.
 TEST_F(StatsMgrTest, commandResetAll) {
-
     // Set a couple of statistics
     StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
     StatsMgr::instance().setValue("beta", 12.34);
-    StatsMgr::instance().setValue("gamma", time_duration(1,2,3,4));
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
     StatsMgr::instance().setValue("delta", "Lorem ipsum");
 
     // Now get them. They're used to generate expected output
@@ -579,17 +672,17 @@ TEST_F(StatsMgrTest, commandResetAll) {
     ASSERT_TRUE(rep_gamma);
     ASSERT_TRUE(rep_delta);
 
-    std::string exp_str_alpha = "[ [ 1234, \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("alpha")
+    std::string exp_str_alpha = "[ [ 1234, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("alpha")
                                    ->getInteger().second) + "\" ] ]";
-    std::string exp_str_beta = "[ [ 12.34, \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("beta")
+    std::string exp_str_beta = "[ [ 12.34, \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("beta")
                                    ->getFloat().second) + "\" ] ]";
-    std::string exp_str_gamma = "[ [ \"01:02:03.000004\", \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("gamma")
+    std::string exp_str_gamma = "[ [ \"01:02:03.000004\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("gamma")
                                    ->getDuration().second) + "\" ] ]";
-    std::string exp_str_delta = "[ [ \"Lorem ipsum\", \""
-        + isc::util::ptimeToText(StatsMgr::instance().getObservation("delta")
+    std::string exp_str_delta = "[ [ \"Lorem ipsum\", \"" +
+        isc::util::ptimeToText(StatsMgr::instance().getObservation("delta")
                                    ->getString().second) + "\" ] ]";
 
     // Check that all of them can be reset at once
@@ -602,10 +695,11 @@ TEST_F(StatsMgrTest, commandResetAll) {
     ASSERT_TRUE(rep_all);
 
     // Check that they're indeed reset
-    EXPECT_EQ(0, StatsMgr::instance().getObservation("alpha")->getInteger().first);
+    EXPECT_EQ(0,
+              StatsMgr::instance().getObservation("alpha")->getInteger().first);
     EXPECT_EQ(0.0f,
               StatsMgr::instance().getObservation("beta")->getFloat().first);
-    EXPECT_EQ(time_duration(0,0,0,0),
+    EXPECT_EQ(time_duration(0, 0, 0, 0),
               StatsMgr::instance().getObservation("gamma")->getDuration().first);
     EXPECT_EQ("",
               StatsMgr::instance().getObservation("delta")->getString().first);
@@ -626,6 +720,7 @@ TEST_F(StatsMgrTest, commandStatisticRemove) {
 
     // It should be gone.
     EXPECT_FALSE(StatsMgr::instance().getObservation("alpha"));
+    EXPECT_EQ(0, StatsMgr::instance().count());
 }
 
 // Test checks if statistic-remove is able to handle:
@@ -633,7 +728,6 @@ TEST_F(StatsMgrTest, commandStatisticRemove) {
 // - a request with missing statistic name
 // - a request for non-existing statistic.
 TEST_F(StatsMgrTest, commandStatisticRemoveNegative) {
-
     // Case 1: a request without parameters
     ConstElementPtr rsp =
         StatsMgr::instance().statisticRemoveHandler("statistic-remove", ElementPtr());
@@ -657,11 +751,10 @@ TEST_F(StatsMgrTest, commandStatisticRemoveNegative) {
 // This test checks whether statistic-remove-all command really resets all
 // statistics correctly.
 TEST_F(StatsMgrTest, commandRemoveAll) {
-
     // Set a couple of statistics
     StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
     StatsMgr::instance().setValue("beta", 12.34);
-    StatsMgr::instance().setValue("gamma", time_duration(1,2,3,4));
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
     StatsMgr::instance().setValue("delta", "Lorem ipsum");
 
     // Check that all of them can be reset at once
@@ -676,6 +769,190 @@ TEST_F(StatsMgrTest, commandRemoveAll) {
     EXPECT_FALSE(StatsMgr::instance().getObservation("beta"));
     EXPECT_FALSE(StatsMgr::instance().getObservation("gamma"));
     EXPECT_FALSE(StatsMgr::instance().getObservation("delta"));
+    EXPECT_EQ(0, StatsMgr::instance().count());
+}
+
+// This test checks whether statistic-sample-age-set command really set
+// max_sample_age_ limit correctly.
+TEST_F(StatsMgrTest, commandSetMaxSampleAge) {
+    StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
+
+    ElementPtr params = Element::createMap();
+    params->set("name", Element::create("alpha"));
+    params->set("duration", Element::create(1245)); // time_duration(0, 20, 45, 0)
+
+    ConstElementPtr rsp =
+        StatsMgr::instance().statisticSetMaxSampleAgeHandler("statistic-sample-age-set", params);
+    int status_code;
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status_code);
+
+    // check if time limit was set properly and whether count limit is disabled
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().second,
+              time_duration(0, 20, 45, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().first, false);
+}
+
+// Test checks if statistic-sample-age-set is able to handle:
+// - a request without parameters
+// - a request without duration parameter
+// - a request with missing statistic name
+// - a request for non-existing statistic.
+TEST_F(StatsMgrTest, commandSetMaxSampleAgeNegative) {
+    // Case 1: a request without parameters
+    ConstElementPtr rsp =
+        StatsMgr::instance().statisticSetMaxSampleAgeHandler("statistic-sample-age-set", ElementPtr());
+    int status_code;
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(status_code, CONTROL_RESULT_ERROR);
+
+    // Case 2: a request without duration parameter
+    ElementPtr params = Element::createMap();
+    params->set("name", Element::create("alpha"));
+    rsp = StatsMgr::instance().statisticSetMaxSampleAgeHandler("statistic-sample-age-set", params);
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(status_code, CONTROL_RESULT_ERROR);
+
+    // Case 3: a request with missing statistic name
+    params = Element::createMap();
+    params->set("duration", Element::create(100));
+    rsp = StatsMgr::instance().statisticSetMaxSampleAgeHandler("statistic-sample-age-set", params);
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(status_code, CONTROL_RESULT_ERROR);
+
+    // Case 4: a request for non-existing statistic
+    params->set("name", Element::create("alpha"));
+    rsp = StatsMgr::instance().statisticSetMaxSampleAgeHandler("statistic-sample-age-set", params);
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"No 'alpha' statistic found\" }",
+              rsp->str());
+}
+
+TEST_F(StatsMgrTest, commandSetMaxSampleAgeAll) {
+    // Set a couple of statistics
+    StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
+    StatsMgr::instance().setValue("beta", 12.34);
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
+    StatsMgr::instance().setValue("delta", "Lorem ipsum");
+
+    ElementPtr params = Element::createMap();
+    params->set("duration", Element::create(3765)); // time_duration(1, 2, 45, 0)
+
+    ConstElementPtr rsp =
+        StatsMgr::instance().statisticSetMaxSampleAgeAllHandler("statistic-sample-age-set-all", params);
+    int status_code;
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status_code);
+
+    // check if time limit was set properly and whether count limit is disabled
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().second,
+              time_duration(1, 2, 45, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleAge().second,
+              time_duration(1, 2, 45, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleCount().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleAge().second,
+              time_duration(1, 2, 45, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleCount().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleAge().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleAge().second,
+              time_duration(1, 2, 45, 0));
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleCount().first, false);
+}
+
+// This test checks whether statistic-sample-count-set command really set
+// max_sample_count_ limit correctly.
+TEST_F(StatsMgrTest, commandSetMaxSampleCount) {
+    StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
+
+    ElementPtr params = Element::createMap();
+    params->set("name", Element::create("alpha"));
+    params->set("max-samples", Element::create(15));
+
+    ConstElementPtr rsp =
+        StatsMgr::instance().statisticSetMaxSampleCountHandler("statistic-sample-count-set", params);
+    int status_code;
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status_code);
+
+    // check if time limit was set properly and whether duration limit is disabled
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().second, 15);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().first, false);
+}
+
+// Test checks if statistic-sample-count-set is able to handle:
+// - a request without parameters
+// - a request without max-samples parameter
+// - a request with missing statistic name
+// - a request for non-existing statistic.
+TEST_F(StatsMgrTest, commandSetMaxSampleCountNegative) {
+    // Case 1: a request without parameters
+    ConstElementPtr rsp =
+        StatsMgr::instance().statisticSetMaxSampleCountHandler("statistic-sample-count-set", ElementPtr());
+    int status_code;
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(status_code, CONTROL_RESULT_ERROR);
+
+    // Case 2: a request without max-samples parameter
+    ElementPtr params = Element::createMap();
+    params->set("name", Element::create("alpha"));
+    rsp = StatsMgr::instance().statisticSetMaxSampleCountHandler("statistic-sample-count-set", params);
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(status_code, CONTROL_RESULT_ERROR);
+
+    // Case 3: a request with missing statistic name
+    params = Element::createMap();
+    params->set("max-samples", Element::create(10));
+    rsp = StatsMgr::instance().statisticSetMaxSampleCountHandler("statistic-sample-count-set", params);
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(status_code, CONTROL_RESULT_ERROR);
+
+    // Case 4: a request for non-existing statistic
+    params->set("name", Element::create("alpha"));
+    rsp = StatsMgr::instance().statisticSetMaxSampleCountHandler("statistic-sample-count-set", params);
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"No 'alpha' statistic found\" }",
+              rsp->str());
+}
+
+TEST_F(StatsMgrTest, commandSetMaxSampleCountAll) {
+    // Set a couple of statistics
+    StatsMgr::instance().setValue("alpha", static_cast<int64_t>(1234));
+    StatsMgr::instance().setValue("beta", 12.34);
+    StatsMgr::instance().setValue("gamma", time_duration(1, 2, 3, 4));
+    StatsMgr::instance().setValue("delta", "Lorem ipsum");
+
+    ElementPtr params = Element::createMap();
+    params->set("max-samples", Element::create(200));
+
+    ConstElementPtr rsp =
+        StatsMgr::instance().statisticSetMaxSampleCountAllHandler("statistic-sample-count-set-all", params);
+    int status_code;
+    ASSERT_NO_THROW(parseAnswer(status_code, rsp));
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status_code);
+
+    // check if count limit was set properly and whether count limit is disabled
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleCount().second, 200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("alpha")->getMaxSampleAge().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleCount().second, 200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("beta")->getMaxSampleAge().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleCount().second, 200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("gamma")->getMaxSampleAge().first, false);
+
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleCount().first, true);
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleCount().second, 200);
+    EXPECT_EQ(StatsMgr::instance().getObservation("delta")->getMaxSampleAge().first, false);
 }
 
 };
